(() => {
    'use strict';

    angular.module('noc').factory('BaseCreateVolume', (usageProfiles, nocNormalizerService, nocCurrentUserService) => {
        return function () {
            const scope = this;

            scope.exportPolices = [
                {
                    name: 'No access to the volume',
                    value: 'none'
                },
                {
                    name: 'Custom export policy',
                    value: 'custom'
                }
            ];
            scope.exportPolicyWrapper = {exportPolicyModel: scope.exportPolices[1]};

            scope.selectDefaultExportPolicy = (policy) => {
                //Sanity:
                if (policy !== 'none' && policy !== 'custom') {
                    throw new Error(`Unknown policy ${policy}`);
                }
                scope.exportPolicyWrapper.exportPolicyModel = _.find(scope.exportPolices, {value: policy});
            };

            scope.initialized = false;

            scope.volumeProtocol = {type: "NFS"};

            scope.usageProfiles = usageProfiles.list;

            scope.getSnapshotPolicyByName = function getSnapshotPolicyByName(name) {
                return _.find(scope.snapshotPolicies, {name});
            };

            scope.selectADefaultSnapshotPolicy = function () {
                if (_.isEmpty(scope.snapshotPolicies)) return;
                var defaultPolicy = scope.getSnapshotPolicyByName("default");
                scope.volumeRequest.snapshotPolicyName = scope.volumeRequest.snapshotPolicyName || (defaultPolicy ? defaultPolicy.name : scope.snapshotPolicies[0].name);
            };

            scope.fetchSnapshotPolicy = function fetchSnapshotPolicy(workingEnvironment, pathPrefix, nocHttp) {
                var url = pathPrefix + "metadata/default-snapshot-policies";
                var params = {};
                if (workingEnvironment) {
                    url = workingEnvironment._pathPrefix + "working-environments/" + workingEnvironment.publicId;
                    params.fields = "snapshotPolicies";
                }

                return nocHttp.get({
                        url: url,
                        params: params
                    },
                    "retrieving Snapshot policies" + (workingEnvironment ? " for working environment " + workingEnvironment.name : ""),
                    nocHttp.BLOCK)
                    .then(function(response) {
                        var snapshotPolicies = response.snapshotPolicies || response;

                        //NOC-3382 hide default-weekly snapshot policy
                        _.remove(snapshotPolicies, ['name','default-1weekly']);

                        scope.snapshotPolicies = snapshotPolicies;
                        scope.selectADefaultSnapshotPolicy();
                        return snapshotPolicies;
                    });
            };

            // If volumeProtocolType is empty then it should be considered NFS.
            scope.generateRequestBody = (quoteResponse, volumeProtocolType) => {
                scope.volumeRequest.verifyNameUniqueness = true;
                const body = _.cloneDeep(scope.volumeRequest);

                if (volumeProtocolType === 'CIFS') {
                    delete body.exportPolicyInfo;
                    body.shareInfo.accessControl.users = nocNormalizerService.normalizeUsersGroups(body.shareInfo.accessControl._users);
                } else {
                    if (body.exportPolicyInfo.policyType === 'none') {
                        body.exportPolicyInfo.ips = [];
                        body.exportPolicyInfo._ips = '';
                    } else {
                        body.exportPolicyInfo.ips = nocNormalizerService.normalizeCustomAccessControl(body.exportPolicyInfo._ips);
                    }
                    delete body.shareInfo;
                }

                if (body.sizeGb && !body.size) {
                    body.size = {size: body.sizeGb, unit: 'GB'};
                    delete body.sizeGb;
                }

                //Replace the usageProfiles with EfficiencyFlags
                if (body.usageProfile) {
                    _.assignIn(body, body.usageProfile.flags);
                    delete body.usageProfile;
                }

                //Not needed for the first request (for the quote)
                if (quoteResponse) {
                    body.maxNumOfDisksApprovedToAdd = quoteResponse.numOfDisks;

                    //This may be redundant when adding volume but that's ok
                    body.aggregateName = quoteResponse.aggregateName;
                }

                return body;
            };

            scope.initSyncToS3Status = function initSyncToS3Status() {
                scope.syncToS3Status = nocCurrentUserService.getCurrentUserCloudSyncLicenseExpired() ? 'EXPIRED' : scope.stateParamSyncToS3Status;
                if (scope.syncToS3Status === 'ENABLED') {
                    scope.volumeRequest.syncToS3 = true;
                }
            };

            /* istanbul ignore next: no logic */
            scope.getSyncToS3DisabledMessage = function getSyncToS3DisabledMessage() {
                switch (scope.syncToS3Status) {
                    case 'DISABLED':
                        return 'Volume cannot be synced to S3, Data Broker is not available';
                    case 'CIFS':
                        return 'CIFS Volume cannot be synced to S3';
                    case 'NO_ACCESS':
                        return 'Volume with no access cannot be synced to S3';
                    case 'EXPIRED':
                        return 'Cloud sync license expired, cannot perform Sync actions.';
                    default:
                        return '';
                }
            };

            scope.policyTypeChanged = function policyTypeChanged() {
                scope.volumeRequest.exportPolicyInfo.policyType = scope.exportPolicyWrapper.exportPolicyModel.value;

                if (!scope.syncToS3Status || scope.syncToS3Status === 'NOT_AVAILABLE' || scope.syncToS3Status === 'EXPIRED') return;

                if (scope.volumeRequest.exportPolicyInfo.policyType === 'none') {
                    scope.syncToS3Status = 'NO_ACCESS';
                    scope.volumeRequest.syncToS3 = false;
                } else {
                    scope.initSyncToS3Status();
                }
            };
        };
    });
})();
