(() => {
    'use strict';

    angular.module('noc').controller('CloudSyncActivationDataBrokerStepController', ($scope, $state, $http, $q, BaseVirginStep,
                                                                                     nocHttp, nocCloudSyncService, $rootScope,
                                                                                     pidgeon, MetadataResource, nocUI) => {
        //To get the list of regions or VPCs, we can GET either /vsa/metadata/... or /aws/ha/metadata/...
        //It doesn't matter, both of them go tho the same code in the server, so we arbitrarily pick one of them:
        const pathPrefixForMetadata = '/vsa/';

        $scope.getKeyPairs = function getKeyPairs() {
            if ($scope.wizardController.type === 'VSA') {
                return nocCloudSyncService.getKetPairsByWorkingEnvironment(nocHttp, $scope.wizardController.workingEnvironmentId)
                    .then((keyPairs) => {
                        $scope.keyPairs = keyPairs;
                    });
            } else {
                return getKeyPairsByRegion();
            }
        };

        function getKeyPairsByRegion() {
            return MetadataResource.getKeyPairs(nocHttp, pidgeon, $scope.wizardController.selectedRegion.code, true,
                $scope.wizardController.cloudProviderAccountId)
                .then((keyPairs) => {
                    $scope.keyPairs = keyPairs;
                });
        }

        $scope.getRegions = function getRegions() {
            return MetadataResource.getRegions(nocHttp, pathPrefixForMetadata)
                .then((regions) => {
                    $scope.regions = regions;
                });
        };

        $scope.getVPCs = function getVPCs() {
            const unloadVpcs = nocUI.loading();

            //In case the user changes the region often, and this function is called time after time,
            //we don't want that newer VPCs response would be overridden by old ones. So we cancel the old request
            //and create new one.
            if ($scope.lastVPCConfig) {
                $scope.lastVPCConfig._.cancelled = true;
            }

            $scope.lastVPCConfig = $scope.wizardController.type !== 'VSA' ? {
                method: 'GET',
                url: pathPrefixForMetadata + 'metadata/vpcs' + ($scope.wizardController.cloudProviderAccountId ? '?cloudProviderAccountId=' + $scope.wizardController.cloudProviderAccountId : ''),
                params: {region: $scope.wizardController.selectedRegion.code},
                _: {}
            } : {
                method: 'GET',
                url: 'cloudsync/vpcs-by-vsa/' + $scope.wizardController.workingEnvironmentId,
                params: {region: $scope.wizardController.selectedRegion.code},
                _: {}
            };

            pidgeon.clear('vpcsWithSubnets');

            return $http($scope.lastVPCConfig)
                .then((response) => {
                    if (response.config._.cancelled) return;

                    $scope.vpcsWithSubnets = response.data;

                    _.forEach($scope.vpcsWithSubnets, (vpc) => {
                        const nameTag = _.find(vpc.tags, {key: 'Name'});
                        vpc._name = nameTag ? nameTag.value : null;
                    });
                })
                .catch((errorResponse) => {
                    if (errorResponse.config._.cancelled) return;

                    pidgeon.error(pidgeon.serverErrorPrefix + 'retrieving list of VPCs with subnets', errorResponse.data, 'vpcsWithSubnets');
                })
                .finally(unloadVpcs);
        };

        $scope.handleRegionChange = (newRegion) => {
            if (!newRegion) return $q.reject();

            $scope.wizardController.selectedRegion = newRegion;

            $scope.wizardController.selectedVpc = null;
            $scope.wizardController.selectedSubnet = null;
            $scope.wizardController.keyPair = null;

            return $scope.getVPCs()
                .then($scope.getKeyPairs);
        };

        $scope.initialize = function initialize() {
            BaseVirginStep.call($scope);
            $scope.getRegions()
                .then(() => {
                    $scope.instancePlacement = $rootScope.instancePlacement;

                    const region = $scope.wizardController.region || $scope.instancePlacement.region;

                    $scope.handleRegionChange(region ? _.find($scope.regions, {code: region}) : $scope.regions[0])
                        .then(() => {
                            const selectedVpcId = $scope.wizardController.vpcId || $scope.instancePlacement.vpcId;
                            if (selectedVpcId) {
                                $scope.wizardController.selectedVpc = _.find($scope.vpcsWithSubnets, {vpcId: selectedVpcId});
                                const defaultSubnetId = $scope.wizardController.subnetId || $scope.instancePlacement.subnetId;
                                $scope.wizardController.selectedSubnet = _.find($scope.wizardController.selectedVpc.subnets, {subnetId: defaultSubnetId});
                            } else if ($scope.vpcsWithSubnets.length > 0) {
                                $scope.wizardController.selectedVpc = $scope.vpcsWithSubnets[0];
                            }

                            $scope.wizardController.keyPair = $scope.wizardController.keyPairName || $scope.instancePlacement.keyPair;
                        });
                });
        };

        $scope.finish = function finish() {
            if ($scope.isValid()) {
                nocCloudSyncService.sync(nocHttp, $scope.wizardController.workingEnvironmentId, {
                    volumeNames: $scope.wizardController.selectedVolumes,
                    s3Location: 's3://' + $scope.wizardController.s3Location,
                    dataBrokerNetworkInformation: {
                        region: $scope.wizardController.selectedRegion.code,
                        vpcId: $scope.wizardController.selectedVpc.vpcId,
                        subnetId: $scope.wizardController.selectedSubnet.subnetId,
                        keyPair: $scope.wizardController.keyPair
                    }
                }, $state.params.type)
                    .then(() => {
                        $scope.wizardController.wizardCompletedSucssesfully = true;
                        $scope.wizardController.exit();
                    });
            } else {
                nocUI.handleErrorsInWizard(true); //Parameter 'true' makes the screen scroll to the first error.
            }
        };

        $scope.initialize();
    });
})();
