(() => {
    'use strict';

    const strong = (str) => {
        return '<strong>' + str + '</strong>';
    };

    angular.module('noc').filter('actionIntoWords', (nocUtil, $filter, $rootScope, amTimeAgoFilter) => {
        const normalizeCapacityFilter = $filter('normalizedCapacity');

        const createCloudSyncLicensingMessage = function createCloudSyncLicensingMessage(actionType, licenseExpiryDate, serialNumber) {
            const verb = _.endsWith(actionType, 'Expired') ? 'expired' : 'expires';
            const url = $rootScope.cloudSyncUrl.url + '/license';
            const suffix = 'see your options in <a href="' + url + '" target="_blank">Cloud Sync License Settings</a>.';
            const serialNumberString = serialNumber ? ('(serial number ' + serialNumber + ') ') : '';

            if (_.includes(actionType, 'Trial')) {
                return 'Your Cloud Sync service free trial period ' + verb + ' ' + amTimeAgoFilter(licenseExpiryDate) + '.<br>To continue using this service after the 14-day free trial ends, ' + suffix;
            } else {
                return 'Your Cloud Sync service BYOL license ' + serialNumberString + verb + ' ' + amTimeAgoFilter(licenseExpiryDate) + '.<br> To continue using this service, ' + suffix;
            }
        };

        return (action, workingEnvironment) => {
            const actionType = action.actionType;
            action.parameters.diskSize = nocUtil.normalizeCapacity(action.parameters.diskSize);
            const capacity = {
                size: action.parameters.diskSize.size * action.parameters.numOfDisks,
                unit: action.parameters.diskSize.unit
            };

            if (actionType === 'NeedsMoreDisks') {
                return 'An additional ' + capacity.size + ' ' + capacity.unit + ' of ' + workingEnvironment._cloudProviderName + ' capacity is required for aggregate ' + strong(action.parameters.aggregateName) + '.';
            }
            if (actionType === 'MoreDisksNeededButDiskLimitExceeded') {
                const volumeMoveParameters = action.parameters.volumeMoveParameters;
                const volMoveMessagePrefix = 'Moving a volume is necessary to avoid capacity issues';
                if (volumeMoveParameters) {
                    if (volumeMoveParameters.isNewTargetAggregate || action.parameters.numOfDisks) {
                        return volMoveMessagePrefix + '. An additional ' + capacity.size + ' ' + capacity.unit + ' of ' + workingEnvironment._cloudProviderName + ' capacity is required';
                    } else return volMoveMessagePrefix;
                } else {
                    if (action.parameters.moreInfo === 'INSTANCE_DISK_LIMIT_EXCEEDED') {
                        return volMoveMessagePrefix + 'Moving a volume is necessary to avoid capacity issues; however, Cloud Manager cannot perform this action for you because the system has reached the disk limit.';
                    } else if (action.parameters.moreInfo === 'NO_SINGLE_VOLUME') {
                        return 'Moving one or more volumes is necessary to avoid capacity issues; however, Cloud Manager cannot perform this action for you.';
                    } else {
                        //Sanity, should never happen
                        return volMoveMessagePrefix;
                    }
                }
            }
            if (actionType === 'MoreDisksNeededButVsaLimitExceeded') {
                const licenseParams = action.parameters.licenseParameters;
                const prefix = 'An additional ' + workingEnvironment._cloudProviderName + ' capacity is required for aggregate ' + strong(action.parameters.aggregateName);
                if (licenseParams) {
                    return prefix + '. Cloud Manager must change the Cloud Volumes ONTAP license to meet the additional capacity needs';
                } else {
                    return prefix + ', however Cloud Volumes ONTAP has reached its max capacity';
                }
            }
            if (actionType === 'LicenseUsedCapacityWarning') {
                const LicenseUsedCapacityWarningNote = '<br>Note: Deleting a volume does not have an immediate effect on used capacity. The capacity amount includes deleted volumes that remain in ONTAP’s recovery queue for 12 hours.';
                if (action.parameters.licenseParameters) {
                    return 'Used capacity for this Cloud Volumes ONTAP system has reached 90% of the license limit. To add additional capacity, Cloud Manager must change the Cloud Volumes ONTAP license.' + LicenseUsedCapacityWarningNote;
                } else {
                    return 'Used capacity for this Cloud Volumes ONTAP system has reached 90% of the license limit.' + LicenseUsedCapacityWarningNote;
                }
            }
            if (actionType === 'LicenseAllocatedCapacityError') {
                const LicenseUsedCapacityWarningNote = "<br>Note: that a system's used capacity includes the size of attached disks and the used capacity of object storage.";
                if (action.parameters.licenseParameters) {
                    return 'The used capacity for this system has exceeded the license limit. To add additional capacity, Cloud Manager must upgrade the license for Cloud Volumes ONTAP.' + LicenseUsedCapacityWarningNote;
                } else {
                    return 'The used capacity for this system has exceeded the license limit.' + LicenseUsedCapacityWarningNote;
                }
            }
            if (actionType === 'NonCompliantConfig') {
                const features = [];
                if (_.includes(action.parameters.moreInfo, 'SIDL')) features.push('The single instance data logging feature');
                if (_.includes(action.parameters.moreInfo, 'CAPACITY_TIERING')) features.push('A tiered storage configuration');
                if (features.length > 1) {
                    return 'The following features were enabled directly from Cloud Volumes ONTAP: ' + _.capitalize(features.join(' and ')) + '. These features are not supported with the current instance type. To use these features, change the instance type.';
                } else {
                    return features + ' was enabled directly from Cloud Volumes ONTAP. This feature is not supported with the current instance type. To use this feature, change the instance type.';
                }
            }

            if (actionType === 'ShouldBeRemoved') {
                return 'Saving opportunity identified: Cloud Volumes ONTAP has ' + normalizeCapacityFilter(capacity, 2, true) + ' of ' + workingEnvironment._cloudProviderName + ' capacity that is not being utilized and can be removed.';
            }

            if (actionType === 'MaxDisksLimitAboutToExceed') {
                if (action.parameters.numOfDisks === parseInt(action.parameters.moreInfo)) {
                    return (workingEnvironment.isHA ? 'This HA pair ' : 'This system ') + 'has reached the maximum number of allowed data disks.';
                }

                return (workingEnvironment.isHA ? 'This HA pair ' : 'This system ') + 'is about to reach the maximum number of allowed data disks.';
            }

            if (actionType === 'MaxDisksLimitExceeded') {
                return (workingEnvironment.isHA ? 'This HA pair ' : 'This system ') + 'is exceeding the maximum number of allowed data disks.';
            }

            if (actionType === 'deprecatedInstance') {
                return 'An EC2 instance type change is recommended';
            }

            if (actionType === 'PaygoNotRegisteredToSupport') {
                return 'This system has not been registered with NetApp support.';
            }

            const fullDateFilter = $filter('fullDate');
            switch (actionType) {
                case 'EvaluationByol': {
                    const evalExpiration = moment(workingEnvironment.ontapClusterProperties.creationTime).add(31, 'days');
                    let messageToReturn;
                    if (evalExpiration.diff(moment(), 'days') <= 0) {
                        messageToReturn = 'Your BYOL evaluation license has been suspended for working environment ' + workingEnvironment.name + ' because your subscription expired on ' + fullDateFilter(action.parameters.licenseExpiryDate);
                    } else {
                        messageToReturn = 'Your BYOL evaluation license expires on ' + fullDateFilter(evalExpiration) + ' (in ' + evalExpiration.diff(moment(), 'days') + ' days).';
                    }

                    return messageToReturn + '<br>To continue using this working environment update the serial number' + (workingEnvironment.isHA ? 's.' : '.');
                }
                case 'licenseWasSuspended':
                    return 'Your BYOL instance has been suspended for working environment ' + workingEnvironment.name + ' because your subscription expired on ' + fullDateFilter(action.parameters.licenseExpiryDate) + '. <br>Renew your subscription in order to regain access to your BYOL instance.';
                case 'licenseGracePeriod':
                    return 'Your BYOL subscription expired on ' + fullDateFilter(action.parameters.licenseExpiryDate) + ' for working environment ' + workingEnvironment.name + '. You are now in violation of the terms of your subscription. Renew your subscription immediately to prevent suspension of your BYOL instance.';
                case 'licenseExpirationWarning':
                    return 'Your BYOL subscription is set to expire on ' + fullDateFilter(action.parameters.licenseExpiryDate) + ' for working environment ' + workingEnvironment.name + '. Renew your subscription prior to ' + fullDateFilter(action.parameters.licenseExpiryDate) + ' to prevent suspension of your BYOL instance.';
                case 'RestrictedAggregatesCanBeFixed':
                    return 'Some of your data on working environment ' + workingEnvironment.name + ' is unavailable because Cloud Volumes ONTAP could not connect to key managers. A connection to at least one key manager is now available. Restart Cloud Volumes ONTAP so it can connect to key managers to unlock the data.';
                case 'RestrictedAggregatesCannotBeFixed':
                    return 'Some of your data on working environment ' + workingEnvironment.name + ' is unavailable because Cloud Volumes ONTAP could not connect to key managers. Make sure your key managers are available, and then restart Cloud Volumes ONTAP so it can connect to key managers to unlock the data.';
                case 'EmptyWorkingEnvironment':
                    return 'Cloud Volumes ONTAP is not being utilized and can be deleted';
                case 'CloudSyncTrialReminder':
                    return createCloudSyncLicensingMessage(actionType, action.parameters.licenseExpiryDate);
                case 'CloudSyncTrialWarning':
                    return createCloudSyncLicensingMessage(actionType, action.parameters.licenseExpiryDate);
                case 'CloudSyncTrialExpired':
                    return createCloudSyncLicensingMessage(actionType, action.parameters.licenseExpiryDate);
                case 'CloudSyncLicenseReminder':
                    return createCloudSyncLicensingMessage(actionType, action.parameters.licenseExpiryDate, action.parameters.serialNumber);
                case 'CloudSyncLicenseWarning':
                    return createCloudSyncLicensingMessage(actionType, action.parameters.licenseExpiryDate, action.parameters.serialNumber);
                case 'CloudSyncLicenseExpired':
                    return createCloudSyncLicensingMessage(actionType, action.parameters.licenseExpiryDate, action.parameters.serialNumber);
                case 'ipaMismatch':
                    return 'Your BYOL subscription license was not installed. The serial number ' + action.parameters.serialNumber + ' does not match your set NSS credentials, please contact account team or speak to a support representative by logging into mysupport.netapp.com and initiate a Live Chat with topic “Product Registration”';
                default:
                    return 'Unknown';
            }
        };
    });

    angular.module('noc').filter('actionIntoWordsDetails', (nocUtil, providerDiskTypes) => {
        return (action, workingEnvironment) => {
            const diskTypes = providerDiskTypes[workingEnvironment._cloudProviderName];
            const actionType = action.actionType;
            action.parameters.diskSize = nocUtil.normalizeCapacity(action.parameters.diskSize);
            const capacity = {
                size: action.parameters.diskSize.size * action.parameters.numOfDisks,
                unit: action.parameters.diskSize.unit
            };
            const haText = workingEnvironment && workingEnvironment.isHA && workingEnvironment._isAWS ? ' per node' : '';

            if (actionType === 'NeedsMoreDisks') {
                return 'I approve for Cloud Manager to purchase ' + capacity.size + ' ' + capacity.unit + ' of ' + workingEnvironment._cloudProviderName + ' capacity in aggregate ' + action.parameters.aggregateName + ' (' + action.parameters.numOfDisks + ' ' + diskTypes[action.parameters.providerVolumeType].display + ' disk' + (action.parameters.numOfDisks > 1 ? 's' : '') + (haText ? ')' + haText + '.' : ').');
            }
            if (actionType === 'MoreDisksNeededButDiskLimitExceeded') {
                const volumeMoveParameters = action.parameters.volumeMoveParameters;
                if (volumeMoveParameters) {
                    if (volumeMoveParameters.isNewTargetAggregate || action.parameters.numOfDisks) {
                        return 'To avoid capacity issues, I approve for Cloud Manager to purchase ' + capacity.size + ' ' + capacity.unit + ' of ' + workingEnvironment._cloudProviderName + ' capacity in aggregate ' + action.parameters.aggregateName + ' (' + action.parameters.numOfDisks + ' ' + diskTypes[action.parameters.providerVolumeType].display + ' disk' + (action.parameters.numOfDisks > 1 ? 's' : '') + haText + ') and to move an existing volume to these new disks.';
                    } else {
                        return 'To avoid capacity issues, I approve for Cloud Manager to move an existing volume to another set of disks.';
                    }
                } else if (action.parameters.moreInfo === 'INSTANCE_DISK_LIMIT_EXCEEDED') {
                    return 'Aggregate ' + action.parameters.aggregateName + ' has reached its capacity limit. You can either free space on the system by deleting or rearranging volumes, or you can move one or more volumes to another system. Refer to the Cloud Manager documentation to get help.';
                } else if (action.parameters.moreInfo === 'NO_SINGLE_VOLUME') {
                    return 'Aggregate ' + action.parameters.aggregateName + ' has reached its capacity limit. You must identify one or more volumes to move out of this aggregate and then move them to another aggregate on this system. Refer to the Cloud Manager documentation to get help.';
                } else {
                    //Sanity, should never happen
                    return 'Moving a volume is necessary to avoid capacity issues; however, Cloud Manager cannot provide recommendations to correct the issue. Refer to the Cloud Manager documentation to get help.';
                }
            }
            if (actionType === 'MoreDisksNeededButVsaLimitExceeded') {
                const licenseParams = action.parameters.licenseParameters;
                if (licenseParams) {
                    const textPrefix = 'Purchasing additional ' + workingEnvironment._cloudProviderName + ' disks requires a license upgrade. ';
                    if (workingEnvironment.isHA) {
                        return textPrefix + 'Cloud Volumes ONTAP will not be highly available during this operation. I/O will not be interrupted.';
                    } else {
                        return textPrefix + 'This process restarts Cloud Volumes ONTAP, which means I/O will be interrupted.';
                    }
                } else {
                    return 'Cloud Volumes ONTAP license limit prevents Cloud Manager from purchasing the required additional ' + workingEnvironment._cloudProviderName + ' capacity. Please refer to Cloud Manager documentation for recommended corrective actions.';
                }
            }
            if (actionType === 'LicenseUsedCapacityWarning') {
                if (action.parameters.licenseParameters) {
                    const licenseUsedCapacityWarningTextPrefix = 'Cloud Manager can change the Cloud Volumes ONTAP license from ' +
                        workingEnvironment.ontapClusterProperties.licenseType.name + ' to ' + action.parameters.licenseParameters.newLicenseType.name +
                        ', which increases the capacity limit to ' + action.parameters.licenseParameters.newLicenseType.capacity_limit + '. ';
                    if (workingEnvironment.isHA) {
                        return licenseUsedCapacityWarningTextPrefix + 'This operation is nondisruptive. The HA pair continues to serve data.';
                    } else {
                        return licenseUsedCapacityWarningTextPrefix + 'This process restarts Cloud Volumes ONTAP, which means I/O is interrupted.';
                    }
                } else {
                    return 'This system uses a license with the highest available capacity limit. To avoid capacity issues, you should move volumes to other Cloud Volumes ONTAP systems.';
                }
            }
            if (actionType === 'LicenseAllocatedCapacityError') {
                if (action.parameters.licenseParameters) {
                    const licenseUsedCapacityWarningTextPrefix = 'Cloud Manager can change the ONTAP Cloud license from ' +
                        workingEnvironment.ontapClusterProperties.licenseType.name + ' to ' + action.parameters.licenseParameters.newLicenseType.name +
                        ', which increases the capacity limit to ' + action.parameters.licenseParameters.newLicenseType.capacity_limit + '. ';
                    if (workingEnvironment.isHA) {
                        return licenseUsedCapacityWarningTextPrefix + 'This operation is nondisruptive. The HA pair continues to serve data.';
                    } else {
                        return licenseUsedCapacityWarningTextPrefix + 'This process restarts ONTAP Cloud, which means I/O is interrupted.';
                    }
                } else {
                    return 'This system uses a license with the highest available capacity limit. To avoid capacity issues, you should move volumes to other ONTAP Cloud systems.';
                }
            }
            if (actionType === 'ShouldBeRemoved') {
                return 'I approve for Cloud Manager to remove ' + capacity.size + ' ' + capacity.unit + ' of unused ' + workingEnvironment._cloudProviderName + ' capacity currently available to Cloud Volumes ONTAP.';
            }
            if (actionType === 'EmptyWorkingEnvironment') {
                return 'I approve for Cloud Manager to remove this working environment and its attached ' + workingEnvironment._cloudProviderName + ' resources, all disks and Snapshots associated with this working environment will be permanently deleted. The ' + (workingEnvironment._cloudProviderName === 'AWS' ? 'VPC' : 'VNet') + ' will not be deleted.';
            }
            if (actionType === 'NonCompliantConfig') {
                const nonCompliantConfigTextPrefix = 'Cloud Manager can change the Cloud Volumes ONTAP license and instance. ';
                if (workingEnvironment.isHA) {
                    return nonCompliantConfigTextPrefix + 'This operation is nondisruptive. The HA pair continues to serve data.';
                } else {
                    return nonCompliantConfigTextPrefix + 'This process restarts Cloud Volumes ONTAP, which means I/O is interrupted.';
                }
            }
            if (_.startsWith(actionType, 'license')) {
                const serialInfo = 'Use your Cloud Volumes ONTAP BYOL Serial Number when requesting renewal services:<br>' + action.parameters.serialNumber;
                const licenseLink = '<br><br>For information on how to renew your subscription, please visit:<br><a href="http://www.netapp.com/us/how-to-buy/index.aspx" target="_blank">http://www.netapp.com/us/how-to-buy/index.aspx</a>';

                return serialInfo + licenseLink;
            }

            if (actionType === 'MaxDisksLimitAboutToExceed') {
                return (workingEnvironment.isHA ? 'Each node in this HA pair ' : 'This system ') + `has ${action.parameters.numOfDisks} data disks. The maximum allowed is ${action.parameters.moreInfo}` + (workingEnvironment.isHA ? ' per node' : '') + (action.parameters.numOfDisks === parseInt(action.parameters.moreInfo) ? '. If the data continues to grow, you might need to move some data to another Cloud Volumes ONTAP system to free space.' : '. Consider creating additional aggregates with larger disks to ensure that you can utilize the maximum capacity allowed by your Cloud Volumes ONTAP license.');
            }

            if (actionType === 'MaxDisksLimitExceeded') {
                return (workingEnvironment.isHA ? 'Each node in this HA pair ' : 'This system ') + `has ${action.parameters.numOfDisks} data disks, which exceeds the limit of ${action.parameters.moreInfo}` + (workingEnvironment.isHA ? ' per node' : '') + '. Contact NetApp technical support for assistance.';
            }

            if (actionType === 'deprecatedInstance') {
                return `The EC2 instance type for this system is ${action.parameters.currentInstanceType}. This instance type is no longer supported with new Cloud Volumes ONTAP systems due to performance limitations. While this Cloud Volumes ONTAP system will continue to operate normally, NetApp strongly recommends changing to ${action.parameters.licenseParameters.newInstanceType} to get the best performance.<br>To review pricing differences between instance types and NetApp licenses, go to the AWS Marketplace for <a target="_blank" href="https://aws.amazon.com/marketplace/pp/B011KEZ734">single-node Cloud Volumes ONTAP systems</a> and for <a target="_blank" href="https://aws.amazon.com/marketplace/pp/B01H4LVJ84">Cloud Volumes ONTAP HA pairs</a>.`;
            }

            if (actionType === 'PaygoNotRegisteredToSupport') {
                return 'This system has not been registered with NetApp support. To register, go to the Support Registration page.';
            }

            return 'Unknown';
        };
    });

    angular.module('noc').filter('actionIntoWordsAdvancedInformation', (nocUtil, providerDiskTypes) => {
        return (action, workingEnvironment) => {
            const diskTypes = providerDiskTypes[workingEnvironment._cloudProviderName];
            const actionType = action.actionType;
            action.parameters.diskSize = nocUtil.normalizeCapacity(action.parameters.diskSize);
            const haText = workingEnvironment && workingEnvironment.isHA && workingEnvironment._isAWS ? ' per node' : '';

            if (actionType === 'MoreDisksNeededButDiskLimitExceeded') {
                const volumeMoveParameters = action.parameters.volumeMoveParameters;
                const sourceAggregateStrong = strong(action.parameters.aggregateName);
                if (volumeMoveParameters) {
                    const volumeToMoveStrong = strong(volumeMoveParameters.volumeName);
                    const targetAggregateStrong = strong(volumeMoveParameters.targetAggregateName);
                    const prefixVolMoveMessage = 'Aggregate ' + sourceAggregateStrong + ' has reached its capacity limit. Cloud Manager will move volume ' + volumeToMoveStrong + ' from ' + sourceAggregateStrong;
                    if (volumeMoveParameters.isNewTargetAggregate) {
                        return prefixVolMoveMessage + ' to a newly created aggregate, ' + targetAggregateStrong + '. Additional space will be available on ' + sourceAggregateStrong + ' for existing volumes and new volumes.';
                    } else if (action.parameters.numOfDisks) {
                        return prefixVolMoveMessage + ' to ' + targetAggregateStrong + ', which will require additional disks. Additional space will be available on ' + sourceAggregateStrong + ' for existing volumes and new volumes.';
                    } else {
                        return prefixVolMoveMessage + ' to ' + targetAggregateStrong + ', which has available capacity. Additional space will be available on ' + sourceAggregateStrong + ' for existing volumes and new volumes.';
                    }
                } else {
                    return 'Aggregate <strong>' + action.parameters.aggregateName + ' </strong> has reached its capacity limit.';
                }
            } else if (actionType === 'ShouldBeRemoved') {
                return 'Aggregate <strong>' + action.parameters.aggregateName + ' </strong> will be deleted along with ' + action.parameters.numOfDisks + ' ' + diskTypes[action.parameters.providerVolumeType].display + ' disk' + (action.parameters.numOfDisks > 1 ? 's' : '') + haText + '.';
            }

            return 'Unknown';
        };
    });

    angular.module('noc').filter('actionButtonText', (actionRequiredService) => {
        return (action) => {
            if (action.actionType === 'ShouldBeRemoved' || action.actionType === 'EmptyWorkingEnvironment') return '& Delete';
            if (action.actionType === 'EvaluationByol') return '& Update';
            if (action.actionType === 'deprecatedInstance') return '& Change';
            if (actionRequiredService.isFixable(action)) return '& Approve';

            return '';
        };
    });

    angular.module('noc').filter('approveButtonText', () => {
        return (action) => {
            if (action.actionType === 'EmptyWorkingEnvironment' || action.actionType === 'ShouldBeRemoved') return 'Delete';
            else if (action.actionType === 'deprecatedInstance') return 'Change';
            else return 'Approve Now';
        };
    });

    angular.module('noc').filter('actionRequiredTitle', () => {
        return (action) => {
            if (action.actionType === 'ShouldBeRemoved' || action.actionType === 'EmptyWorkingEnvironment') {
                return 'Review & Delete';
            } else if (_.startsWith(action.actionType, 'license')) {
                return 'License Renewal';
            } else if (action.actionType === 'NonCompliantConfig') {
                return 'License or instance change';
            } else if (action.actionType === 'MaxDisksLimitAboutToExceed' || action.actionType === 'MaxDisksLimitExceeded' || action.actionType === 'PaygoNotRegisteredToSupport') {
                return 'Review';
            } else if (action.actionType === 'deprecatedInstance') {
                return 'An EC2 instance type change is recommended';
            } else {
                return 'Capacity threshold exceeded';
            }
        };
    });
})();
