(() => {
    'use strict';

    angular.module('noc').factory('aggregatesResource', (nocPoller, nocUtil, $filter) => {
        const resource = {};

        resource.getUrlForGetAggregatesApi = function getUrlForGetAggregatesApi(workingEnvironment) {
            // All working environment types except Azure and Google (ON_PREM, NPS, VSA and HA) are using a query parameter, which
            // is wrong but can't be changed for backward compatibility. Only Azure is using the right API with a path
            // parameter for working environment id. Hence the url for Azure & Google is built differently.
            return (workingEnvironment._typeAndProvider === 'VSA_Azure' || workingEnvironment._typeAndProvider === 'VSA_GCP') ?
                workingEnvironment._pathPrefix + 'aggregates/' + workingEnvironment.publicId :
                workingEnvironment._pathPrefix + 'aggregates?workingEnvironmentId=' + workingEnvironment.publicId;
        };

        resource.initializeAggregate = function initializeAggregate(aggregate) {
            aggregate._normalizedAvailableCapacity = nocUtil.normalizeCapacity(aggregate.availableCapacity);
            aggregate._normalizedTotalCapacity = nocUtil.normalizeCapacity(aggregate.totalCapacity);
            //Add normalized capacity field to every volume to optimize filtering and display:
            aggregate._displayCapacity = $filter('normalizedCapacity')(aggregate._normalizedTotalCapacity, 2, true);
            aggregate._displayUsedCapacity = $filter('normalizedCapacity')(aggregate.usedCapacity, 2, true);
            if (aggregate.capacityTier) {
                aggregate._displayUsedCapacityTier = $filter('normalizedCapacity')(aggregate.capacityTierUsed, 2, true);
            }
            aggregate._numOfDisks = _.get(aggregate.disks, 'length');
            aggregate._numOfVolumes = _.get(aggregate.volumes, 'length');
            if (aggregate.providerVolumes) {
                aggregate._providerVolumeType = _.get(aggregate, 'providerVolumes[0].diskType');
            }

            const totalCapacity = nocUtil.convertTo(aggregate.totalCapacity, 'Byte').size;
            const usedCapacity = nocUtil.convertTo(aggregate.usedCapacity, 'Byte').size;
            const capacityTierUsed = nocUtil.convertTo(aggregate.capacityTierUsed, 'Byte').size;
            const capacityBarMaxWidth = aggregate.capacityTier === 'cloudStorage' ? 222 : 240;

            if (!capacityTierUsed || (totalCapacity > capacityTierUsed)) {
                aggregate._diskAllocatedWidth = capacityBarMaxWidth;
                aggregate._diskUsedWidth = usedCapacity / totalCapacity * aggregate._diskAllocatedWidth;
                aggregate._tierWidth = capacityTierUsed ? (capacityTierUsed / totalCapacity * aggregate._diskAllocatedWidth) : 0;
            } else {
                aggregate._tierWidth = capacityBarMaxWidth;
                aggregate._diskAllocatedWidth = totalCapacity / capacityTierUsed * aggregate._tierWidth;
                aggregate._diskUsedWidth = usedCapacity / totalCapacity * aggregate._diskAllocatedWidth;
            }
        };

        resource.startAggregatesPolling = function startAggregatesPolling(workingEnvironment, successCallback, errorCallback) {
            return nocPoller.start({
                method: 'GET',
                url: resource.getUrlForGetAggregatesApi(workingEnvironment)
            }, 1 * 60 * 1000, (aggregatesResponse) => {
                _.forEach(aggregatesResponse, resource.initializeAggregate);
                successCallback(aggregatesResponse);
            }, errorCallback, null, true);
        };

        resource.getAggregates = function getAggregates(nocHttp, workingEnvironment) {
            return nocHttp.get(resource.getUrlForGetAggregatesApi(workingEnvironment),
                'retrieving aggregates for working environment ' + workingEnvironment.name, nocHttp.LOADING)
                .then((aggregatesResponse) => {
                    _.forEach(aggregatesResponse, (aggregate) => {
                        aggregate._providerVolumeType = _.get(aggregate, 'providerVolumes[0].diskType');
                    });

                    return aggregatesResponse;
                });
        };

        resource.createAggregate = function createAggregate(nocHttp, workingEnvironment, aggregateRequest) {
            return nocHttp.post({
                url: workingEnvironment._pathPrefix + 'aggregates',
                data: aggregateRequest
            }, 'creating aggregate', nocHttp.BLOCK);
        };

        resource.addDisksToAggregate = function addDisksToAggregate(nocHttp, workingEnvironment, aggregate, addDisksRequest) {
            return nocHttp.post({
                url: workingEnvironment._pathPrefix + 'aggregates/' + workingEnvironment.publicId + '/' + aggregate.name + '/disks',
                data: addDisksRequest
            }, 'adding disks to aggregate ' + aggregate.name, nocHttp.BLOCK);
        };

        resource.deleteAggregate = function deleteAggregate(nocHttp, workingEnvironment, aggregate) {
            return nocHttp.delete(workingEnvironment._pathPrefix + 'aggregates/' + workingEnvironment.publicId + '/' + aggregate.name,
                'deleting aggregate ' + aggregate.name,
                nocHttp.BLOCK);
        };

        return resource;
    });
})();
