(() => {
    'use strict';

    angular.module('noc').factory('auditResource', (nocPoller, $rootScope, $http, $log) => {
        var httpConfig;
        var resource = {
            auditItems: [],
            openedNodes: {}
        };

        resource.AUDIT_GET_EVENT = "auditGetSuccess";
        resource.AUDIT_GET_ERROR_EVENT = "auditGetError";

        resource.summaryToRefreshFrom = function (summaries) {
            if (summaries && summaries.length) {
                return _.findLast(summaries, function (summary) {
                        return summary.status === 'Received';
                    }) || summaries[0];
            } else {
                return undefined;
            }
        };

        //calls the handler only if the original request is still relevant
        var ifPending = function (originalHttpConfig, handler) {
            return function (response) {
                if (_.isEqual(originalHttpConfig.params, httpConfig.params)) {
                    handler(response);
                } else {
                    $log.debug("audit: got new request to get audit (tab move or timeFrame change). Ignoring result of obsolete request.");
                }
            };
        };
        var successHandler = function (workingEnvironmentId) {
            return function (summaries) {
                resource.processRawAuditData(summaries, workingEnvironmentId, false);
            };
        };

        var errorHandler = function (workingEnvironmentId) {
            return function (error) {
                $rootScope.$broadcast(resource.AUDIT_GET_ERROR_EVENT, {workingEnvironment: workingEnvironmentId, error: error});
            };
        };

        resource.processRawAuditData = function(summaries, workingEnvironmentId, mergeGroupRecords) {
            var processedAudits = resource.buildAuditSummery(summaries);
            processedAudits = resource.mergeSubGroups(processedAudits);
            if(!mergeGroupRecords) {
                var summaryToRefreshFrom = resource.summaryToRefreshFrom(processedAudits);
                if (summaryToRefreshFrom) httpConfig.params.after = summaryToRefreshFrom.startDate;
            }
            processedAudits = resource.processSummaries(processedAudits);
            processedAudits = resource.mergeSummaries(processedAudits, resource.auditItems, mergeGroupRecords);
            if(!_.isEqual(processedAudits, resource.auditItems)) {
                angular.copy(processedAudits, resource.auditItems);
            }
            $rootScope.$broadcast(resource.AUDIT_GET_EVENT, {workingEnvironment: workingEnvironmentId, summaries: processedAudits});
        };

        resource.parseParameters = function(parameters){
            if(parameters){
                //noinspection UnusedCatchParameterJS,EmptyCatchBlockJS
                try {
                    return JSON.parse(parameters);
                } catch (e) {
                    //do nothing
                }
            }

            return undefined;
        };

        resource.mergeSummaries = function(newSummaries, oldSummaries, isExpand) {
            if (!newSummaries || !newSummaries.length) return oldSummaries;
            if (!oldSummaries || !oldSummaries.length) return newSummaries;

            if(!isExpand) {
                var lastNewIdx = _.findIndex(oldSummaries, {id : _.last(newSummaries).id});

                if (lastNewIdx === oldSummaries.length - 1) {
                    return newSummaries;
                } else {
                    return _.concat(newSummaries, oldSummaries.slice(lastNewIdx + 1));
                }
            } else {
                _.forEach(newSummaries, function(summery) {
                    var entryIndex = _.findIndex(oldSummaries, {id : summery.id});

                    if(entryIndex !== -1) {
                        oldSummaries[entryIndex] = summery;
                    }
                });

                return oldSummaries;
            }
        };

        resource.processNode = (node, parametersKey) => {
            const clone = _.clone(node);
            clone._open = resource.openedNodes[node.id];

            if (node[parametersKey]) {
                clone._jsonParameters = resource.parseParameters(node[parametersKey]);
                if (clone.workingEnvironmentName) {
                    clone._jsonParameters.workingEnvironmentName = clone.workingEnvironmentName;
                }
            }

            if (clone._jsonParameters) {
                clone._formatedParameters = JSON.stringify(clone._jsonParameters, null, 2);
            }

            return clone;
        };

        resource.processSummaries = function(summaries) {
            summaries = _.reject(summaries, function(summary){
                return summary.actionName === "Backup Ontap Configuration" || summary.actionName === "Receive Asup From Ontap Cloud";
            });

            return _.map(summaries, function (summary) {
                var _summary = resource.processNode(summary, 'actionParameters');

                _summary._records = _.map(_summary.records, function(record){
                    return resource.processNode(record, 'parameters');
                });

                return _summary;
            });
        };

        resource.mergeSubGroups = function(audits) {
            var parts = _.partition(audits, 'parentId');

            _.forEach(parts[0], function(subgroup) {
                var parentIndex = _.findIndex(parts[1], {_id: subgroup.parentId });
                if(parentIndex !== -1) {
                    parts[1][parentIndex].records = parts[1][parentIndex].records.concat(subgroup.records);
                    parts[1][parentIndex].records = _.sortBy(parts[1][parentIndex].records, 'date').reverse();
                }
            });

            return parts[1];
        };

        resource.buildAuditSummery = function(rawAudits) {
            return _.sortBy(rawAudits, 'startDate').reverse();
        };

        function buildHttpConfigData(parameters, workingEnvironmentId) {
            httpConfig = {
                method: 'GET',
                url: '/audit/auditserver',
                params: _.clone(parameters) || {}
            };
            if (workingEnvironmentId) {
                httpConfig.params.workingEnvironmentId = workingEnvironmentId;
            }
        }

        resource.startPolling = function(parameters, workingEnvironmentId) {
            buildHttpConfigData(parameters, workingEnvironmentId);
            var onSuccess = ifPending(httpConfig, successHandler(workingEnvironmentId));
            var onError = ifPending(httpConfig, errorHandler(workingEnvironmentId));
            return nocPoller.start(httpConfig, 5000, onSuccess, onError, true);
        };

        resource.get = function (parameters, workingEnvironment, isExpand) {
            buildHttpConfigData(parameters, workingEnvironment);
            var onSuccess = ifPending(httpConfig, successHandler(workingEnvironment));
            var onError = ifPending(httpConfig, errorHandler(workingEnvironment));

            buildHttpConfigData(parameters, workingEnvironment);
            $log.debug('audit: calling once with ' + JSON.stringify(httpConfig));

            $http(httpConfig)
                .then(function (response) {
                    if(isExpand) {
                        resource.processRawAuditData(response.data, undefined, true);
                    } else {
                        onSuccess(response.data);
                    }
                })
                .catch(function (response) {
                    onError(response.data);
                });
        };

        resource.expandGroup = function(node) {
            resource.openedNodes[node.id] = node._open;
            if(node.records.length === 0) {
                resource.get({
                    groupId: node._id
                }, undefined, true);
            }
        };

        resource.reset = function() {
            angular.copy([], resource.auditItems);
            resource.openedNodes = {};
        };

        return resource;
    });
})();
