(() => {
    'use strict';

    angular.module('noc').factory('volViewVolumesResource', ($q, $filter, nocPoller, $http, providerDiskTypes) => {
        const normalizeCapacityFilter = $filter('normalizedCapacity');
        const resource = {},
            MANAGED_VOLUMES_URL = '/managed/aws/vsa/volumes';

        resource.getRegions = function (nocHttp) {
            return nocHttp.get("/vsa/metadata/regions", "retrieving regions", nocHttp.LOADING);
        };

        resource.getVPCs = function (nocHttp, region) {
            return nocHttp.get(
                { url: "/vsa/metadata/vpcs", params: {region: region}},
                "retrieving VPCs",
                nocHttp.LOADING)
                .then(function(vpcs){
                    _.forEach(vpcs,function(vpc){
                        var nameTag = _.find(vpc.tags,{key:'Name'});
                        if(nameTag){
                            vpc._name = nameTag.value;
                        }
                    });

                    return vpcs;
                });
        };

        resource.startVolumesPolling = function (successFn) {
            var initializeVolumesResponse = function (response) {
                _.forEach(response.volumes, resource.initVolume);
                successFn(response);
            };
            return nocPoller.start(MANAGED_VOLUMES_URL, 30 * 1000, initializeVolumesResponse, _.noop, _.noop, true);
        };

        resource.createVolume = function (nocHttp, volumeRequest) {
            return nocHttp.post({
                url: MANAGED_VOLUMES_URL + (volumeRequest.haParams ? '/ha' : ''),
                data: volumeRequest
            }, null, nocHttp.BLOCK);
        };

        resource.updateVolume = function (nocHttp, volumeRequest) {
            return nocHttp.put({
                url: "/managed/aws/vsa/volumes/" + volumeRequest.id,
                data: volumeRequest
            }, "updating volume", nocHttp.BLOCK);
        };

        resource.deleteVolume = function (nocHttp, volume) {
            return nocHttp.delete('/managed/aws/vsa/volumes/' + volume.id, "deleting volume " + volume.name, nocHttp.LOADING);
        };

        resource.cloneVolume = function (nocHttp, sourceVolId, cloneVolName) {
            return nocHttp.post({
                url: '/managed/aws/vsa/volumes/' + sourceVolId + '/clone',
                data: {
                    newVolumeName: cloneVolName
                }
            }, 'cloning volume', nocHttp.LOADING);
        };

        var creationTimes = {
            "allocating new resources and creating": 25,
            "adding resources and creating": 5,
            "creating": 1
        };

        var statusColors = {
            //Volumes statuses:
            online: 'green',
            updating: 'green',
            offline: 'red',
            deleting: 'red',
            creating: 'blue',
            degraded: 'orange'
        };

        resource.initVolume = function (volume) {
            volume._diskType = volume.diskType ? providerDiskTypes['AWS'][volume.diskType].display : '';
            volume._size = normalizeCapacityFilter(volume.size, 2, true);
            volume._usedSize = normalizeCapacityFilter(volume.usedSize, 2, true);
            volume._mountPoint = volume.mountPoint || 'Not Exported';
            volume._location = $filter('regionFormat')(volume.region) + (volume.subnetCidrs ? (", " + volume.subnetCidrs.join(", ")) : "");
            volume.status = _(volume.status).toLower().replace(/_/g, ' ');
            volume._displayStatus = _.split(volume.status, ' ').pop(); //last word of the status, should be "online", "offline", "degraded" or "creating".
            volume._statusColor = statusColors[volume._displayStatus];
            var creatingTime = creationTimes[volume.status];
            volume._statusPostfix = creatingTime ? (" [About " + creatingTime + " minute" + (creatingTime === 1 ? "]" : "s]")) : "";
            volume._menuActive = volume.status === 'online' || volume.status === 'degraded';
            volume._attributes = volume.parentVolumeName ? 'Cloned Volume' : '';

            return volume;
        };

        resource.validateVolumeName = function (volumeName) {
            return $http.get(MANAGED_VOLUMES_URL, {params: { volumeName: volumeName }})
                .then(function (response) {
                    return response.data.volumes.length > 0 ? $q.reject() : $q.when();
                });
        };

        // Checks whether there are unavailable* instances for managed volume creation.
        // *unavailable in a sense that they could have been selected for volume creation if they were ON
        resource.hasUnavailableInstances = function (nocHttp, volumeType) {
            return nocHttp.get(MANAGED_VOLUMES_URL, 'retrieving unavailable instances')
                .then(function (response) {
                    return _(response['unavailableInstances'])
                        .filter(function (item) {
                            var expectedNumberOfInstances = (volumeType === 'ha' ? 2 : 1);
                            return item.instanceIds.length === expectedNumberOfInstances;
                        })
                        .some(function (item) {
                            return item.status === "UPDATING" || item.status === "STOPPING" || item.status === "STARTING" || item.status === "OFF";
                        }) ||
                        (volumeType === 'ha' && _.some(response['volumes'], {status: 'DEGRADED'}));
                });
        };

        resource.possiblePlacement = function (nocHttp, request) {
            var url = request.haParams ? '/managed/aws/vsa/volumes/ha/possible-placement' : '/managed/aws/vsa/volumes/possible-placement';
            return nocHttp.post({
                url: url,
                data: request
            }, 'checking for possible placement for volume', nocHttp.BLOCK);
        };

        resource.getPossibleDiskTypes = function getPossibleDiskTypes(isHa) {
            return _(providerDiskTypes['AWS'])
                .reject({name: 'standard'}) //'standard' is deprecated.
                .reject({name: 'io1'}) //not supported currently in volume  view
                .reject(function (type) {
                    return _.endsWith(type.name, 'S3');
                }) //Composite tiers are not supported in Volume View.
                .filter(function (diskType) {
                    return !isHa || diskType.supportedInHa;
                }).value();
        };

        resource.quoteForDiskTypeChange = function (nocHttp, volume, diskType) {
            return nocHttp.post({
                url: '/managed/aws/vsa/volumes/' + volume.id + '/quote',
                data: {
                    diskType: diskType
                }
            }, 'requesting quote for disk type change', nocHttp.LOADING);
        };

        resource.changeDiskType = function changeDiskType(nocHttp, volume, diskType, quote) {
            var changeDiskTypeData = {
                diskType: diskType,
                aggregateName: _.get(quote, 'vsaAndVolume.aggregateName')
            };
            if (quote && quote.upgradeInfo) {
                changeDiskTypeData.allowVsaUpgrade = true;
                changeDiskTypeData.upgradeCreateRequest = {
                    vsaAndVolume: quote.vsaAndVolume,
                    upgradeInfo: quote.upgradeInfo
                };
            } else {
                changeDiskTypeData.allowVsaUpgrade = false;
            }
            return nocHttp.post({
                url: '/managed/aws/vsa/volumes/' + volume.id + '/change-disk-type',
                data: changeDiskTypeData
            }, 'changing underlying disk type', nocHttp.LOADING);
        };

        return resource;
    });
})();
