(() => {
    'use strict';
    function restore(key) {
        const value = _.attempt(JSON.parse, window.localStorage[key]);

        return _.isError(value) ? null : value;
    }

    function save(key, state) {
        window.localStorage[key] = JSON.stringify(state);
    }

    function clear(key) {
        delete window.localStorage[key];
    }

    const factory = function (base, secondaryKeyOrFunction) {
        function key() {
            const secondaryKey = _.isFunction(secondaryKeyOrFunction) ? secondaryKeyOrFunction() : secondaryKeyOrFunction;

            return base + (secondaryKey ? '.' + secondaryKey : '');
        }

        return {
            restore: function (defaultValue) {
                return restore(key()) || defaultValue;
            },
            save: function (value) {
                save(key(), value);
            },
            clear: function () {
                clear(key());
            },
            isDefined: function() {
                return !_.isNil(window.localStorage[key()]);
            }
        };
    };

    const gridFactory = function (base) {
        const service = factory(base);
        const parentRestore = service.restore;
        service.restore = function (gridApi) {
            if (!gridApi) throw new Error('Must pass grid API to restore grid definitions');

            const areGridsEqual = function areGridsEqual(gridApi, restoredValue) {
                const gridApiColumns = _.map(gridApi.grid.columns, 'name');
                const localStorageColumns = _.map(restoredValue.columns, 'name');

                return _.isEqual(gridApiColumns, localStorageColumns);
            };

            const restoredValue = parentRestore();

            if (restoredValue && areGridsEqual(gridApi, restoredValue)) {
                return restoredValue;
            } else {
                this.clear();

                return null;
            }
        };

        return service;
    };

    const storages = {};

    /*eslint-disable no-multi-spaces*/
    storages.DEBUG =                                                   factory('noc.debug.enabled');
    storages.ENCRYPTION_VIEW =                                         factory('noc.encryption.keyManager.shown');
    storages.GALLERY_VIEW_SHOWN =                                      _.partial(factory, 'noc.gallery.grid.shown');
    storages.SELECTED_TENANT =                                         factory('noc.selected.tenant');
    storages.INTERCOM_PRIVATE_INFO =                                   _.partial(factory, 'noc.intercom.private.info');
    storages.LOG_STORAGE =                                             factory('occm.logger.store');
    storages.DISABLE_ASYNC_RESPONDER =                                 factory('occm.network.disableasync');
    storages.VOLUME_VIEW =                                             factory('occm.mode.volume.view'); // Boolean
    storages.LATEST_UPDATE_IGNORED =                                   factory('occm.latest.update.noticed');
    storages.DONT_SHOW_CLOUD_SYNC_AD =                                 factory('occm.dont.show.cloud.sync.ad');
    storages.LATEST_LOGIN_VERSION =                                    factory('occm.latest.login.version');
    storages.DISABLE_AZURE_MP_CHECK =                                  factory('occm.azure.mp.check');
    storages.LAST_SCAN_DATE =                                          factory('occm.last.scan.date');
    storages.DONT_SHOW_CLOUD_STORAGE_AUTOMATION_TOOLTIP_IN_TESTIM_IO = factory('occm.dont.show.cloud.storage.automation.tooltip.in.testim.io');
    storages.DONT_SHOW_AUTO_CAPACITY_DIALOG =                          factory('occm.dont.show.auto.capacity.dialog');
    storages.LOGINN_AFTER_MIGRATION =                                  factory('occm.login.after.migration');
    storages.DONT_SHOW_MARKETING_POPUP =                               factory('occm.dont.show.marketing.popup');
    storages.MARKETING_POPUP_COUNTER =                                 factory('occm.marketing.popup.counter');

    storages.GALLERY_GRID =                                            gridFactory('noc.gallery.grid.state');
    storages.REPLICATION_STATUS_GRID =                                 gridFactory('noc.replication.status.grid.state');
    storages.VOLUME_VIEW_VOLUMES_GRID =                                gridFactory('noc.volume.view.volumes.grid.state');
    storages.VOLUME_VIEW_RESOURCES_GRID =                              gridFactory('noc.volume.view.resources.grid.state');
    /*eslint-enable*/

    angular.module('noc').constant('nocLocalStorage', _.assignIn(factory, storages));
})();
