(() => {
    'use strict';

    angular.module("noc").factory('statusRequestPoller', function ($injector, $rootScope, $timeout,$log) {
        var poller = {};
        poller.REQUEST_STATUS_POLL_INTERVAL_MILLIS = 1000;
        poller.REQUEST_STATUS_POLL_TIMEOUT_MILLIS = 60 * 60 * 1000;
        poller.REQUEST_STATUS_CHANGED_EVENT = 'requestStatusChanged';

        var paused = false;

        poller.pause = function() {
            paused = true;
        };

        poller.unpause = function() {
            paused = false;
        };
        poller.pollOnRequestStatus = function (requestId, requestConfig, pollStartTime) {
            if (paused) {
                // if it's paused we schedule it to re-poll for this request id later
                // (if then it wouldn't be paused otherwise it would reschedule again)
                $timeout(
                    function () {
                        poller.pollOnRequestStatus(requestId, requestConfig, pollStartTime);
                    },
                    poller.REQUEST_STATUS_POLL_INTERVAL_MILLIS
                );
                return;
            }

            //fetching the $http service this way to avoid circular dependency with the interceptor
            var $http = $injector.get("$http");

            $http.get('/audit/' + requestId)
                .then(function (response) {
                    var data = response.data;
                    var status = response.status;
                    if (status === 200 && _.isArray(data) && data.length === 1) {

                        //to play is safe, only retries if the status is Received
                        if (data[0].status === 'Received') {
                            $log.debug('Request status poller - retries in ' + poller.REQUEST_STATUS_POLL_INTERVAL_MILLIS/1000 + ' seconds to check status of ' + requestConfig.method + 'ing ' + requestConfig.url);

                            if (Date.now() - pollStartTime < poller.REQUEST_STATUS_POLL_TIMEOUT_MILLIS) {
                                $timeout(
                                    function () {
                                        poller.pollOnRequestStatus(requestId, requestConfig, pollStartTime);
                                    },
                                    poller.REQUEST_STATUS_POLL_INTERVAL_MILLIS
                                );
                            }else{
                                $log.warn("Request status poller - more than "+poller.REQUEST_STATUS_POLL_TIMEOUT_MILLIS/(60*1000)+" minutes have passed since start of request status poller. Request will probably never complete... Stop polling for status.");
                            }
                        } else {
                            $log.debug(new Date() + 'Request status poller - broadcasting a change event for ' + requestConfig.method + 'ing ' + requestConfig.url);

                            $rootScope.$broadcast(poller.REQUEST_STATUS_CHANGED_EVENT);
                        }
                    } else {
                        $log.error('Request status poller - got bad response for checking status of ' + requestConfig.method + 'ing ' + requestConfig.url + ". Status is " + status + " and data is " + data);
                    }
                })
                .catch(function(error){
                    $log.error('Request status poller - got bad response for checking status of ' + requestConfig.method + 'ing ' + requestConfig.url + ". Status is " + error.status + " and message is "+ error.message);
                });
        };

        return poller;
    });

    angular.module('noc').config(($provide) => {
        $provide.factory('statusRequestInterceptor', (statusRequestPoller, $log) => {
            return {
                response: (response) => {
                    if (/(POST|PUT|DELETE)/.test(response.config.method)) {
                        const requestId = response.headers()['oncloud-request-id'];

                        //  Starting with a simplistic approach - broadcasting the completion of every post.
                        //  All pollers are notified and refresh, regardless of the relevance to what they're polling
                        const url = response.config.url;

                        if (!(/\/login$/.test(url))) { //Ignore login requests
                            if (requestId) {
                                statusRequestPoller.pollOnRequestStatus(requestId, response.config, Date.now());
                            } else if (/client-logs/.test(url)) {
                                //Ignore requests to the client-log
                            } else {
                                $log.warn('Received POST/PUT/DELETE without a request id: ' + response.config.method + ' / ' + url);
                            }
                        }
                    }

                    return response;
                }
            };
        });
    });
})();
