describe('Action into words filter', () => {
    let theFilter, theDetailedFilter, actionRequiredTitleFilter, we;

    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });

    beforeEach(() => {
        j.$rootScope.cloudSyncUrl = {
            url: 'http://www.datafabric.io'
        };
        theFilter = j.$filter('actionIntoWords');
        theDetailedFilter = j.$filter('actionIntoWordsDetails');
        actionRequiredTitleFilter = j.$filter('actionRequiredTitle');
        we = {
            isHA: false,
            _cloudProviderName: 'AWS',
            name: 'WE_name',
            ontapClusterProperties: {licenseType: {name: 'current license name'}}
        };
    });

    it('should pluralize nicely', () => {
        const pluralize = {
            actionType: 'NeedsMoreDisks',
            parameters: {
                aggregateName: 'aggr1',
                numOfDisks: 1,
                diskSize: {size: 500, unit: 'GB'},
                providerVolumeType: 'st1'
            }
        };
        let result = theFilter(pluralize, we);
        expect(result).toEqual('An additional 500 GB of AWS capacity is required for aggregate <strong>aggr1</strong>.');

        result = theDetailedFilter(pluralize, we);
        expect(result).toEqual(
            'I approve for Cloud Manager to purchase 500 GB of AWS capacity in aggregate aggr1 (1 Throughput Optimized HDD (st1) disk).');

        pluralize.parameters.numOfDisks = 2;
        result = theFilter(pluralize, we);
        expect(result).toEqual('An additional 1000 GB of AWS capacity is required for aggregate <strong>aggr1</strong>.');

        result = theDetailedFilter(pluralize, we);
        expect(result).toEqual(
            'I approve for Cloud Manager to purchase 1000 GB of AWS capacity in aggregate aggr1 (2 Throughput Optimized HDD (st1) disks).');

        we.isHA = true;
        we._isAWS = true;
        result = theDetailedFilter(pluralize, we);
        expect(result).toEqual(
            'I approve for Cloud Manager to purchase 1000 GB of AWS capacity in aggregate aggr1 (2 Throughput Optimized HDD (st1) disks) per node.');
        we.isHA = false;
    });

    it('should produce right words for limit related actions', () => {
        const parameters = {
            aggregateName: 'aggr1',
            numOfDisks: 1,
            diskSize: {size: 500, unit: 'GB'},
            licenseExpiryDate: 1432637993000,
            serialNumber: 1234
        };

        const diskLimitExceededAction = {actionType: 'MoreDisksNeededButDiskLimitExceeded', parameters};
        expect(theFilter(diskLimitExceededAction, we)).toEqual('Moving a volume is necessary to avoid capacity issues');
        expect(theDetailedFilter(diskLimitExceededAction, we)).toEqual('Moving a volume is necessary to avoid capacity issues; however, Cloud Manager cannot provide recommendations to correct the issue. Refer to the Cloud Manager documentation to get help.');

        const vsaLimitExceededAction = {actionType: 'MoreDisksNeededButVsaLimitExceeded', parameters};
        expect(theFilter(vsaLimitExceededAction, we)).toEqual('An additional AWS capacity is required for aggregate <strong>aggr1</strong>, however Cloud Volumes ONTAP has reached its max capacity');
        expect(theDetailedFilter(vsaLimitExceededAction, we)).toEqual('Cloud Volumes ONTAP license limit prevents Cloud Manager from purchasing the required additional AWS capacity. Please refer to Cloud Manager documentation for recommended corrective actions.');

        const ShouldBeRemovedAction = {actionType: 'ShouldBeRemoved', parameters};
        expect(theFilter(ShouldBeRemovedAction, we)).toEqual('Saving opportunity identified: Cloud Volumes ONTAP has 500 GB of AWS capacity that is not being utilized and can be removed.');
        expect(theDetailedFilter(ShouldBeRemovedAction, we)).toEqual('I approve for Cloud Manager to remove 500 GB of unused AWS capacity currently available to Cloud Volumes ONTAP.');
    });

    it('should produce right words for license related actions', () => {
        const parameters = {
            licenseExpiryDate: 1432637993000,
            serialNumber: 1234,
            instances: [
                {
                    instanceName: 'WE_name-01',
                    instanceType: 'm4.xlarge',
                    instanceId: 'i-2'
                }
            ]
        };
        const expectedLicenseActionDetails = 'Use your Cloud Volumes ONTAP BYOL Serial Number when requesting renewal services:<br>1234<br><br>For information on how to renew your subscription, please visit:<br><a href="http://www.netapp.com/us/how-to-buy/index.aspx" target="_blank">http://www.netapp.com/us/how-to-buy/index.aspx</a>';

        const licenseWasSuspended = {actionType: 'licenseWasSuspended', parameters};
        expect(theFilter(licenseWasSuspended, we)).toEqual('Your BYOL instance has been suspended for working environment WE_name because your subscription expired on May 26, 2015. <br>Renew your subscription in order to regain access to your BYOL instance.');
        expect(theDetailedFilter(licenseWasSuspended, we)).toEqual(expectedLicenseActionDetails);

        const licenseGracePeriod = {actionType: 'licenseGracePeriod', parameters};
        expect(theFilter(licenseGracePeriod, we)).toEqual('Your BYOL subscription expired on May 26, 2015 for working environment WE_name. You are now in violation of the terms of your subscription. Renew your subscription immediately to prevent suspension of your BYOL instance.');
        expect(theDetailedFilter(licenseGracePeriod, we)).toEqual(expectedLicenseActionDetails);

        const licenseExpirationWarning = {actionType: 'licenseExpirationWarning', parameters};
        expect(theFilter(licenseExpirationWarning, we)).toEqual('Your BYOL subscription is set to expire on May 26, 2015 for working environment WE_name. Renew your subscription prior to May 26, 2015 to prevent suspension of your BYOL instance.');
        expect(theDetailedFilter(licenseExpirationWarning, we)).toEqual(expectedLicenseActionDetails);
    });

    it('should produce right words for encryption related actions', () => {
        const parameters = {
            licenseExpiryDate: 1432637993000,
            serialNumber: 1234,
            instances: [
                {
                    instanceName: 'WE_name-01',
                    instanceType: 'm4.xlarge',
                    instanceId: 'i-2'
                }
            ]
        };
        const RestrictedAggregatesCanBeFixedResult = theFilter({
            actionType: 'RestrictedAggregatesCanBeFixed',
            parameters
        }, we);
        expect(RestrictedAggregatesCanBeFixedResult).toEqual('Some of your data on working environment WE_name is unavailable because Cloud Volumes ONTAP could not connect to key managers. A connection to at least one key manager is now available. Restart Cloud Volumes ONTAP so it can connect to key managers to unlock the data.');

        const RestrictedAggregatesCannotBeFixedResult = theFilter({
            actionType: 'RestrictedAggregatesCannotBeFixed',
            parameters
        }, we);
        expect(RestrictedAggregatesCannotBeFixedResult).toEqual('Some of your data on working environment WE_name is unavailable because Cloud Volumes ONTAP could not connect to key managers. Make sure your key managers are available, and then restart Cloud Volumes ONTAP so it can connect to key managers to unlock the data.');
    });

    it('should return "unknown" for unknown', () => {
        expect(theFilter({actionType: 'BlaBla', parameters: {}})).toEqual('Unknown');
        expect(theDetailedFilter({actionType: 'BlaBla', parameters: {}}, we)).toEqual('Unknown');
    });

    it('should create action required titles', () => {
        expect(actionRequiredTitleFilter({actionType: 'ShouldBeRemoved'})).toEqual('Review & Delete');
        expect(actionRequiredTitleFilter({actionType: 'EmptyWorkingEnvironment'})).toEqual('Review & Delete');
        expect(actionRequiredTitleFilter({actionType: 'licenseWasSuspended'})).toEqual('License Renewal');
        expect(actionRequiredTitleFilter({actionType: 'NonCompliantConfig'})).toEqual('License or instance change');
        expect(actionRequiredTitleFilter({actionType: 'MoreDisksNeededButDiskLimitExceeded'})).toEqual('Capacity threshold exceeded');
    });

    it('should produce the correct words for NO_SINGLE_VOLUME case', () => {
        const action = {
            actionType: 'MoreDisksNeededButDiskLimitExceeded',
            parameters: {
                moreInfo: 'NO_SINGLE_VOLUME',
                aggregateName: 'aggr1'
            }
        };

        expect(theFilter(action, we)).toEqual('Moving one or more volumes is necessary to avoid capacity issues; however, Cloud Manager cannot perform this action for you.');
        expect(theDetailedFilter(action, we)).toEqual('Aggregate aggr1 has reached its capacity limit. You must identify one or more volumes to move out of this aggregate and then move them to another aggregate on this system. Refer to the Cloud Manager documentation to get help.');
    });

    it('should produce the correct words for INSTANCE_DISK_LIMIT_EXCEEDED case', () => {
        const action = {
            actionType: 'MoreDisksNeededButDiskLimitExceeded',
            parameters: {
                moreInfo: 'INSTANCE_DISK_LIMIT_EXCEEDED',
                aggregateName: 'aggr1'
            }
        };

        expect(theFilter(action, we)).toEqual('Moving a volume is necessary to avoid capacity issuesMoving a volume is necessary to avoid capacity issues; however, Cloud Manager cannot perform this action for you because the system has reached the disk limit.');
        expect(theDetailedFilter(action, we)).toEqual('Aggregate aggr1 has reached its capacity limit. You can either free space on the system by deleting or rearranging volumes, or you can move one or more volumes to another system. Refer to the Cloud Manager documentation to get help.');
    });

    describe('LicenseUsedCapacityWarning', () => {
        it('should produce the right words for upgradeable', () => {
            const action = {
                actionType: 'LicenseUsedCapacityWarning',
                parameters: {
                    licenseParameters: {
                        newLicenseType: {
                            name: 'Cloud Volumes ONTAP Premium',
                            'capacity_limit': '368TB'
                        }
                    }
                }
            };

            expect(theFilter(action, we)).toEqual('Used capacity for this Cloud Volumes ONTAP system has reached 90% of the license limit. To add additional capacity, Cloud Manager must change the Cloud Volumes ONTAP license.<br>Note: Deleting a volume does not have an immediate effect on used capacity. The capacity amount includes deleted volumes that remain in ONTAP’s recovery queue for 12 hours.');
            expect(theDetailedFilter(action, we)).toEqual('Cloud Manager can change the Cloud Volumes ONTAP license from current license name to Cloud Volumes ONTAP Premium, which increases the capacity limit to 368TB. This process restarts Cloud Volumes ONTAP, which means I/O is interrupted.');

            we.isHA = true;
            expect(theDetailedFilter(action, we)).toEqual('Cloud Manager can change the Cloud Volumes ONTAP license from current license name to Cloud Volumes ONTAP Premium, which increases the capacity limit to 368TB. This operation is nondisruptive. The HA pair continues to serve data.');
        });

        it('should produce the right words for non upgradeable', () => {
            const action = {
                actionType: 'LicenseUsedCapacityWarning',
                parameters: {}
            };

            expect(theFilter(action, we)).toEqual('Used capacity for this Cloud Volumes ONTAP system has reached 90% of the license limit.<br>Note: Deleting a volume does not have an immediate effect on used capacity. The capacity amount includes deleted volumes that remain in ONTAP’s recovery queue for 12 hours.');
            expect(theDetailedFilter(action, we)).toEqual('This system uses a license with the highest available capacity limit. To avoid capacity issues, you should move volumes to other Cloud Volumes ONTAP systems.');
        });
    });

    describe('NonCompliantConfig', () => {
        it('should produce the correct words for SIDL', () => {
            const action = {
                actionType: 'NonCompliantConfig',
                parameters: {
                    moreInfo: 'SIDL',
                    licenseParameters: {
                        newLicenseType: {
                            name: 'Cloud Volumes ONTAP Premium',
                            'capacity_limit': '368TB'
                        }
                    }
                }
            };

            expect(theFilter(action, we)).toEqual('The single instance data logging feature was enabled directly from Cloud Volumes ONTAP. This feature is not supported with the current instance type. To use this feature, change the instance type.');
            expect(theDetailedFilter(action, we)).toEqual('Cloud Manager can change the Cloud Volumes ONTAP license and instance. This process restarts Cloud Volumes ONTAP, which means I/O is interrupted.');

            we.isHA = true;
            expect(theDetailedFilter(action, we)).toEqual('Cloud Manager can change the Cloud Volumes ONTAP license and instance. This operation is nondisruptive. The HA pair continues to serve data.');
        });

        it('should produce the correct words for CAPACITY_TIERING', () => {
            const action = {
                actionType: 'NonCompliantConfig',
                parameters: {
                    moreInfo: 'CAPACITY_TIERING',
                    licenseParameters: {
                        newLicenseType: {
                            name: 'Cloud Volumes ONTAP Premium',
                            'capacity_limit': '368TB'
                        }
                    }
                }
            };

            expect(theFilter(action, we)).toEqual('A tiered storage configuration was enabled directly from Cloud Volumes ONTAP. This feature is not supported with the current instance type. To use this feature, change the instance type.');
        });

        it('should produce the correct words for both SIDL CAPACITY_TIERING', () => {
            const action = {
                actionType: 'NonCompliantConfig',
                parameters: {
                    moreInfo: 'SIDL,CAPACITY_TIERING',
                    licenseParameters: {
                        newLicenseType: {
                            name: 'Cloud Volumes ONTAP Premium',
                            'capacity_limit': '368TB'
                        }
                    }
                }
            };

            expect(theFilter(action, we)).toEqual('The following features were enabled directly from Cloud Volumes ONTAP: The single instance data logging feature and a tiered storage configuration. These features are not supported with the current instance type. To use these features, change the instance type.');
        });
    });

    describe('Cloud Sync actions', () => {
        beforeEach(() => {
            j.$rootScope.about = {environment: 'production'};
            advanceTo(moment('2016-12-01').toDate());
        });

        it('should produce the right words for CloudSyncTrialReminder', () => {
            const action = {
                actionType: 'CloudSyncTrialReminder',
                parameters: {licenseExpiryDate: moment('2016-12-21')}
            };

            expect(theFilter(action, we)).toEqual('Your Cloud Sync service free trial period expires in 20 days.<br>To continue using this service after the 14-day free trial ends, see your options in <a href="http://www.datafabric.io/license" target="_blank">Cloud Sync License Settings</a>.');
        });

        it('should produce the right words for CloudSyncTrialWarning', () => {
            const action = {
                actionType: 'CloudSyncTrialWarning',
                parameters: {licenseExpiryDate: moment('2016-12-03')}
            };

            expect(theFilter(action, we)).toEqual('Your Cloud Sync service free trial period expires in 2 days.<br>To continue using this service after the 14-day free trial ends, see your options in <a href="http://www.datafabric.io/license" target="_blank">Cloud Sync License Settings</a>.');
        });

        it('should produce the right words for CloudSyncTrialExpired', () => {
            const action = {
                actionType: 'CloudSyncTrialExpired',
                parameters: {licenseExpiryDate: moment('2016-11-26')}
            };

            expect(theFilter(action, we)).toEqual('Your Cloud Sync service free trial period expired 5 days ago.<br>To continue using this service after the 14-day free trial ends, see your options in <a href="http://www.datafabric.io/license" target="_blank">Cloud Sync License Settings</a>.');
        });

        it('should produce the right words for CloudSyncLicenseReminder', () => {
            const action = {
                actionType: 'CloudSyncLicenseReminder',
                parameters: {
                    licenseExpiryDate: moment('2016-12-21'),
                    serialNumber: 88888888
                }
            };

            expect(theFilter(action, we)).toEqual('Your Cloud Sync service BYOL license (serial number 88888888) expires in 20 days.<br> To continue using this service, see your options in <a href="http://www.datafabric.io/license" target="_blank">Cloud Sync License Settings</a>.');
        });

        it('should produce the right words for CloudSyncLicenseWarning', () => {
            const action = {
                actionType: 'CloudSyncLicenseWarning',
                parameters: {
                    licenseExpiryDate: moment('2016-12-03'),
                    serialNumber: 88888888
                }
            };

            expect(theFilter(action, we)).toEqual('Your Cloud Sync service BYOL license (serial number 88888888) expires in 2 days.<br> To continue using this service, see your options in <a href="http://www.datafabric.io/license" target="_blank">Cloud Sync License Settings</a>.');
        });

        it('should produce the right words for CloudSyncLicenseExpired', () => {
            const action = {
                actionType: 'CloudSyncLicenseExpired',
                parameters: {
                    licenseExpiryDate: moment('2016-11-26'),
                    serialNumber: 88888888
                }
            };

            expect(theFilter(action, we)).toEqual('Your Cloud Sync service BYOL license (serial number 88888888) expired 5 days ago.<br> To continue using this service, see your options in <a href="http://www.datafabric.io/license" target="_blank">Cloud Sync License Settings</a>.');
        });

        it('should produce the right words for CloudSyncLicenseExpired without serial number', () => {
            const action = {
                actionType: 'CloudSyncLicenseExpired',
                parameters: {licenseExpiryDate: moment('2016-11-26')}
            };

            expect(theFilter(action, we)).toEqual('Your Cloud Sync service BYOL license expired 5 days ago.<br> To continue using this service, see your options in <a href="http://www.datafabric.io/license" target="_blank">Cloud Sync License Settings</a>.');
        });

        it('should route to dev environment of cloud sync for staging or testing env of OCCM', () => {
            j.$rootScope.about.environment = 'staging';
            const action = {
                actionType: 'CloudSyncLicenseExpired',
                parameters: {
                    licenseExpiryDate: moment('2016-11-26'),
                    serialNumber: 88888888
                }
            };

            expect(theFilter(action, we)).toEqual('Your Cloud Sync service BYOL license (serial number 88888888) expired 5 days ago.<br> To continue using this service, see your options in <a href="http://www.datafabric.io/license" target="_blank">Cloud Sync License Settings</a>.');
        });
    });
});
