describe('Normalized Capacity Filter', () => {
    let theFilter;
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    beforeEach(() => {
        theFilter = j.$filter('normalizedCapacity');
    });

    it('should round down to the closest whole number when wanted to', () => {
        const capacityObject = {size: 2048, unit: 'GB'};
        expect(theFilter(capacityObject)).toEqual('2 TB');
    });

    it('should hide zero decimal digits like a boss', () => {
        expect(theFilter({size: 1000.01, unit: 'GB'}, 1, true)).toEqual('1,000 GB');
        expect(theFilter({size: 1000.002, unit: 'GB'}, 2, true)).toEqual('1,000 GB');
        expect(theFilter({size: 1001.11, unit: 'GB'}, 1, true)).toEqual('1,001.1 GB');
        expect(theFilter({size: 1001.11, unit: 'GB'}, 0, true)).toEqual('1,001 GB');
        expect(theFilter({size: 1001.11, unit: 'GB'}, 0, false)).toEqual('1,001 GB');
        expect(theFilter({size: 1001, unit: 'GB'}, 0, false)).toEqual('1,001 GB');
        expect(theFilter({size: 1001, unit: 'GB'}, 0, true)).toEqual('1,001 GB');
        expect(theFilter({size: 1001.901, unit: 'GB'}, 3, false)).toEqual('1,001.901 GB');
        expect(theFilter({size: 1001.901, unit: 'GB'}, 3, true)).toEqual('1,001.901 GB');
        expect(theFilter({size: 0.001, unit: 'Byte'}, 3, true)).toEqual('0.001 Byte');
    });

    it('should not add the unit to the resulting string when it does not need to, like a boss of course', () => {
        expect(theFilter({size: 1000.01, unit: 'GB'}, 1, true, 'GB', true)).toEqual('1,000');
    });

    it('should not round up', () => {
        const capacityObject = {size: 1022, unit: 'GB'};
        expect(theFilter(capacityObject)).toEqual('1,022 GB');
    });

    it('should handle any kind of unit', () => {
        const capacityObject = {size: 1.22, unit: 'GB'};
        expect(theFilter(capacityObject)).toEqual('1 GB');
        expect(theFilter(capacityObject, 2)).toEqual('1.22 GB');
        expect(theFilter({size: 1024, unit: 'MB'})).toEqual('1 GB');
        expect(theFilter({size: 100, unit: 'GB'})).toEqual('100 GB');
        expect(theFilter({size: 500, unit: 'GB'})).toEqual('500 GB');
        expect(theFilter({size: 1, unit: 'TB'})).toEqual('1 TB');
    });

    it('should be able to round number according to input', () => {
        const capacityObject = {size: 5000, unit: 'GB'};
        expect(theFilter(capacityObject)).toEqual('5 TB');
        expect(theFilter(capacityObject, 2)).toEqual('4.88 TB');
    });

    it('should return zero for zero', () => {
        const capacityObject = {size: 0, unit: 'GB'};
        expect(theFilter(capacityObject, 2, true, 'TB')).toEqual('0 TB');
        expect(theFilter(capacityObject, 2, false, 'TB')).toEqual('0.00 TB');
        expect(theFilter(capacityObject, 2, true, 'GB')).toEqual('0 GB');
    });

    it('should return "< 0.0*1 [Unit]" for small capacities', () => {
        let capacityObject = {size: 1, unit: 'GB'};
        expect(theFilter(capacityObject, 0, true, 'TB')).toEqual('< 1 TB');
        expect(theFilter(capacityObject, 1, true, 'TB')).toEqual('< 0.1 TB');
        expect(theFilter(capacityObject, 2, true, 'TB')).toEqual('< 0.01 TB');
        expect(theFilter(capacityObject, 3, true, 'TB')).toEqual('< 0.001 TB');
        expect(theFilter(capacityObject, 4, true, 'TB')).toEqual('0.0010 TB');
        capacityObject = {size: 1, unit: 'MB'};
        expect(theFilter(capacityObject, 1, true, 'GB')).toEqual('< 0.1 GB');
        expect(theFilter(capacityObject, 2, true, 'GB')).toEqual('< 0.01 GB');
        expect(theFilter(capacityObject, 3, true, 'GB')).toEqual('< 0.001 GB');
        expect(theFilter(capacityObject, 4, true, 'GB')).toEqual('0.0010 GB');
        expect(theFilter({size: 536870912, unit: 'Byte'}, 4, true, 'TB')).toEqual('0.0005 TB');
        expect(theFilter({size: 536870912, unit: 'Byte'}, 2, true, 'TB')).toEqual('< 0.01 TB');

        expect(theFilter({size: 0.001, unit: 'Byte'}, 2, false)).toEqual('< 0.01 Byte');
        expect(theFilter({size: 0.001, unit: 'Byte'}, 2, true)).toEqual('< 0.01 Byte');
    });

    it('should convert correctly between units', () => {
        expect(theFilter({size: 1, unit: 'MB'}, 2, true, 'TB')).toEqual('< 0.01 TB');
        expect(theFilter({size: 1, unit: 'TB'}, 2, true, 'MB')).toEqual('1,048,576 MB');
        expect(theFilter({size: 536870912, unit: 'Byte'}, 2, true, 'MB')).toEqual('512 MB');
        expect(theFilter({size: 536870912, unit: 'Byte'}, 2, true, 'GB')).toEqual('0.50 GB');
    });

    it('should act normally for capacities large enough to be displayed with requested decimal digits', () => {
        const capacityObject = {size: 11, unit: 'GB'}; // 10 GB < 0.01 TB < 11 GB
        expect(theFilter(capacityObject, 2, true, 'TB')).toEqual('0.01 TB');
        expect(theFilter(capacityObject, 2, true, 'GB')).toEqual('11 GB');
        expect(theFilter({size: 512, unit: 'GB'}, 2, true, 'TB')).toEqual('0.50 TB');
        expect(theFilter({size: 10, unit: 'GB'}, 3, true, 'TB')).toEqual('0.010 TB');
        expect(theFilter({size: 1, unit: 'GB'}, 3, true, 'MB')).toEqual('1,024 MB');
    });

    it('should treat no input or invalid input as 0, default unit KB', () => {
        expect(theFilter()).toEqual('0 KB');
        expect(theFilter({size: 'lulu', unit: 'GB'}, 2, true, 'TB')).toEqual('0 TB');
        expect(theFilter({unit: 'GB'}, 2, true, 'MB')).toEqual('0 MB');
    });
});

describe('removeDecimalZeros', () => {
    let theFilter;
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    beforeEach(() => {
        theFilter = j.$filter('removeDecimalZeros');
    });
    it('should remove decimal zeros', () => {
        expect(theFilter('3.00')).toEqual('3');
        expect(theFilter('234')).toEqual('234');
    });

    it('should not remove non zeros', () => {
        expect(theFilter('3.14')).toEqual('3.14');
        expect(theFilter('3.000000000014')).toEqual('3.000000000014');
    });
});

describe('Gallery capacity', () => {
    let theFilter;
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    beforeEach(() => {
        theFilter = j.$filter('galleryCapacity');
    });
    it('should turn 0 to < 0.01', () => {
        expect(theFilter(0)).toEqual('< 0.01');
    });

    it('should turn 0.001 to < 0.01', () => {
        expect(theFilter(0.001)).toEqual('< 0.01');
    });

    it('should turn 0.01 to  0.01', () => {
        expect(theFilter(0.01)).toEqual('0.01');
    });

    it('should turn 2.2321 to 2.23', () => {
        expect(theFilter(2.2321)).toEqual('2.23');
    });

    it('should not show decimal digits if they are zero, 2.0021 turns to 2', () => {
        expect(theFilter(2.0021)).toEqual('2');
    });

    it('should not show decimal digits if they are zero, 2.00 turns to 2', () => {
        expect(theFilter(2.00)).toEqual('2');
    });

    it('should show a number bigger than 1000 correctly', () => {
        expect(theFilter(5500)).toEqual('5,500');
    });
});

describe('allocated aggregate capacity', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let theFilter, aggregates;

    beforeEach(() => {
        theFilter = j.$filter('allocatedAggregatesCapacity');
    });

    it('should get aggregates and return size object with the total raw size', () => {
        aggregates = [{providerVolumes: [{size: {size: 10, unit: 'TB'}}, {size: {size: 10, unit: 'TB'}}]}];
        expect(theFilter(aggregates)).toEqual({size: 21990232555520, unit: 'Byte'});

        aggregates = [{
            providerVolumes: [
                {size: {size: 10, unit: 'TB'}},
                {size: {size: 10, unit: 'TB'}}
            ]
        }, {
            providerVolumes: [
                {size: {size: 10, unit: 'TB'}},
                {size: {size: 10, unit: 'TB'}}
            ]
        }];
        expect(theFilter(aggregates)).toEqual({size: 43980465111040, unit: 'Byte'});
    });

    it('should return 0 bytes for empty inputs', () => {
        aggregates = null;
        expect(theFilter(aggregates)).toEqual({size: 0, unit: 'Byte'});

        aggregates = [];
        expect(theFilter(aggregates)).toEqual({size: 0, unit: 'Byte'});
    });
});

describe('volumePermissionToString', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let theFilter;

    beforeEach(() => {
        theFilter = j.$filter('volumePermissionToString');
    });

    it('should return the right string for each permission', () => {
        expect(theFilter('full_control')).toEqual('Full Control');
        expect(theFilter('change')).toEqual('Read/Write');
        expect(theFilter('read')).toEqual('Read');
        expect(theFilter('no_access')).toEqual('No Access');
    });

    it('should return the original string for unknown', () => {
        expect(theFilter('new_permission')).toEqual('new_permission');
    });
});
