describe('MetadataResource', () => {
    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    let MetadataResource;

    beforeEach(inject((_MetadataResource_) => {
        MetadataResource = _MetadataResource_;
    }));

    describe('stringifySelectedRouteTables', () => {
        it('should stringify only the selected', () => {
            const routeTables = [
                {id: 'id1', name: 'name1', selected: false},
                {id: 'id2', name: 'name2', selected: true}
            ];
            expect(MetadataResource.stringifySelectedRouteTables(routeTables)).toEqual('name2 (id2)');
        });

        it('should work without name', () => {
            const routeTables = [
                {id: 'id1', selected: false},
                {id: 'id2', selected: true}
            ];
            expect(MetadataResource.stringifySelectedRouteTables(routeTables)).toEqual('id2');
        });

        it('should handle multiple selection', () => {
            const routeTables = [
                {id: 'id1', selected: false},
                {id: 'id2', selected: true},
                {id: 'id3', selected: false, name: 'name3'},
                {id: 'id4', selected: true, name: 'name4'}
            ];
            expect(MetadataResource.stringifySelectedRouteTables(routeTables)).toEqual('id2, name4 (id4)');
        });
    });

    describe('getRouteTables', () => {
        it('success case: should add expanded: false, should sort tags by key, should add name parameter', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when([
                {tags: null},
                {tags: [{key: 'kuki', value: 'lala'}, {key: 'dada', value: 'nana'}]},
                {tags: [{key: 'Name', value: 'Name1'}]}
            ]));

            expect(MetadataResource.getRouteTables(j.nocHttp, 'region1', 'vpc1')).toBeResolvedWith([
                {tags: [], 'expanded': false},
                {tags: [{key: 'dada', value: 'nana'}, {key: 'kuki', value: 'lala'}], expanded: false},
                {tags: [{key: 'Name', value: 'Name1'}], name: 'Name1', expanded: false}
            ]);

            expect(j.nocHttp.get).toHaveBeenCalledWith('/aws/ha/metadata/route-tables?region=region1&vpcId=vpc1', 'retrieving route-table', j.nocHttp.LOADING);
        });

        it('error case', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.reject(j.errorResponse));

            expect(MetadataResource.getRouteTables(j.nocHttp, 'region1', 'vpc1')).toBeRejectedWith(j.errorResponse);

            expect(j.nocHttp.get).toHaveBeenCalledWith('/aws/ha/metadata/route-tables?region=region1&vpcId=vpc1', 'retrieving route-table', j.nocHttp.LOADING);
        });
    });

    describe('getKeyPairs', () => {
        it('should return the keyPairs for single node upon success', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when(['keyPair1', 'keyPair2']));

            expect(MetadataResource.getKeyPairs(j.nocHttp, j.pidgeon, 'region1', true)).toBeResolvedWith(['keyPair1', 'keyPair2']);

            expect(j.nocHttp.get).toHaveBeenCalledWith('/vsa/metadata/key-pairs?region=region1', 'retrieving Key Pairs', j.nocHttp.LOADING);
        });

        it('should return the keyPairs for HA upon success', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when(['keyPair1', 'keyPair2']));

            expect(MetadataResource.getKeyPairs(j.nocHttp, j.pidgeon, 'region1', false)).toBeResolvedWith(['keyPair1', 'keyPair2']);

            expect(j.nocHttp.get).toHaveBeenCalledWith('/aws/ha/metadata/key-pairs?region=region1', 'retrieving Key Pairs', j.nocHttp.LOADING);
        });

        it('should pidgeon error when no keyPairs available', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when([]));

            expect(MetadataResource.getKeyPairs(j.nocHttp, j.pidgeon, 'region1', false)).toBeRejectedWith('No key pairs are available in this region. Please add Key Pair or try another region.');

            expect(j.pidgeon.error).toHaveBeenCalledWith('No key pairs are available in this region. Please add Key Pair or try another region.', null, 'getKeyPairs');
        });

        afterEach(() => {
            expect(j.pidgeon.clear).toHaveBeenCalledWith('getKeyPairs');
        });
    });

    describe('getPackages', () => {
        const packages = [
            {
                name: 'aws_poc',
                diskType: 'gp2',
                diskSize: {size: 500, unit: 'GB'},
                capacityTier: null
            },
            {
                name: 'aws_standard',
                diskType: 'gp2',
                diskSize: {size: 1, unit: 'TB'},
                capacityTier: null
            },
            {
                name: 'aws_dr',
                diskType: 'st1',
                diskSize: {size: 1, unit: 'TB'},
                capacityTier: 'S3'
            },
            {
                name: 'aws_fastest',
                diskType: 'gp2',
                diskSize: {size: 8, unit: 'TB'},
                capacityTier: null
            }
        ];

        it('should get the package and initialize them correctly', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when(packages));

            expect(MetadataResource.getPackages(j.nocHttp, 'VSA', 'AWS')).toBeResolvedWith([{
                name: 'aws_poc',
                diskType: 'gp2',
                diskSize: {size: 500, unit: 'GB', _identifier: '500 GB'},
                capacityTier: null,
                _iconName: 'light-bulb'
            }, {
                name: 'aws_standard',
                diskType: 'gp2',
                diskSize: {size: 1, unit: 'TB', _identifier: '1 TB'},
                capacityTier: null,
                _iconName: 'volume-star'
            }, {
                name: 'aws_dr',
                diskType: 'st1',
                diskSize: {size: 1, unit: 'TB', _identifier: '1 TB'},
                capacityTier: 'S3',
                _iconName: 'low-cost'
            }, {
                name: 'aws_fastest',
                diskType: 'gp2',
                diskSize: {size: 8, unit: 'TB', _identifier: '8 TB'},
                capacityTier: null,
                _iconName: 'rocket'
            }]);

            expect(j.nocHttp.get).toHaveBeenCalledWith('/vsa/metadata/packages', 'retrieving packages', j.nocHttp.LOADING);
        });

        it('error case', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.reject(j.errorResponse));

            expect(MetadataResource.getPackages(j.nocHttp, 'VSA', 'AWS')).toBeRejectedWith(j.errorResponse);

            expect(j.nocHttp.get).toHaveBeenCalledWith('/vsa/metadata/packages', 'retrieving packages', j.nocHttp.LOADING);
        });
    });

    describe('getInstanceProfiles', () => {
        const instanceProfiles = [
            {
                name: 'instanceProfileName1',
                instanceProfileId: 'instanceProfileName1'
            },
            {
                name: 'instanceProfileName2',
                instanceProfileId: 'instanceProfileName2'
            }
        ];

        it('should get the instanceProfiles and initialize them correctly - VSA', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when(instanceProfiles));

            expect(MetadataResource.getInstanceProfiles(j.nocHttp, '/vsa/')).toBeResolvedWith([
                {
                    name: 'instanceProfileName1',
                    instanceProfileId: 'instanceProfileName1'
                },
                {
                    name: 'instanceProfileName2',
                    instanceProfileId: 'instanceProfileName2'
                }
            ]);

            expect(j.nocHttp.get).toHaveBeenCalledWith('/vsa/metadata/instance-profiles', 'retrieving instance-profiles');
        });

        it('should get the instanceProfiles and initialize them correctly - HA', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when(instanceProfiles));

            expect(MetadataResource.getInstanceProfiles(j.nocHttp, '/aws/ha/')).toBeResolvedWith([
                {
                    name: 'instanceProfileName1',
                    instanceProfileId: 'instanceProfileName1'
                },
                {
                    name: 'instanceProfileName2',
                    instanceProfileId: 'instanceProfileName2'
                }
            ]);

            expect(j.nocHttp.get).toHaveBeenCalledWith('/aws/ha/metadata/instance-profiles', 'retrieving instance-profiles');
        });
    });

    describe('getRegions', () => {
        const regions = ['region1', 'region2'];

        it('should get the regions upon success', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when(regions));

            expect(MetadataResource.getRegions(j.nocHttp, '/vsa/', 'withVnets', '', 'cloudProviderAccountId')).toBeResolvedWith(regions);

            expect(j.nocHttp.get).toHaveBeenCalledWith('/vsa/metadata/regions?fields=vnets&cloudProviderAccountId=cloudProviderAccountId', 'retrieving regions', j.nocHttp.BLOCK);
        });

        it('should get the regions with vnets if requested', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when(regions));

            expect(MetadataResource.getRegions(j.nocHttp, '/azure/vsa/', true, 'subscriptionId=1111')).toBeResolvedWith(regions);

            expect(j.nocHttp.get).toHaveBeenCalledWith('/azure/vsa/metadata/regions?fields=vnets&subscriptionId=1111&cloudProviderAccountId=undefined', 'retrieving regions', j.nocHttp.BLOCK);
        });

        it('error case', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.reject(j.errorResponse));

            expect(MetadataResource.getRegions(j.nocHttp, '/vsa/', false, '')).toBeRejectedWith(j.errorResponse);

            expect(j.nocHttp.get).toHaveBeenCalledWith('/vsa/metadata/regions?cloudProviderAccountId=undefined', 'retrieving regions', j.nocHttp.BLOCK);
        });
    });

    describe('getNetworkRequirements', () => {
        const response = {
            haNodeMinimumRequiredIps: 1,
            haMediatorMinimumRequiredIps: 2,
            nonHaNodeMinimumRequiredIps: 3
        };

        it('should work upon success for HA with failoverMode PrivateIP', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when(response));

            expect(MetadataResource.getNetworkRequirements(j.nocHttp, '/aws/ha/', 'HA', 'PrivateIP')).toBeResolvedWith({node: 4});

            expect(j.nocHttp.get).toHaveBeenCalledWith('/aws/ha/metadata/network-requirements', 'retrieving network requirements', j.nocHttp.BLOCK);
        });

        it('should work upon success for HA with failoverMode FloatingIP', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when(response));

            expect(MetadataResource.getNetworkRequirements(j.nocHttp, '/aws/ha/', 'HA', 'FloatingIP'))
                .toBeResolvedWith({node: 1, mediator: 2});

            expect(j.nocHttp.get).toHaveBeenCalledWith('/aws/ha/metadata/network-requirements', 'retrieving network requirements', j.nocHttp.BLOCK);
        });

        it('should work upon success for non-HA aws vsa', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when(response));

            expect(MetadataResource.getNetworkRequirements(j.nocHttp, '/vsa/', 'VSA')).toBeResolvedWith({node: 3});

            expect(j.nocHttp.get).toHaveBeenCalledWith('/vsa/metadata/network-requirements', 'retrieving network requirements', j.nocHttp.BLOCK);
        });

        it('should work upon success for azure', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when({vsaMinimumRequiredIps: 6}));

            expect(MetadataResource.getNetworkRequirements(j.nocHttp, '/azure/vsa/', 'VSA')).toBeResolvedWith({node: 6});

            expect(j.nocHttp.get).toHaveBeenCalledWith('/azure/vsa/metadata/network-requirements', 'retrieving network requirements', j.nocHttp.BLOCK);
        });

        it('should work upon error', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.reject(j.errorResponse));

            expect(MetadataResource.getNetworkRequirements(j.nocHttp, '/vsa/', 'VSA')).toBeRejectedWith(j.errorResponse);

            expect(j.nocHttp.get).toHaveBeenCalledWith('/vsa/metadata/network-requirements', 'retrieving network requirements', j.nocHttp.BLOCK);
        });
    });
});
