describe('volumesResource', () => {
    'use strict';
    let j;
    let volumesResource;
    let volumes;

    beforeEach(() => {
        j = magnifispec.setup();
    });

    const fakeInit = (volumes) => {
        return _.map(volumes, (vol) => {
            vol.initialized = true;

            return vol;
        });
    };

    beforeEach(inject((_volumesResource_) => {
        volumesResource = _volumesResource_;
        volumes = [
            {
                name: 'csg_ag_quorum',
                svmName: 'sv5-csg-01',
                size: {size: 5.0, unit: 'GB'},
                usedSize: {size: 3, unit: 'GB'},
                capacityTierUsedSize: {size: 2, unit: 'GB'},
                junctionPath: '/csg_ag_quorum',
                compressionSpaceSaved: {size: 0.0, unit: 'GB'},
                deduplicationSpaceSaved: {size: 0.0, unit: 'GB'},
                thinProvisioning: false,
                snapshotPolicy: 'default',
                parentVolumeName: '',
                state: 'online',
                shareNames: ['share1', 'share2']
            }, {
                name: 'sdw_test',
                svmName: 'sv5-csg-01',
                size: {size: 5.0, unit: 'GB'},
                usedSize: {size: 3, unit: 'GB'},
                capacityTierUsedSize: {size: 3.1, unit: 'GB'},
                junctionPath: '',
                compressionSpaceSaved: {size: 0.0, unit: 'GB'},
                deduplicationSpaceSaved: {size: 0.0, unit: 'GB'},
                thinProvisioning: false,
                snapshotPolicy: 'none',
                parentVolumeName: '',
                state: 'online'
            }, {
                name: 'shuki',
                svmName: 'sv5-csg-02',
                size: {size: 50, unit: 'GB'},
                usedSize: {size: 30, unit: 'GB'},
                capacityTierUsedSize: undefined,
                junctionPath: '/shuki',
                compressionSpaceSaved: {size: 0.0, unit: 'GB'},
                deduplicationSpaceSaved: {size: 0.0, unit: 'GB'},
                thinProvisioning: false,
                snapshotPolicy: 'default',
                parentVolumeName: '',
                state: 'online'
            }
        ];
    }));

    describe('startVolumesPolling', () => {
        let stopFunction, unloading;
        beforeEach(() => {
            unloading = 'unloading function';
            jest.spyOn(volumesResource, 'initVolumes');
            stopFunction = jest.fn();
            jest.spyOn(j.nocPoller, 'start').mockReturnValue(stopFunction);
        });

        it('should start the poller with the correct parameters', () => {
            const workingEnvironment = {
                publicId: 'the working environment',
                workingEnvironmentType: 'VSA',
                _pathPrefix: '/vsa/'
            };
            volumesResource.startVolumesPolling(workingEnvironment, unloading);

            expect(j.nocPoller.start).toHaveBeenCalled();
            const args = mostRecentCall(j.nocPoller.start);

            expect(args[0]).toEqual({
                method: 'GET',
                url: j.nocUtil.workingEnvironmentTypeToRequestPathPrefix(workingEnvironment.workingEnvironmentType, 'AWS') + 'volumes?workingEnvironmentId=' + workingEnvironment.publicId
            });
            expect(args[1]).toEqual(60 * 1000);

            jest.spyOn(j.$rootScope, '$broadcast').mockImplementation(() => {
            });

            args[2]([
                {
                    name: 'volume 1',
                    usedSize: {size: 0, unit: 'KB'},
                    size: {size: 1, unit: 'GB'},
                    capacityTierUsedSize: {size: 0, unit: 'KB'}
                },
                {
                    name: 'volume 2',
                    usedSize: {size: 0, unit: 'KB'},
                    size: {size: 1, unit: 'GB'},
                    capacityTierUsedSize: {size: 0, unit: 'KB'}
                }
            ]);
            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(volumesResource.VOLUMES_GET_EVENT, {
                workingEnvironment,
                volumes: [
                    {
                        name: 'volume 1',
                        usedSize: {size: 0, unit: 'KB'},
                        size: {size: 1, unit: 'GB'},
                        capacityTierUsedSize: {size: 0, unit: 'KB'},
                        _displaySize: '1 GB',
                        _displayUsedSize: '0 KB',
                        _displayUsedDiskSize: '0 KB',
                        _displayCapacityTierUsedSize: '0 KB',
                        _showCapacityTierUsedSizeInVolumesCards: false,
                        _shares: undefined,
                        _totalUsedBytesPercent: 0,
                        _diskUsedPercent: 0
                    },
                    {
                        name: 'volume 2',
                        usedSize: {size: 0, unit: 'KB'},
                        size: {size: 1, unit: 'GB'},
                        capacityTierUsedSize: {size: 0, unit: 'KB'},
                        _displaySize: '1 GB',
                        _displayUsedSize: '0 KB',
                        _displayUsedDiskSize: '0 KB',
                        _displayCapacityTierUsedSize: '0 KB',
                        _showCapacityTierUsedSizeInVolumesCards: false,
                        _shares: undefined,
                        _totalUsedBytesPercent: 0,
                        _diskUsedPercent: 0
                    }
                ],
                unloading: 'unloading function'
            });

            args[3]({message: 'error'});
            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(volumesResource.VOLUMES_GET_ERROR_EVENT, {
                workingEnvironment,
                error: {message: 'error'},
                unloading: 'unloading function'
            });
        });

        it('should start the poller with the correct parameters for ha', () => {
            const workingEnvironment = {
                publicId: 'the working environment',
                workingEnvironmentType: 'VSA',
                isHA: true,
                _pathPrefix: '/aws/ha/'
            };
            volumesResource.startVolumesPolling(workingEnvironment);

            const args = mostRecentCall(j.nocPoller.start);

            expect(args[0]).toEqual({
                method: 'GET',
                url: '/aws/ha/' + 'volumes?workingEnvironmentId=' + workingEnvironment.publicId
            });
        });

        it('should return a function which stops the poller when called', () => {
            const stopper = volumesResource.startVolumesPolling({workingEnvironmentType: 'VSA'});
            stopper();

            expect(stopFunction).toHaveBeenCalled();
        });
    });

    describe('refreshVolumesData', () => {
        const workingEnvironment = {publicId: 'id1', _pathPrefix: '/vsa/'};
        beforeEach(() => {
            jest.spyOn(volumesResource, 'initVolumes').mockImplementation(fakeInit);
            jest.spyOn(j.$rootScope, '$broadcast');
        });

        it('should retrieve the volumes and broadcast them initialized', () => {
            const volumesResponse = [{name: 'vol1'}, {name: 'vol2'}];
            jest.spyOn(j.nocHttp, 'get').mockImplementation(() => j.$q.when(volumesResponse));

            expect(volumesResource.refreshVolumesData(j.nocHttp, workingEnvironment)).toBeResolved();

            expect(volumesResource.initVolumes).toHaveBeenCalledWith(volumesResponse);
            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(volumesResource.VOLUMES_GET_EVENT, {
                workingEnvironment,
                volumes: [{name: 'vol1', initialized: true}, {name: 'vol2', initialized: true}],
                unloading: _.noop
            });
        });

        it('should fail in case of error', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.reject(j.errorResponse));
            expect(volumesResource.refreshVolumesData(j.nocHttp, workingEnvironment)).toBeResolved();

            expect(j.$rootScope.$broadcast).toHaveBeenCalledWith(volumesResource.VOLUMES_GET_ERROR_EVENT, {
                workingEnvironment,
                error: j.errorResponse,
                unloading: _.noop
            });
        });

        afterEach(() => {
            expect(j.nocHttp.get).toHaveBeenCalledWith({
                url: '/vsa/volumes?workingEnvironmentId=id1',
                headers: {'X-Occm-Bypass-Cache': true}
            }, null, j.nocHttp.LOADING);
        });
    });

    describe('getVolumes', () => {
        beforeEach(() => {
            jest.spyOn(volumesResource, 'initVolumes').mockImplementation(fakeInit);
        });

        it('should retrieve the volumes and initialize them', () => {
            const volumesResponse = [{name: 'vol1'}, {name: 'vol2'}];
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when(volumesResponse));

            expect(volumesResource.getVolumes(j.nocHttp, 'id1', '/vsa/')).toBeResolvedWith([
                {name: 'vol1', initialized: true},
                {name: 'vol2', initialized: true}
            ]);

            expect(volumesResource.initVolumes).toHaveBeenCalledWith(volumesResponse);
        });

        it('should fail in case of error', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.reject(j.errorResponse));
            expect(volumesResource.getVolumes(j.nocHttp, 'id1', '/vsa/')).toBeRejectedWith(j.errorResponse);
        });

        afterEach(() => {
            expect(j.nocHttp.get).toHaveBeenCalledWith({
                url: '/vsa/volumes',
                params: {workingEnvironmentId: 'id1'}
            }, 'retrieving volumes for working environment id1', j.nocHttp.LOADING);
        });
    });

    describe('initVolumes', () => {
        it('should normalize the sizes correctly', () => {
            const updatedVolumes = volumesResource.initVolumes(volumes);
            _.forEach(updatedVolumes, (volume) => {
                expect(volume._displaySize).toEqual(j.$filter('normalizedCapacity')(volume.size, 2, true));
                expect(volume._displayUsedSize).toEqual(j.$filter('normalizedCapacity')(volume.usedSize, 2, true));
                expect(volume._displayCapacityTierUsedSize).toEqual(j.$filter('normalizedCapacity')(volume.capacityTierUsedSize, 2, true));
            });

            expect(_.map(updatedVolumes, '_displayUsedDiskSize')).toEqual(['3 GB', '3 GB', '30 GB']);
            expect(_.map(updatedVolumes, '_shares')).toEqual(['share1, share2', undefined, undefined]);
        });

        it('should remove temp volumes', () => {
            const volumes = [
                {name: 'a', volumeType: 'dp', usedSize: {size: 0, unit: 'KB'}, capacityTierUsedSize: {size: 0, unit: 'KB'}},
                {name: 'b', volumeType: 'tmp', usedSize: {size: 0, unit: 'KB'}, capacityTierUsedSize: {size: 0, unit: 'KB'}},
                {name: 'c', volumeType: 'rw', usedSize: {size: 0, unit: 'KB'}, capacityTierUsedSize: {size: 0, unit: 'KB'}}
            ];

            expect(_.map(volumesResource.initVolumes(volumes), 'name')).toEqual(['a', 'c']);
        });

        it('should sort the volumes alphabetically (case insensitive)', () => {
            const volumes = [
                {name: 'b', usedSize: {size: 0, unit: 'KB'}, capacityTierUsedSize: {size: 0, unit: 'KB'}},
                {name: 'c', usedSize: {size: 0, unit: 'KB'}, capacityTierUsedSize: {size: 0, unit: 'KB'}},
                {name: 'Ba', usedSize: {size: 0, unit: 'KB'}, capacityTierUsedSize: {size: 0, unit: 'KB'}},
                {name: 'BB', usedSize: {size: 0, unit: 'KB'}, capacityTierUsedSize: {size: 0, unit: 'KB'}},
                {name: '1', usedSize: {size: 0, unit: 'KB'}, capacityTierUsedSize: {size: 0, unit: 'KB'}},
                {name: '2', usedSize: {size: 0, unit: 'KB'}, capacityTierUsedSize: {size: 0, unit: 'KB'}},
                {name: '10', usedSize: {size: 0, unit: 'KB'}, capacityTierUsedSize: {size: 0, unit: 'KB'}}
            ];
            expect(_.map(volumesResource.initVolumes(volumes), 'name')).toEqual(['1', '10', '2', 'b', 'Ba', 'BB', 'c']);
        });
    });

    describe('tier related tests', () => {
        it('should initialize tier related features for AWS volumes', () => {
            const volumesAws = [
                {name: 'a', tieringPolicy: 'none', providerVolumeType: 'gp2', capacityTier: null},
                {name: 'b', tieringPolicy: 'auto', providerVolumeType: 'gp2', capacityTier: 'S3'},
                {name: 'c', tieringPolicy: 'none', providerVolumeType: 'standard', capacityTier: null},
                {name: 'd', providerVolumeType: 'st1', capacityTier: null},
                {name: 'e', tieringPolicy: 'snapshot_only', providerVolumeType: 'st1', capacityTier: 'S3'},
                {name: 'f', providerVolumeType: 'sc1', capacityTier: null},
                {name: 'g', tieringPolicy: 'none', providerVolumeType: 'io1', capacityTier: null}
            ];

            const initializedVolumes = volumesResource.initVolumesForMonitoringDisplay(null, volumesAws, 'AWS');

            expect(_.map(initializedVolumes, '_iconName'))
                .toEqual(['volume-gp2', 'volume-gp2', 'volume-std', 'volume-st1', 'volume-st1', 'volume-sc1', 'volume-io1']);
            expect(_.map(initializedVolumes, '_tierDisplay')).toEqual([
                'General Purpose SSD (gp2)',
                'General Purpose SSD (gp2) and S3',
                'Previous generation HDD (standard)',
                'Throughput Optimized HDD (st1)',
                'Throughput Optimized HDD (st1) and S3',
                'Cold HDD (sc1)',
                'Provisioned IOPS SSD (io1)'
            ]);
            expect(_.map(initializedVolumes, '_diskTypeLetters')).toEqual(['gp2', 'gp2', 'std', 'st1', 'st1', 'sc1', 'io1']);
            expect(_.map(initializedVolumes, '_capacityTier')).toEqual(['gp2', 'gp2+S3', 'std', 'st1', 'st1+S3', 'sc1', 'io1']);
        });

        it('should initialize tier related features for Azure volumes', () => {
            const volumesAzure = [
                {name: 'a', providerVolumeType: 'Standard_LRS'},
                {name: 'b', providerVolumeType: 'Premium_LRS'}
            ];

            const initializedVolumes = volumesResource.initVolumesForMonitoringDisplay(null, volumesAzure, 'Azure');

            expect(_.map(initializedVolumes, '_iconName')).toEqual(['volume-std', 'volume-prm']);
            expect(_.map(initializedVolumes, '_tierDisplay')).toEqual([
                'Standard HDD',
                'Premium SSD'
            ]);
            expect(_.map(initializedVolumes, '_diskTypeLetters')).toEqual(['std', 'prm']);
            expect(_.map(initializedVolumes, '_capacityTier')).toEqual(['std', 'prm']);
        });

        it('should initialize tier related features for ON_PREM volumes', () => {
            const volumesOnPrem = [
                {name: 'a', providerVolumeType: undefined},
                {name: 'b', providerVolumeType: undefined, capacityTier: 'S3'}
            ];

            const initializedVolumes = volumesResource.initVolumesForMonitoringDisplay(null, volumesOnPrem, null);

            expect(_.map(initializedVolumes, '_iconName')).toEqual(['resource-volume', 'resource-volume']);
            expect(_.map(initializedVolumes, '_tierDisplay')).toEqual(['', '']);
            expect(_.map(initializedVolumes, '_diskTypeLetters')).toEqual(['', '']);
            expect(_.map(initializedVolumes, '_capacityTier')).toEqual(['', '']);
        });

        it('should set the correct icon and tooltip for volumes in disk type changing process', () => {
            const volumesAws = [
                {name: 'a', providerVolumeType: 'gp2', moving: true}
            ];

            const initializedVolumes = volumesResource.initVolumesForMonitoringDisplay(null, volumesAws, 'AWS');

            expect(_.map(initializedVolumes, '_iconName')).toEqual(['volume-move']);
            expect(_.map(initializedVolumes, '_tierDisplay')).toEqual(['Volume is moving']);
        });
    });

    describe('initVolumesForMonitoringDisplay', () => {
        describe('tier related tests', () => {
            it('should initialize tier related features for AWS volumes', () => {
                const volumesAws = [
                    {name: 'a', tieringPolicy: 'none', providerVolumeType: 'gp2', capacityTier: null},
                    {name: 'b', tieringPolicy: 'auto', providerVolumeType: 'gp2', capacityTier: 'S3'},
                    {name: 'c', tieringPolicy: 'none', providerVolumeType: 'standard', capacityTier: null},
                    {name: 'd', providerVolumeType: 'st1', capacityTier: null},
                    {name: 'e', tieringPolicy: 'snapshot_only', providerVolumeType: 'st1', capacityTier: 'S3'},
                    {name: 'f', providerVolumeType: 'sc1', capacityTier: null},
                    {name: 'g', tieringPolicy: 'none', providerVolumeType: 'io1', capacityTier: null}
                ];

                const initializedVolumes = volumesResource.initVolumesForMonitoringDisplay(null, volumesAws, 'AWS');

                expect(_.map(initializedVolumes, '_iconName'))
                    .toEqual(['volume-gp2', 'volume-gp2', 'volume-std', 'volume-st1', 'volume-st1', 'volume-sc1', 'volume-io1']);
                expect(_.map(initializedVolumes, '_tierDisplay')).toEqual([
                    'General Purpose SSD (gp2)',
                    'General Purpose SSD (gp2) and S3',
                    'Previous generation HDD (standard)',
                    'Throughput Optimized HDD (st1)',
                    'Throughput Optimized HDD (st1) and S3',
                    'Cold HDD (sc1)',
                    'Provisioned IOPS SSD (io1)'
                ]);
                expect(_.map(initializedVolumes, '_diskTypeLetters')).toEqual(['gp2', 'gp2', 'std', 'st1', 'st1', 'sc1', 'io1']);
                expect(_.map(initializedVolumes, '_capacityTier')).toEqual(['gp2', 'gp2+S3', 'std', 'st1', 'st1+S3', 'sc1', 'io1']);
            });

            it('should initialize tier related features for Azure volumes', () => {
                const volumesAzure = [
                    {name: 'a', providerVolumeType: 'Standard_LRS'},
                    {name: 'b', providerVolumeType: 'Premium_LRS'}
                ];

                const initializedVolumes = volumesResource.initVolumesForMonitoringDisplay(null, volumesAzure, 'Azure');

                expect(_.map(initializedVolumes, '_iconName')).toEqual(['volume-std', 'volume-prm']);
                expect(_.map(initializedVolumes, '_tierDisplay')).toEqual([
                    'Standard Storage (HDD)',
                    'Premium Storage (SSD)'
                ]);
                expect(_.map(initializedVolumes, '_diskTypeLetters')).toEqual(['std', 'prm']);
                expect(_.map(initializedVolumes, '_capacityTier')).toEqual(['std', 'prm']);
            });

            it('should initialize tier related features for ON_PREM volumes', () => {
                const volumesOnPrem = [
                    {name: 'a', providerVolumeType: undefined},
                    {name: 'b', providerVolumeType: undefined, capacityTier: 'S3'}
                ];

                const initializedVolumes = volumesResource.initVolumesForMonitoringDisplay(null, volumesOnPrem, null);

                expect(_.map(initializedVolumes, '_iconName')).toEqual(['resource-volume', 'resource-volume']);
                expect(_.map(initializedVolumes, '_tierDisplay')).toEqual(['', '']);
                expect(_.map(initializedVolumes, '_diskTypeLetters')).toEqual(['', '']);
                expect(_.map(initializedVolumes, '_capacityTier')).toEqual(['', '']);
            });

            it('should set the correct icon and tooltip for volumes in disk type changing process', () => {
                const volumesAws = [
                    {name: 'a', providerVolumeType: 'gp2', moving: true}
                ];

                const initializedVolumes = volumesResource.initVolumesForMonitoringDisplay(null, volumesAws, 'AWS');

                expect(_.map(initializedVolumes, '_iconName')).toEqual(['volume-move']);
                expect(_.map(initializedVolumes, '_tierDisplay')).toEqual(['Volume is moving']);
            });
        });

        describe('cloudSync related tests', () => {
            it('should set _isCloudSynced to false if cloudSyncProperties is not defined', () => {
                const initializedVolumes = volumesResource.initVolumesForMonitoringDisplay(undefined, volumes);

                _.forEach(initializedVolumes, function (volume) {
                    expect(volume._isCloudSynced).toBeFalsy();
                });
            });

            it('should set _isCloudSynced to false for not synced volumes in synced WE', () => {
                const initializedVolumes = volumesResource.initVolumesForMonitoringDisplay({isSynced: true}, volumes);

                _.forEach(initializedVolumes, function (volume) {
                    expect(volume._isCloudSynced).toBeFalsy();
                });

            });

            it('should have falsy _isCloudSynced status for volumes in not synced WE', () => {
                const initializedVolumes = volumesResource.initVolumesForMonitoringDisplay({isSynced: false}, volumes);

                _.forEach(initializedVolumes, function (volume) {
                    expect(volume._isCloudSynced).toBeFalsy();
                });
            });

            it('should set _isCloudSynced and _cloudSyncFailureMessages properties correctly (case data broker is not FAILED)', () => {
                const cloudSyncProperties = {
                    isSynced: true,
                    status: {status: 'COMPLETE'},
                    relationships: [
                        {
                            volumeName: 'csg_ag_quorum',
                            relationshipStatus: 'FAILED',
                            failureMessage: 'problem with this relationship'
                        },
                        {volumeName: 'sdw_test', relationshipStatus: 'ON'}
                    ]
                };

                const initializedVolumes = volumesResource.initVolumesForMonitoringDisplay(cloudSyncProperties, volumes);

                expect(_.map(initializedVolumes, '_isCloudSynced')).toEqual([true, true, undefined]);
                expect(_.map(initializedVolumes, '_cloudSyncFailureMessages')).toEqual([['Relationship status: failed', 'problem with this relationship'], null, undefined]);
            });

            it('should set _isCloudSynced and _cloudSyncFailureMessages properties correctly (case data broker is FAILED)', () => {
                const cloudSyncProperties = {
                    isSynced: true,
                    status: {status: 'FAILED', failureMessage: 'Data broker is failed'},
                    relationships: [
                        {
                            volumeName: 'csg_ag_quorum',
                            relationshipStatus: 'FAILED',
                            failureMessage: 'problem with this relationship'
                        },
                        {
                            volumeName: 'sdw_test',
                            relationshipStatus: 'VERY_GOOD'
                        }
                    ]
                };

                const initializedVolumes = volumesResource.initVolumesForMonitoringDisplay(cloudSyncProperties, volumes);

                expect(_.map(initializedVolumes, '_isCloudSynced')).toEqual([true, true, undefined]);
                expect(_.map(initializedVolumes, '_cloudSyncFailureMessages')).toEqual([
                    ['Data broker is failed', 'Relationship status: failed', 'problem with this relationship'],
                    ['Data broker is failed'],
                    undefined
                ]);
            });
        });
    });

    describe('getSnapshots', () => {
        it('should get the snapshots upon success and add correct display access time', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.when([
                {'name': 'snapshotVol1', 'accessTime': 111111111, 'label': 'daily'},
                {'name': 'snapshotVol2', 'accessTime': 222222222, 'label': 'weekly'}
            ]));

            expect(volumesResource.getSnapshots(j.nocHttp, '/vsa/', 'id1', 'svm1', 'vol1')).toBeResolvedWith([
                {
                    'name': 'snapshotVol1',
                    'accessTime': 111111111,
                    'label': 'daily',
                    'displayAccessTime': 'Jan 2, 1970 08:51 am'
                },
                {
                    'name': 'snapshotVol2',
                    'accessTime': 222222222,
                    'label': 'weekly',
                    'displayAccessTime': 'Jan 3, 1970 03:43 pm'
                }
            ]);
        });

        it('should fail upon error', () => {
            jest.spyOn(j.nocHttp, 'get').mockReturnValue(j.$q.reject(j.errorResponse));

            expect(volumesResource.getSnapshots(j.nocHttp, '/vsa/', 'id1', 'svm1', 'vol1')).toBeRejectedWith(j.errorResponse);
        });

        afterEach(() => {
            expect(j.nocHttp.get).toHaveBeenCalledWith('/vsa/volumes/id1/svm1/vol1/snapshots', 'retrieving Snapshots for volume vol1', j.nocHttp.LOADING);
        });
    });

    describe('askForQuote', () => {
        it('should call API correctly', () => {
            jest.spyOn(j.nocHttp, 'post').mockReturnValue(j.$q.when('quoteResponse'));

            expect(volumesResource.askForQuote(j.nocHttp, '/vsa/', 'quoteRequest', 'creating volume')).toBeResolvedWith('quoteResponse');
        });

        it('should fail upon error', () => {
            jest.spyOn(j.nocHttp, 'post').mockReturnValue(j.$q.reject(j.errorResponse));

            expect(volumesResource.askForQuote(j.nocHttp, '/vsa/', 'quoteRequest', 'creating volume')).toBeRejectedWith(j.errorResponse);
        });

        afterEach(() => {
            expect(j.nocHttp.post).toHaveBeenCalledWith({url: '/vsa/volumes/quote', data: 'quoteRequest'}, 'creating volume', j.nocHttp.BLOCK);
        });
    });

    describe('getUserApprovalForNewVolumeIfNeeded', () => {
        it('should resolve instantly for auto capacity management', () => {
            jest.spyOn(j.nocDialog, 'confirm').mockImplementation(() => {});
            const quoteResponse = {
                autoVsaCapacityManagement: true
            };

            expect(volumesResource.getUserApprovalForNewVolumeIfNeeded(quoteResponse, 'AWS', 'General Purpose SSD', false, 'vol1')).toBeResolvedWith(quoteResponse);
            expect(j.nocDialog.confirm).not.toHaveBeenCalled();
        });

        describe('in manual mode', () => {
            let nocApprovalMessageBuilderSpy;
            const quoteResponse = {
                autoVsaCapacityManagement: false,
                numOfDisks: 20
            };

            beforeEach(inject((nocApprovalMessageBuilder) => {
                nocApprovalMessageBuilderSpy = jest.spyOn(nocApprovalMessageBuilder, 'buildTitle').mockReturnValue('title');
            }));

            it('should display confirm dialog and resolve with quoteResponse if user approves', () => {
                magnifispec.mockConfirm(true);

                expect(volumesResource.getUserApprovalForNewVolumeIfNeeded(quoteResponse, 'AWS', 'General Purpose SSD', false, 'vol1')).toBeResolvedWith(quoteResponse);

                expect(nocApprovalMessageBuilderSpy).toHaveBeenCalledWith({
                    autoVsaCapacityManagement: false,
                    numOfDisks: 20,
                    provider: 'AWS',
                    diskType: 'General Purpose SSD',
                    isHA: false
                });
                expect(j.nocDialog.confirm).toHaveBeenCalledWith('', 'assets/templates/dialogs/add-volume-approval-content.html', {
                    html: 'title',
                    error: {
                        autoVsaCapacityManagement: false,
                        numOfDisks: 20,
                        provider: 'AWS',
                        diskType: 'General Purpose SSD',
                        isHA: false
                    },
                    volumeName: 'vol1',
                    hideHint: false,
                    show: false
                });
                expect(quoteResponse).toEqual({autoVsaCapacityManagement: false, numOfDisks: 20});
            });

            it('should not be resolved if user does not approve', () => {
                magnifispec.mockConfirm(false);

                expect(volumesResource.getUserApprovalForNewVolumeIfNeeded(quoteResponse, 'AWS', 'General Purpose SSD', false, 'vol1')).toBeRejected();
                expect(j.nocDialog.confirm).toHaveBeenCalled();
            });

            it('should display the correct number of disks for HA', () => {
                magnifispec.mockConfirm(false);

                expect(volumesResource.getUserApprovalForNewVolumeIfNeeded(quoteResponse, 'AWS', 'General Purpose SSD', true, 'vol1')).toBeRejected();

                expect(nocApprovalMessageBuilderSpy).toHaveBeenCalledWith(jasmine.objectContaining({numOfDisks: 10}));
                expect(quoteResponse).toEqual({autoVsaCapacityManagement: false, numOfDisks: 20});
            });
        });
    });
});
