describe('Config Service', () => {
    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    describe('Get Config', () => {
        const assertResponse = (message, params, expectedResponse) => {
            it(message, () => {
                const onSuccess = jest.fn();
                const onError = jest.fn();

                j.configService.getConfig(j.nocHttp, params)
                    .then(onSuccess)
                    .catch(onError);
                j.$httpBackend.flush();

                expect(onSuccess).toHaveBeenCalledWith(expectedResponse);
                expect(onError).not.toHaveBeenCalled();
            });
        };

        describe('handles response with multiple parameters', () => {
            const serverResponse = {
                a: '1',
                b: '2',
                c: '3'
            };
            beforeEach(() => {
                j.$httpBackend.expectGET(/config$/).respond(200, serverResponse);
            });

            assertResponse('asking for one of them', ['a'], {a: '1'});
            assertResponse('asking for all of them', ['a', 'b', 'c'], {a: '1', b: '2', c: '3'});
            assertResponse('asking for all of them different order', ['b', 'a', 'c'], {a: '1', b: '2', c: '3'});
            assertResponse('asking for none of them', [], {});
        });

        describe('handles response with a single parameter', () => {
            const serverResponse = {a: '1'};
            beforeEach(() => {
                j.$httpBackend.expectGET(/config$/).respond(200, serverResponse);
            });

            assertResponse('asking for one/all of them', ['a'], {a: '1'});
            assertResponse('asking for none of them', [], {});
        });

        describe('handles empty response from server', () => {
            const serverResponse = {};
            beforeEach(() => {
                j.$httpBackend.expectGET(/config$/).respond(200, serverResponse);
            });

            assertResponse('asking for none of them', [], {});
        });

        describe('handles error from server', () => {
            beforeEach(() => {
                j.$httpBackend.expectGET(/config$/).respond(400, j.errorResponse);
            });

            const assertException = (message, params) => {
                it(message, () => {
                    const onSucess = jest.fn();
                    const onError = jest.fn();

                    j.configService.getConfig(j.nocHttp, params)
                        .then(onSucess)
                        .catch(onError);

                    j.$httpBackend.flush();

                    expect(onSucess).not.toHaveBeenCalled();
                    expect(onError).toHaveBeenCalledWith(new j.NocErrorResponse({
                        message: 'generic failure message',
                        status: 400
                    }));
                });
            };

            assertException('asking for one', ['d']);
        });

        describe('throws error if parameter does not exist', () => {
            const serverResponse = {
                a: '1',
                b: '2',
                c: '3'
            };
            beforeEach(() => {
                j.$httpBackend.expectGET(/config$/).respond(200, serverResponse);
            });

            const assertException = (message, params) => {
                it(message, () => {
                    const onSucess = jest.fn();
                    const onError = jest.fn();

                    j.configService.getConfig(j.nocHttp, params)
                        .then(onSucess)
                        .catch(onError);

                    j.$httpBackend.flush();

                    expect(onSucess).not.toHaveBeenCalled();
                    expect(onError).toHaveBeenCalledWith(new j.NocErrorResponse({message: 'Some of the parameters \'' + params.join(', ') + '\' do not exist in the server response: ' + JSON.stringify(serverResponse)}));
                });
            };

            assertException('asking for one', ['d']);
            assertException('asking for several', ['d', 'e']);
            assertException('asking for one in addition to existing parameters', ['a', 'b', 'c', 'd']);
        });
    });

    describe('Update Config', () => {
        describe('should do nothing when nothing has changed', () => {
            const assertResponse = (message, params, newValues, originalValues) => {
                it(message, () => {
                    j.configService.updateConfig(j.nocHttp, params, newValues, originalValues);

                    j.$httpBackend.verifyNoOutstandingExpectation();
                    expect(newValues).toEqual(originalValues);
                });
            };
            assertResponse('updating nothing when nothing has changed', ['a', 'b'], {a: '1', b: '2'}, {a: '1', b: '2'});
        });

        describe('should update a single parameter if it has changed', () => {
            const assertResponse = (message, params, newValues, originalValues) => {
                it(message, () => {
                    j.configService.updateConfig(j.nocHttp, params, newValues, originalValues);
                    j.$httpBackend.flush();
                });
            };
            beforeEach(() => {
                j.$httpBackend.expectPUT(/config$/, {a: '7'}).respond(200);
            });
            assertResponse('updating a single parameter', ['a', 'b'], {a: '7', b: '2'}, {a: '1', b: '2'});
        });

        describe('should update all parameters if they have changed', () => {
            const assertResponse = (message, params, newValues, originalValues) => {
                it(message, () => {
                    j.configService.updateConfig(j.nocHttp, params, newValues, originalValues);
                    j.$httpBackend.flush();
                });
            };
            beforeEach(() => {
                j.$httpBackend.expectPUT(/config$/, {a: '7', b: '8'}).respond(200);
            });
            assertResponse('updating all changed parameters', ['a', 'b'], {a: '7', b: '8'}, {a: '1', b: '2'});
        });
    });
});
