describe("nocEncryptionResourcesService", function () {
    "use strict";

    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });

    describe("getOccmCertificate", function () {
        beforeEach(function () {
            jest.spyOn(j.nocUtil, "certificateExpirationState").mockReturnValue("mocked_state");
        });

        it("should work upon success", function () {
            jest.spyOn(j.nocEncryptionResourcesService, "formatCertificate").mockImplementation(_.noop);
            jest.spyOn(j.nocHttp, "get").mockReturnValue(j.$q.when({_pathPrefix: "/vsa/" ,publicId: "cert1", endDate: "tomorrow"}));

            expect(j.nocEncryptionResourcesService.getOccmCertificate(j.nocHttp))
                .toBeResolvedWith({_pathPrefix: "/vsa/", publicId: "cert1", endDate: "tomorrow"});

            expect(j.nocEncryptionResourcesService.formatCertificate)
                .toHaveBeenCalledWith({_pathPrefix: "/vsa/", publicId: "cert1", endDate: "tomorrow"});
        });

        it("should work upon success with no certificate", function () {
            jest.spyOn(j.nocHttp, "get").mockReturnValue(j.$q.when(null));

            expect(j.nocEncryptionResourcesService.getOccmCertificate(j.nocHttp)).toBeResolvedWith(null);
        });

        it("should work upon error", function () {
            jest.spyOn(j.nocHttp, "get").mockReturnValue(j.$q.reject(j.errorResponse));

            expect(j.nocEncryptionResourcesService.getOccmCertificate(j.nocHttp)).toBeRejectedWith(j.errorResponse);
        });

        afterEach(function () {
            expect(j.nocHttp.get).toHaveBeenCalledWith('/occm/encryption/certificate', 'retrieving Cloud Manager certificate', j.nocHttp.LOADING);
        });
    });

    describe("getKeyManagersCaCertificates", function () {
        beforeEach(function () {
            jest.spyOn(j.nocUtil, "certificateExpirationState").mockReturnValue("mocked_state");
        });
        it("should work upon success", function () {
            jest.spyOn(j.nocEncryptionResourcesService, "formatCertificate").mockImplementation(_.noop);
            j.$httpBackend.expectGET(j.nocBaseUrl + "/occm/encryption/key-managers-ca-certificates").respond(200, [{_pathPrefix:"/vsa/",publicId: "id1"}, {_pathPrefix:"/vsa/",publicId: "id2"}]);

            j.nocEncryptionResourcesService.getKeyManagersCaCertificates(j.nocHttp)
                .then(function (response) {
                    expect(response).toEqual([{_pathPrefix:"/vsa/",publicId: 'id1'}, {_pathPrefix:"/vsa/",publicId: 'id2'}]);
                });
            j.$httpBackend.flush();

            expect(j.nocEncryptionResourcesService.formatCertificate.mock.calls).toEqual([[{_pathPrefix:"/vsa/",publicId: 'id1'}], [{_pathPrefix:"/vsa/",publicId: 'id2'}]]);
        });

        it("should work upon error", function () {
            j.$httpBackend.expectGET(j.nocBaseUrl + "/occm/encryption/key-managers-ca-certificates").respond(400, j.errorResponse);

            var promise = j.nocEncryptionResourcesService.getKeyManagersCaCertificates(j.nocHttp);
            j.$httpBackend.flush();

            expect(promise).toBeRejectedWith(j.errorResponse);
        });
    });

    describe("getKeyManagersCaCertificatesWithOptions", function () {
        beforeEach(function () {
            jest.spyOn(j.nocEncryptionResourcesService, "formatKeyManagersCaCertificate").mockReturnValue("mockedFormattingFunction");
        });
        it("should work upon success", function () {
            jest.spyOn(j.nocEncryptionResourcesService, "getKeyManagersCaCertificates").mockImplementation(
                magnifispec.mockPromise([{_pathPrefix:"/vsa/",publicId: 'id1'}, {_pathPrefix:"/vsa/",publicId: 'id2'}])
            );

            j.nocEncryptionResourcesService.getKeyManagersCaCertificatesWithOptions()
                .then(function (response) {
                    expect(response).toEqual({
                        data: [{_pathPrefix:"/vsa/",publicId: 'id1'}, {_pathPrefix:"/vsa/",publicId: 'id2'}],
                        select2options: {
                            formatResult: "mockedFormattingFunction",
                            formatSelection: "mockedFormattingFunction"
                        }
                    });
                });
            magnifispec.processPromise();
        });

        it("should work upon error", function () {
            jest.spyOn(j.nocEncryptionResourcesService, "getKeyManagersCaCertificates").mockImplementation(
                magnifispec.mockPromise()
            );
            expect(j.nocEncryptionResourcesService.getKeyManagersCaCertificates())
                .toBeRejectedWith(j.errorResponse);
        });
    });

    describe("formatCertificate", function () {
        var certificate1;
        beforeEach(function () {
            certificate1 = {_pathPrefix:"/vsa/",publicId: 'id1', directIssuerName: "a,b,c", subjectName: "d,e,f", endDate: "date1"};
            jest.spyOn(j.nocUtil, "certificateExpirationState").mockReturnValue("mocked_state");
        });
        it("should calculate expiration state", function () {
            j.nocEncryptionResourcesService.formatCertificate(certificate1);
            expect(certificate1._expirationState).toEqual("mocked_state");
            expect(j.nocUtil.certificateExpirationState).toHaveBeenCalledWith("date1");
        });
        it("should add spaces after commas", function () {
            j.nocEncryptionResourcesService.formatCertificate(certificate1);
            expect(certificate1.directIssuerName).toEqual("a, b, c");
            expect(certificate1.subjectName).toEqual("d, e, f");
        });
        it("should work with no certificate", function () {
            j.nocEncryptionResourcesService.formatCertificate(undefined);
        });
    });

    describe("stringifyKeyManagerCaCertificate", function () {
        it("should return the common name from the subject string", function () {
            var cert = {"publicId":"pid1","subjectName":"C=US,ST=California,L=Mountain View,O=Google Inc,CN=www.google.com","endDate":1435017600000};
            expect(j.nocEncryptionResourcesService.stringifyKeyManagerCaCertificate(cert)).toEqual("www.google.com");
        });
        it("should return the common name from the subject string, even when there is something after it", function () {
            var cert = {"publicId":"pid1","subjectName":"C=US,ST=California,L=Mountain View,O=Google Inc,CN=www.google.com,LALALA=more content","endDate":1435017600000};
            expect(j.nocEncryptionResourcesService.stringifyKeyManagerCaCertificate(cert)).toEqual("www.google.com");
        });
    });

    describe("formatKeyManagersCaCertificate", function () {
        var keyManagersCaCertificates;
        beforeEach(function () {
            keyManagersCaCertificates = [
                {"publicId":"pid1", "subjectName":"C=US,ST=California,L=Mountain View,O=Google Inc,CN=www.google.com", "endDate":1435017600000},
                {"publicId":"pid2"}
            ];
            jest.spyOn(j.nocEncryptionResourcesService, "stringifyKeyManagerCaCertificate").mockReturnValue("COMMON_NAME");
        });
        it("should return nothing for nothing", function () {
            expect(j.nocEncryptionResourcesService.formatKeyManagersCaCertificate(keyManagersCaCertificates)({})).toEqual("");
        });

        it('should return the right code for a certificate with long expiry', () => {
            keyManagersCaCertificates[0]._expirationState = {state: 'certificate-valid-long', message: ''};
            expect(j.nocEncryptionResourcesService.formatKeyManagersCaCertificate(keyManagersCaCertificates)({text: 'pid1'}))
                .toEqual('<div class=\'select2-format-options clearfix\' title=\'C=US,ST=California,L=Mountain View,O=Google Inc,CN=www.google.com. \'>' +
                    '<span>COMMON_NAME</span>' +
                    '<div>' +
                    '<span class="certificate-valid-long">Expiration: June 23, 2015</span>' +
                    '</div>' +
                    '</div>');
        });

        it('should return the right code for near expiration', () => {
            keyManagersCaCertificates[0]._expirationState = {
                state: 'certificate-warning',
                message: 'Certificate will expire in less than one year.'
            };
            expect(j.nocEncryptionResourcesService.formatKeyManagersCaCertificate(keyManagersCaCertificates)({text: 'pid1'})).toEqual('<div class=\'select2-format-options clearfix\' title=\'C=US,ST=California,L=Mountain View,O=Google Inc,CN=www.google.com. Certificate will expire in less than one year.\'>' +
                '<span>COMMON_NAME</span>' +
                '<div>' +
                '<span class="certificate-warning">Expiration: June 23, 2015</span>' +
                '</div>' +
                '</div>');
        });

        it('should paint the expiration red if expired', () => {
            keyManagersCaCertificates[0]._expirationState = {
                state: 'certificate-expired',
                message: 'Certificate expired.'
            };
            expect(j.nocEncryptionResourcesService.formatKeyManagersCaCertificate(keyManagersCaCertificates)({text: 'pid1'})).toEqual('<div class=\'select2-format-options clearfix\' title=\'C=US,ST=California,L=Mountain View,O=Google Inc,CN=www.google.com. Certificate expired.\'>' +
                '<span>COMMON_NAME</span>' +
                '<div>' +
                '<span class="certificate-expired">Expiration: June 23, 2015</span>' +
                '</div>' +
                '</div>');
        });
    });

    describe("getKeyManagers", function () {
        it("should work upon success", function () {
            jest.spyOn(j.nocHttp, "get").mockReturnValue(j.$q.when('keyManager1'));

            expect(j.nocEncryptionResourcesService.getKeyManagers(j.nocHttp))
                .toBeResolvedWith('keyManager1');
        });

        it("should work upon error", function () {
            jest.spyOn(j.nocHttp, "get").mockReturnValue(j.$q.reject(j.errorResponse));

            expect(j.nocEncryptionResourcesService.getKeyManagers(j.nocHttp))
                .toBeRejectedWith(j.errorResponse);
        });
        afterEach(function () {
            expect(j.nocHttp.get).toHaveBeenCalledWith('/occm/encryption/key-managers', 'retrieving key managers', j.nocHttp.LOADING);
        });
    });

    describe('isKeyFromAnotherAccount', () => {
        it('should check if key is in arnKeyRegex pattern', () => {
            expect(j.nocEncryptionResourcesService.isKeyFromAnotherAccount({keyId: '1234'})).toBe(false);
            expect(j.nocEncryptionResourcesService.isKeyFromAnotherAccount({keyId: 'arn:aws:kms:1234'})).toBe(true);
        });
    });
});
