describe('nocPollerSpec', () => {
    'use strict';
    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let nocPoller;
    const pollUrl = 'kuki';
    const prefixedPollUrl = '/occm/api/' + pollUrl;
    beforeEach(inject((_nocPoller_) => {
        nocPoller = _nocPoller_;
    }));

    describe('Determining the frequency', () => {
        it('should return the local frequency if not nopoll', () => {
            nocPoller.nopoll = false;
            expect(nocPoller.determineFrequency(4000)).toEqual(4000);
        });

        it('should return 500000 if nopoll', () => {
            nocPoller.nopoll = true;
            expect(nocPoller.determineFrequency(4000)).toEqual(500000);
        });
    });

    describe('Constructing params object', () => {
        it('should return the same params object if it is an object', () => {
            const obj = {method: 'PUT', url: 'kuki'};
            expect(nocPoller.constructParams(obj)).toEqual(obj);
        });

        it('should construct a GET with the URL if the parameter was a string', () => {
            expect(nocPoller.constructParams('puki')).toEqual({method: 'GET', url: 'puki'});
        });
    });

    describe('Pausing and unpausing', () => {
        it('should not trigger a poll while paused, and then should trigger it while unpaused', () => {
            const successFunction = jest.fn();
            nocPoller.pause();
            nocPoller.start(pollUrl, 5000, successFunction, _.noop);
            expect(nocPoller.delayedPolls.length).toEqual(1);
            expect(j.$httpBackend.flush).toThrowError('No pending request to flush !');

            j.$httpBackend.expectGET(prefixedPollUrl).respond(200, {test: 'test'});
            nocPoller.unpause();
            j.$httpBackend.flush();
            expect(successFunction).toHaveBeenCalled();
            expect(mostRecentCall(successFunction)[0]).toEqual({test: 'test'});
            expect(nocPoller.delayedPolls.length).toEqual(0);
        });
    });

    describe('Polling', () => {
        it('should call the success function in the case of success after first poll (and trigger a new poll)', () => {
            j.$httpBackend.expectGET(prefixedPollUrl).respond(200, {test: 'test'});
            const successFunction = jest.fn();
            nocPoller.start(pollUrl, 5000, successFunction, _.noop);
            j.$httpBackend.flush();
            expect(successFunction).toHaveBeenCalled();
            expect(mostRecentCall(successFunction)[0]).toEqual({test: 'test'});
            expect(mostRecentCall(successFunction)[3].url).toEqual('/occm/api/kuki'); //Test to make sure config object is being passed, but only test the url because lots of other stuff is on the config itself which cant be compared

            j.$httpBackend.expectGET(prefixedPollUrl).respond(200, {test: 'test'});
            j.$timeout.flush();
            j.$httpBackend.flush(); //Not throwing an error means it was triggered again
        });

        it('should call the error function in the case of error after first poll (and trigger a new poll)', () => {
            j.$httpBackend.expectGET(prefixedPollUrl).respond(400, {message: 'generic failure message', status: 400});
            const errorFunction = jest.fn();
            nocPoller.start(pollUrl, 5000, _.noop, errorFunction);
            j.$httpBackend.flush();
            expect(errorFunction).toHaveBeenCalled();
            expect(mostRecentCall(errorFunction)[0]).toEqual(j.errorResponse);

            j.$httpBackend.expectGET(prefixedPollUrl).respond(200, {test: 'test'});
            j.$timeout.flush();
            j.$httpBackend.flush(); //Not throwing an error means it was triggered again
        });

        it('should continue silently in case of error but no error function was present', () => {
            j.$httpBackend.expectGET(prefixedPollUrl).respond(400, {message: 'generic failure message'});
            nocPoller.start(pollUrl, 5000, _.noop);
            j.$httpBackend.flush();

            j.$httpBackend.expectGET(prefixedPollUrl).respond(200, {test: 'test'});
            j.$timeout.flush();
            j.$httpBackend.flush(); //Not throwing an error means it was triggered again
        });
    });

    describe('Stopping a specific poll instance', () => {
        it('should not trigger a new poll session if stopped during timeout', () => {
            j.$httpBackend.expectGET(prefixedPollUrl).respond(200);
            const stop = nocPoller.start(pollUrl, 5000, _.noop, _.noop);
            j.$httpBackend.flush();
            stop();
            j.$timeout.flush();
            expect(j.$httpBackend.flush).toThrowError(j.backendFlushFailMessage);
        });

        it('should not perform success function if stopped', () => {
            j.$httpBackend.expectGET(prefixedPollUrl).respond(200);
            const successFunction = jest.fn();
            const stop = nocPoller.start(pollUrl, 5000, successFunction, _.noop);
            stop();
            j.$httpBackend.flush();
            expect(successFunction).not.toHaveBeenCalled();
        });

        it('should not perform error function if not stopped', () => {
            j.$httpBackend.expectGET(prefixedPollUrl).respond(400, {});
            const errorFunction = jest.fn();
            const stop = nocPoller.start(pollUrl, 5000, _.noop, errorFunction);
            stop();
            j.$httpBackend.flush();
            expect(errorFunction).not.toHaveBeenCalled();
        });
    });
});
