describe('Util Service', () => {
    let nocUtil;

    var j;    beforeEach(() => {        j = magnifispec.setup();    });

    beforeEach(inject((_nocUtil_) => {
        nocUtil = _nocUtil_;
    }));

    describe('mergeExistingObject', () => {
        const object = {
            id: 1,
            'c_val': 'hi',
            'n_val': 'bye'
        };

        it('should merge the data from a new object in the updated collection with the data from the old object in the previous collection', () => {
            const collection = [{id: 1, 'c_val': 'hello', 'n_val': 'bye'}];

            nocUtil.mergeExistingObject([object], collection, 'id');
            expect(collection[0].c_val).toEqual('hello');
            expect(collection[0]).toEqual(object);
        });

        it('should merge the data from a new object in the updated collection with the data from the old object in the previous collection finding with more than one key', () => {
            const collection = [{id: 1, 'c_val': 'hello', 'n_val': 'bye'}];

            nocUtil.mergeExistingObject([object], collection, ['id', 'n_val']);
            expect(collection[0].c_val).toEqual('hello');
            expect(collection[0]).toEqual(object);
        });

        it('should not affect a new object that does not have a corresponding old object in the previous collection', () => {
            const newObject = {id: 2, 'c_val': 'hello', 'n_val': 'bye'};
            const collection = [newObject];
            nocUtil.mergeExistingObject([object], collection, 'id');
            expect(collection).toEqual([{id: 2, 'c_val': 'hello', 'n_val': 'bye'}]);
            expect(collection[0]).toEqual(newObject);
        });

        it('should not update anything if the collection was empty', () => {
            const collection = [{id: 1, 'c_val': 'hello', 'n_val': 'bye'}];
            nocUtil.mergeExistingObject([], collection, 'id');
            expect(collection).toEqual([{id: 1, 'c_val': 'hello', 'n_val': 'bye'}]);
        });

        it('should not add anything to an empty collection', () => {
            const collection = [];
            nocUtil.mergeExistingObject([object], collection, 'id');
            expect(collection).toEqual([]);
        });

        it('should add the keys from the old collection if they do not exist in the new one', () => {
            const collection = [{id: 1, 'c_val': 'hello'}];
            nocUtil.mergeExistingObject([object], collection, 'id');
            expect(collection[0]).toEqual({id: 1, 'c_val': 'hello', 'n_val': 'bye'});
        });
    });

    describe('convertTo', () => {
        it('should be able to convert 500 GB to anything!', () => {
            const expected = {
                'GB': 500,
                'TB': 500 / 1024,
                'MB': 500 * 1024,
                'KB': 500 * 1024 * 1024,
                'Byte': 500 * 1024 * 1024 * 1024
            };
            _.forEach(['GB', 'TB', 'MB', 'KB', 'Byte'], (unit) => {
                expect(nocUtil.convertTo({size: 500, unit: 'GB'}, unit)).toEqual({size: expected[unit], unit});
            });
        });
        it('should convert 1030 GB to TB', () => {
            expect(nocUtil.convertTo({size: 1030, unit: 'GB'}, 'TB')).toEqual({size: 1030 / 1024, unit: 'TB'});
        });

        it('should throw an exception when unit is not valid', () => {
            expect(() => {
                nocUtil.convertTo({size: 500, unit: 'GB'}, 'Celsius');
            }).toThrowError('Celsius units are not supported');
            expect(() => {
                nocUtil.convertTo({size: 500, unit: 'Celsius'}, 'GB');
            }).toThrowError('Celsius units are not supported');
        });

        it('should be able to convert zero sizes', () => {
            _.forEach(['KB', 'MB', 'GB', 'TB'], (unit) => {
                expect(nocUtil.convertTo({size: 0, unit: 'GB'}, unit)).toEqual({size: 0, unit});
            });
        });
    });

    describe('Normalize Capacity', () => {
        it('should return 0 KB on capacity with illegal parameter', () => {
            const illegalParam = [false, true, undefined];
            _.forEach(illegalParam, (param) => {
                expect(nocUtil.normalizeCapacity(param)).toEqual({size: 0, unit: 'KB'});
            });
        });

        it('should return 0 KB on capacity with illegal unit', () => {
            expect(nocUtil.normalizeCapacity({size: 24, unit: 'not real'})).toEqual({size: 0, unit: 'KB'});
        });

        it('should return 0 KB/MB/GB/TB given 0 KB/MB/GB/TB', () => {
            const units = ['KB', 'MB', 'GB', 'TB'];
            _.forEach(units, (unit) => {
                expect(nocUtil.normalizeCapacity({size: 0, unit})).toEqual({size: 0, unit});
            });
        });

        it('should be return 512 KB given 512KB, 0.5MB, 0.0004GB, 0.0000004TB', () => {
            const capacities = [
                {size: 512, unit: 'KB'},
                {size: 0.5, unit: 'MB'},
                {size: 0.00048828125, unit: 'GB'},
                {size: 0.000000476837158203125, unit: 'TB'}
            ];
            _.forEach(capacities, (capacity) => {
                expect(nocUtil.normalizeCapacity(capacity)).toEqual({size: 512, unit: 'KB'});
            });
        });

        it('should return 1MB given 1024KB, 1MB, 0.0009765625GB, 0.0000009765625TB', () => {
            const capacities = [
                {size: 1024, unit: 'KB'},
                {size: 1, unit: 'MB'},
                {size: 0.0009765625, unit: 'GB'},
                {size: 0.00000095367431640625, unit: 'TB'}
            ];
            _.forEach(capacities, (capacity) => {
                expect(nocUtil.normalizeCapacity(capacity)).toEqual({size: 1, unit: 'MB'});
            });
        });

        it('should return 1GB given 1048576KB, 1024MB, 1GB, 0.0009765625TB', () => {
            const capacities = [
                {size: 1048576, unit: 'KB'},
                {size: 1024, unit: 'MB'},
                {size: 1, unit: 'GB'},
                {size: 0.0009765625, unit: 'TB'}
            ];
            _.forEach(capacities, (capacity) => {
                expect(nocUtil.normalizeCapacity(capacity)).toEqual({size: 1, unit: 'GB'});
            });
        });

        it('should return 1TB given 0.000000000931322574615478515625KB, 0.00000095367431640625MB, 0.0009765625GB, 1TB', () => {
            const capacities = [
                {size: 1073741824, unit: 'KB'},
                {size: 1048576, unit: 'MB'},
                {size: 1024, unit: 'GB'},
                {size: 1, unit: 'TB'}
            ];
            _.forEach(capacities, (capacity) => {
                expect(nocUtil.normalizeCapacity(capacity)).toEqual({size: 1, unit: 'TB'});
            });
        });
    });

    describe('sumCapacityUsingField', () => {
        it('should sum capacities by field', () => {
            const collection = [
                {a: {size: 1024, unit: 'KB'}},
                {a: {size: 1, unit: 'MB'}}
            ];

            expect(nocUtil.sumCapacityUsingField(collection, 'a')).toEqual({size: 2097152, unit: 'Byte'}); //2 MB in bytes
        });

        it('should return 0 for empty collection', () => {
            expect(nocUtil.sumCapacityUsingField([], 'a')).toEqual({size: 0, unit: 'Byte'});
        });

        it('should return 0 for undefined collection', () => {
            expect(nocUtil.sumCapacityUsingField(undefined, 'a')).toEqual({size: 0, unit: 'Byte'});
        });
    });

    describe('subtractCapacities', () => {
        it('should keep units if the same', () => {
            expect(nocUtil.subtractCapacities({size: 7, unit: 'KB'}, {size: 5, unit: 'KB'}))
                .toEqual({size: 2, unit: 'KB'});
        });

        it('should handle bad formats as second argument', () => {
            expect(nocUtil.subtractCapacities({size: 7, unit: 'KB'}, undefined))
                .toEqual({size: 7, unit: 'KB'});
            expect(nocUtil.subtractCapacities({size: 7, unit: 'KB'}, null))
                .toEqual({size: 7, unit: 'KB'});
            expect(nocUtil.subtractCapacities({size: 7, unit: 'KB'}, {}))
                .toEqual({size: 7, unit: 'KB'});
            expect(nocUtil.subtractCapacities({size: 7, unit: 'MB'}, {size: '7', unit: 'KB'}))
                .toEqual({size: 7 * 1024 * 1024, unit: 'Byte'});
        });

        it('should subtract correctly', () => {
            expect(nocUtil.subtractCapacities({size: 1, unit: 'MB'}, {size: 512, unit: 'KB'}))
                .toEqual({size: 512 * 1024, unit: 'Byte'});
        });
    });

    describe('Fill objects by keys', () => {
        it('should return an empty array if the keys array is empty', () => {
            expect(nocUtil.fillObjectsByKeys([], [{_pathPrefix: '/vsa/', publicId: 1, name: '1'}, {
                _pathPrefix: '/vsa/',
                publicId: 2,
                name: '2'
            }])).toEqual([]);
        });

        it('should use key = id if no key is passed', () => {
            expect(nocUtil.fillObjectsByKeys([1], [{
                _pathPrefix: '/vsa/',
                publicId: 1,
                name: '1'
            }, {_pathPrefix: '/vsa/', publicId: 2, name: '2'}])).toEqual([{
                _pathPrefix: '/vsa/',
                publicId: 1,
                name: '1'
            }]);
        });

        it('should return an empty array if the key does not exist in the object', () => {
            expect(nocUtil.fillObjectsByKeys([1], [{
                _pathPrefix: '/vsa/',
                publicId: 1,
                name: '1'
            }, {_pathPrefix: '/vsa/', publicId: 2, name: '2'}], 'kuki')).toEqual([]);
        });

        it('should work when there are more ids than objects', () => {
            expect(nocUtil.fillObjectsByKeys([1, 2, 3], [{kuki: 1, name: '1'}, {
                kuki: 2,
                name: '2'
            }], 'kuki')).toEqual([{kuki: 1, name: '1'}, {kuki: 2, name: '2'}]);
        });

        it('should work when there are more objects than ids', () => {
            expect(nocUtil.fillObjectsByKeys([1], [{kuki: 1, name: '1'}, {
                kuki: 2,
                name: '2'
            }], 'kuki')).toEqual([{kuki: 1, name: '1'}]);
        });

        it('should work when there are the same number of objects as ids', () => {
            expect(nocUtil.fillObjectsByKeys([1, 2], [{kuki: 1, name: '1'}, {
                kuki: 2,
                name: '2'
            }], 'kuki')).toEqual([{kuki: 1, name: '1'}, {kuki: 2, name: '2'}]);
        });
    });

    describe('sort by ip', () => {
        it('should sort array of objects with ips according to IPs', () => {
            expect(nocUtil.sortByIp([{ip: '1.2.4.4'}, {ip: '1.2.3.4'}, {ip: '1.20.3.4'}, {ip: '1.20.2.4'}, {ip: '1.1.3.4'}], 'ip'))
                .toEqual([{ip: '1.1.3.4'}, {ip: '1.2.3.4'}, {ip: '1.2.4.4'}, {ip: '1.20.2.4'}, {ip: '1.20.3.4'}]);
            expect(nocUtil.sortByIp([{ip: '172.31.59.240'}, {ip: '172.31.6.1'}, {ip: '172.31.3.11'}, {ip: '172.31.1.37'}, {ip: '172.31.1.153'}], 'ip'))
                .toEqual([{ip: '172.31.1.37'}, {ip: '172.31.1.153'}, {ip: '172.31.3.11'}, {ip: '172.31.6.1'}, {ip: '172.31.59.240'}]);
        });
    });

    describe('floorToDecimalPoint', () => {
        const pi = 3.14159265359;
        it('should floor the number to desired decimal unit', () => {
            expect(nocUtil.floorToDecimalPoint(pi, 0)).toEqual(3);
            expect(nocUtil.floorToDecimalPoint(pi, 1)).toEqual(3.1);
            expect(nocUtil.floorToDecimalPoint(pi, 2)).toEqual(3.14);
            expect(nocUtil.floorToDecimalPoint(pi, 3)).toEqual(3.141);
            expect(nocUtil.floorToDecimalPoint(pi, 4)).toEqual(3.1415);
            expect(nocUtil.floorToDecimalPoint(3, 2)).toEqual(3);
        });

        it('should floor zero to zero', () => {
            expect(nocUtil.floorToDecimalPoint(0, 0)).toEqual(0);
            expect(nocUtil.floorToDecimalPoint(0, 3)).toEqual(0.000);
        });

        it('should not round', () => {
            expect(nocUtil.floorToDecimalPoint(0.0019, 3)).toEqual(0.001);
        });

        it('should work for negatives as floor, not truncate', () => {
            expect(nocUtil.floorToDecimalPoint(-pi, 0)).toEqual(-4);
            expect(nocUtil.floorToDecimalPoint(-pi, 1)).toEqual(-3.2);
            expect(nocUtil.floorToDecimalPoint(-pi, 2)).toEqual(-3.15);
            expect(nocUtil.floorToDecimalPoint(-1, 2)).toEqual(-1);
        });

        it('should work large numbers', () => {
            expect(nocUtil.floorToDecimalPoint(pi * 10000000, 3)).toEqual(31415926.535);
        });

        it('should work with negative decimal digits', () => {
            expect(nocUtil.floorToDecimalPoint(pi * 1000, -2)).toEqual(3100);
        });
    });

    describe('genericItemFilter', () => {
        it('should return true for empty search string', () => {
            expect(nocUtil.genericItemFilter('', {}, [])).toEqual(true);
        });

        it('should return true if search matches in right fields', () => {
            expect(nocUtil.genericItemFilter('a', {1: 'bab', 2: 'ccc'}, ['1'])).toEqual(true);
        });

        it('should return false if search matches in wrong fields', () => {
            expect(nocUtil.genericItemFilter('a', {1: 'bab', 2: 'ccc'}, ['2'])).toEqual(false);
        });

        it('should not be case sensitive', () => {
            expect(nocUtil.genericItemFilter('A', {1: 'bab'}, ['1'])).toEqual(true);
        });

        it('should work for numbers', () => {
            expect(nocUtil.genericItemFilter('42', {1: 420}, ['1'])).toEqual(true);
            expect(nocUtil.genericItemFilter('-42.5', {1: -42.52}, ['1'])).toEqual(true);
        });

        it('should work for booleans', () => {
            expect(nocUtil.genericItemFilter('tru', {1: true}, ['1'])).toEqual(true);
            expect(nocUtil.genericItemFilter('fa', {1: false}, ['1'])).toEqual(true);
        });
    });

    describe('volumesMonitoringFilter', () => {
        const volumes = [
            {name: 'csg_ag_quorum', size: {size: 5.0, unit: 'GB'}},
            {name: 'sdw_test', size: {size: 5.0, unit: 'GB'}},
            {name: 'shuki', size: {size: 0.029296875, unit: 'GB'}}
        ];

        beforeEach(inject(($filter) => {
            _.forEach(volumes, (volume) => {
                volume._displaySize = $filter('normalizedCapacity')(volume.size, 2, true); //Add normalized size field to every volume to optimize filtering and display.
            });
        }));

        it('should return all volumes if search string is empty', () => {
            expect(nocUtil.filterVolumes(volumes, '').length).toEqual(3);
        });

        it('should volumes matching search on name and false otherwise', () => {
            expect(nocUtil.filterVolumes(volumes, 'qu')).toEqual([volumes[0]]);
            expect(nocUtil.filterVolumes(volumes, '_')).toEqual([volumes[0], volumes[1]]);
            expect(nocUtil.filterVolumes(volumes, 'shuki')).toEqual([volumes[2]]);
        });

        it('should return true for matching search on capacity and false otherwise', () => {
            expect(nocUtil.filterVolumes(volumes, '5 GB')).toEqual([volumes[0], volumes[1]]);
            expect(nocUtil.filterVolumes(volumes, '30 MB')).toEqual([volumes[2]]);
        });

        it('should be not case sensitive', () => {
            expect(nocUtil.filterVolumes(volumes, 'cSg_aG_qUorUM')).toEqual([volumes[0]]);
            expect(nocUtil.filterVolumes(volumes, '30 mb')).toEqual([volumes[2]]);
        });
    });

    describe('formatSelect2', () => {
        it('should format when all parameters are passed', () => {
            expect(nocUtil.formatSelect2('title', 'a', 'b', 'c', 'd')).toEqual('<div class=\'select2-format-options clearfix\' title=\'title\'><span>a</span><span>b</span><div><span>c</span><span>d</span></div></div>');
        });

        it('should format with just one param in each line', () => {
            expect(nocUtil.formatSelect2('title', 'a', undefined, 'c')).toEqual('<div class=\'select2-format-options clearfix\' title=\'title\'><span>a</span><div><span>c</span></div></div>');
        });

        it('should format with two in the first line and one in the second', () => {
            expect(nocUtil.formatSelect2('title', 'a', 'b', 'c')).toEqual('<div class=\'select2-format-options clearfix\' title=\'title\'><span>a</span><span>b</span><div><span>c</span></div></div>');
        });
    });

    describe('getCurrentTime', () => {
        it('should call moment', () => {
            jest.spyOn(window, 'moment').mockImplementation(() => {});
            nocUtil.getCurrentTime();
            expect(moment).toHaveBeenCalled();
        });
    });

    describe.skip('certificateExpirationState', () => {
        const longDate = '2016-01-02 01:01:01Z',
            futureStartDate = '2015-05-01 01:01:01Z',
            pastStartDate = '2014-12-01 01:01:01Z';

        beforeEach(() => {
            jest.spyOn(nocUtil, 'getCurrentTime').mockReturnValue(moment('2015-01-01 01:01:01Z'));
        });

        it('should return "certificate-valid-long" for long validation dates', () => {
            expect(nocUtil.certificateExpirationState(longDate, pastStartDate)).toEqual({
                state: 'certificate-valid-long',
                message: ''
            });
        });

        it('should return "future-activation" for start dates that begin after the current time', () => {
            expect(nocUtil.certificateExpirationState(longDate, futureStartDate)).toEqual({
                state: 'future-activation',
                message: 'Certificate will be activated on ' + (moment(futureStartDate))
            });
        });

        it('should consider the warning period', () => {
            expect(nocUtil.certificateExpirationState(longDate, undefined, 730)).toEqual({
                state: 'certificate-warning',
                message: 'Certificate will expire in a year.'
            });
        });

        it('should return "certificate-warning" for short validation dates with appropriate warning', () => {
            expect(nocUtil.certificateExpirationState('2015-12-31 23:59:59Z')).toEqual({
                state: 'certificate-warning',
                message: 'Certificate will expire in a year.'
            });
            expect(nocUtil.certificateExpirationState('2015-08-04 15:01:03Z')).toEqual({
                state: 'certificate-warning',
                message: 'Certificate will expire in 7 months.'
            });
            expect(nocUtil.certificateExpirationState('2015-02-01 01:01:01Z')).toEqual({
                state: 'certificate-warning',
                message: 'Certificate will expire in a month.'
            });
            expect(nocUtil.certificateExpirationState('2015-01-04 01:01:01Z')).toEqual({
                state: 'certificate-warning',
                message: 'Certificate will expire in 3 days.'
            });
        });

        it('should return "certificate-expired" for expired', () => {
            expect(nocUtil.certificateExpirationState('2015-01-01 01:01:00Z')).toEqual({
                state: 'certificate-expired',
                message: 'Certificate expired.'
            });
        });
    });

    describe('trimText', () => {
        it('should return the text trimmed if larger than given size', () => {
            expect(nocUtil.trimText('text longer than given size should be trimmed', 10)).toEqual('text lo...');
        });

        it('should return the same text when in the right size', () => {
            expect(nocUtil.trimText('text in right size', 20)).toEqual('text in right size');
        });
    });

    describe('workingEnvironmentTypeToRequestPathPrefix', () => {
        it('should return correct prefix for each WE type', () => {
            expect(nocUtil.workingEnvironmentTypeToRequestPathPrefix('VSA', 'AWS')).toEqual('/vsa/');
            expect(nocUtil.workingEnvironmentTypeToRequestPathPrefix('VSA', 'Azure')).toEqual('/azure/vsa/');
            expect(nocUtil.workingEnvironmentTypeToRequestPathPrefix('HA')).toEqual('/aws/ha/');
            expect(nocUtil.workingEnvironmentTypeToRequestPathPrefix('ON_PREM')).toEqual('/onprem/');
            expect(nocUtil.workingEnvironmentTypeToRequestPathPrefix('Whatever')).not.toBeDefined();
        });
    });

    describe('concatArrayPropertyData', () => {
        it('should concat properties of an empty array', () => {
            expect(nocUtil.concatArrayPropertyData([], 'serialNumber')).toEqual('');
        });

        it('should concat properties of an array with a single item', () => {
            expect(nocUtil.concatArrayPropertyData([{serialNumber: '0'}], 'serialNumber')).toEqual('0');
        });

        it('should concat properties of an array with many items', () => {
            expect(nocUtil.concatArrayPropertyData([{serialNumber: '0'}, {serialNumber: '1'}, {serialNumber: '2'}], 'serialNumber')).toEqual('0, 1, 2');
        });
    });

    describe('splitDataInto2Lists', () => {
        function assertSplit(testCase, arr, res) {
            it('should split an array to half: ' + testCase, () => {
                const split = nocUtil.splitDataInto2Lists(arr);

                expect(split).toEqual(res);
            });
        }

        assertSplit('empty array', [], {leftList: undefined, rightList: undefined});
        assertSplit('one member', [1], {leftList: [1], rightList: undefined});
        assertSplit('2 members', [1, 2], {leftList: [1], rightList: [2]});
        assertSplit('3 members', [1, 2, 3], {leftList: [1, 2], rightList: [3]});
        assertSplit('4 members', [1, 2, 3, 4], {leftList: [1, 2], rightList: [3, 4]});
    });

    describe('normalizeStatus() should return', () => {
        const assertNormalizeStatus = (status, expected, desc) => it(desc, () => expect(nocUtil.normalizeStatus(status)).toBe(expected));

        assertNormalizeStatus('on', 'on', 'keep status');
        assertNormalizeStatus('ON', 'on', 'lower the casing');
        assertNormalizeStatus('not_found', 'not-found', 'underscore to dash');
        assertNormalizeStatus('failed_to_create', 'failed-to-create', '2 underscores to dashes');
        assertNormalizeStatus('', '', 'empty string');
        assertNormalizeStatus(undefined, '', 'undefined');
    });

    describe('getOccmInstanceAws', () => {
        it('should return the correct instance for VSA_AWS', () => {
            const workingEnvironment = {
                publicId: 'weId1',
                awsProperties: {
                    instances: [
                        {
                            id: 'instance1'
                        }, {
                            id: 'instance2',
                            tags: {
                                'CloudONTAP.serial-number': 12345678,
                                'WorkingEnvironmentId': 'weId1'
                            }
                        }
                    ]
                }
            };

            expect(nocUtil.getOccmInstanceAws(workingEnvironment)).toEqual({
                id: 'instance2',
                tags: {
                    'CloudONTAP.serial-number': 12345678,
                    'WorkingEnvironmentId': 'weId1'
                }
            });
        });

        it('should not return databroker instance', () => {
            const workingEnvironment = {
                publicId: 'weId1',
                awsProperties: {
                    instances: [
                        {
                            id: 'data-broker-instance',
                            tags: {
                                'WorkingEnvironmentId': 'weId1'
                            }

                        }, {
                            id: 'instance2',
                            tags: {
                                'CloudONTAP.serial-number': 12345678,
                                'WorkingEnvironmentId': 'weId1'
                            }
                        }
                    ]
                }
            };

            expect(nocUtil.getOccmInstanceAws(workingEnvironment)).toEqual({
                id: 'instance2',
                tags: {
                    'CloudONTAP.serial-number': 12345678,
                    'WorkingEnvironmentId': 'weId1'
                }
            });
        });

        it('should throw for non VSA_AWS we', () => {
            const workingEnvironment = {publicId: 'weId1'};

            expect(() => nocUtil.getOccmInstanceAws(workingEnvironment))
                .toThrow(Error('Cannot get Cloud Manager instance without awsProperties'));
        });
    });

    describe('dontOverrideObjectWithNull', () => {
        it('should return new object when updated', () => {
            expect(nocUtil.dontOverrideObjectWithNull({a: 'a1'}, {a: 'a2'})).toEqual({a: 'a2'});
        });

        it('should override null with new object', () => {
            expect(nocUtil.dontOverrideObjectWithNull(null, {a: 'a2'})).toEqual({a: 'a2'});
        });

        it('should not override existing object with null', () => {
            expect(nocUtil.dontOverrideObjectWithNull({a: 'a1'}, null)).toEqual({a: 'a1'});
        });
    });
});
