describe('Asup Controller', () => {
    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });

    beforeEach(() => {
        j.$httpBackend.expectGET(/asup$/).respond(200, {name: 'asup', schedule: {dayOfWeek: 1, hourOfDay: 2}});
        j.$httpBackend.expectGET(/config$/).respond(200, {debugLogLevel: 'debug', cotRollback: true});
        j.$httpBackend.expectGET(/working-environments\?fields=status$/).respond(200, {
            vsaWorkingEnvironments: [],
            azureVsaWorkingEnvironments: []
        });

        magnifispec.mockStandaloneController('AsupController');

        j.$httpBackend.flush();
    });

    describe('updateIntercomAvailability', () => {
        let supportServicesCall;

        beforeEach(() => {
            supportServicesCall = j.$httpBackend.expectGET(/support-services$/);
            jest.spyOn(j.intercomNotifier, 'boot').mockImplementation(() => {
            });
            jest.spyOn(j.intercomNotifier, 'logout').mockImplementation(() => {
            });
            j.$rootScope.supportServices = {};
        });

        it('should call boot if enabled', () => {
            supportServicesCall.respond({intercomEnabled: true});
            j.scope.updateIntercomAvailability();
            j.$httpBackend.flush();

            expect(j.$rootScope.supportServices.intercomEnabled).toEqual(true);
            expect(j.intercomNotifier.boot).toHaveBeenCalled();
            expect(j.intercomNotifier.logout).not.toHaveBeenCalled();
        });

        it('should call logout if going to disabled', () => {
            supportServicesCall.respond({intercomEnabled: false});
            j.scope.updateIntercomAvailability();
            j.$httpBackend.flush();

            expect(j.$rootScope.supportServices.intercomEnabled).toEqual(false);
            expect(j.intercomNotifier.boot).not.toHaveBeenCalled();
            expect(j.intercomNotifier.logout).toHaveBeenCalled();
        });
    });

    describe('Initialization', () => {
        beforeEach(function () {
            j.$httpBackend.expectGET(/asup$/).respond(200, {name: 'asup', schedule: {dayOfWeek: 1, hourOfDay: 2}});
            j.$httpBackend.expectGET(/working-environments\?fields=status$/).respond(200, {
                vsaWorkingEnvironments: [],
                azureVsaWorkingEnvironments: []
            });
        });

        it('should get the values and populate the scope', () => {
            magnifispec.mockDeferredPromise(true, j.configService, 'getConfig', {
                debugLogLevel: 'debug',
                cotRollback: true
            });
            j.scope.initialize();
            j.$httpBackend.flush();

            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.scope.values).toEqual({
                debugLogLevel: 'debug',
                protocolLoggingEnabled: true,
                cotRollback: true,
                rollbackEnabled: true
            });
            expect(j.scope.values).toEqual(j.scope.originalValues);
            expect(j.scope.values === j.scope.originalValues).toEqual(false);
            expect(j.scope.configuration).toEqual({name: 'asup', schedule: {dayOfWeek: '1', hourOfDay: '2'}});
            expect(j.scope.configuration).toEqual(j.scope.originalConfiguration);
            expect(j.scope.configuration === j.scope.originalConfiguration).toEqual(false);
            expect(j.nocUI.unloading.mock.calls.length).toEqual(5);
        });

        it('should enable logging if log level is verbose and enable rollback if cotRollback is true', () => {
            magnifispec.mockDeferredPromise(true, j.configService, 'getConfig', {
                debugLogLevel: 'verbose',
                cotRollback: true
            });
            j.scope.initialize();
            j.$httpBackend.flush();

            expect(j.scope.values).toEqual({
                debugLogLevel: 'verbose',
                protocolLoggingEnabled: true,
                cotRollback: true,
                rollbackEnabled: true
            });
        });

        it('should disable logging if log level is not debug or verbose and disable rollback if cotRollback is false', () => {
            magnifispec.mockDeferredPromise(true, j.configService, 'getConfig', {
                debugLogLevel: '',
                cotRollback: false
            });
            j.scope.initialize();
            j.$httpBackend.flush();

            expect(j.scope.values).toEqual({
                debugLogLevel: '',
                protocolLoggingEnabled: false,
                cotRollback: false,
                rollbackEnabled: false
            });
        });
    });

    describe('downloadCotNow', () => {
        it('should call download with correct url', () => {
            jest.spyOn(j.scope, 'download').mockImplementation(() => {
            });

            j.scope.downloadCotNow();

            expect(j.scope.download).toHaveBeenCalledWith('/occm/asup/download-ontap-cloud-asups');
        });
    });

    describe('downloadNow', () => {
        it('should call download with correct url', () => {
            jest.spyOn(j.scope, 'download').mockImplementation(() => {
            });

            j.scope.downloadNow();

            expect(j.scope.download).toHaveBeenCalledWith('/occm/asup/download');
        });
    });

    describe('sendNow', () => {
        it('should do nothing!', () => {
            j.scope.sendNow();
        });
    });

    describe('sendCotPerformanceNow', () => {
        it('should do nothing!', () => {
            j.scope.asupTriggerSelect = {
                model: {publicId: 1}
            };
            j.scope.sendCotPerformanceNow();
        });
    });

    describe('save', () => {
        beforeEach(function () {
            jest.spyOn(j.scope, 'updateIntercomAvailability').mockImplementation(() => {
            });
        });

        it('should do nothing if the form is not dirty', () => {
            j.scope.originalConfiguration = 'origValue';
            j.scope.originalValues = 'debug';

            j.scope.values = j.scope.originalValues;
            j.scope.configuration = j.scope.originalConfiguration;

            j.scope.save();

            j.$httpBackend.verifyNoOutstandingExpectation();
            expect(j.scope.configuration).toEqual('origValue');
            expect(j.scope.originalConfiguration).toEqual('origValue');
            expect(j.scope.values).toEqual('debug');
            expect(j.scope.originalValues).toEqual('debug');
        });

        it('should save changes if asup is dirty', () => {
            j.scope.originalConfiguration = 'origValue';
            j.scope.configuration = 'newConfig';
            j.$httpBackend.expectPUT(j.nocBaseUrl + '/occm/asup', j.scope.configuration).respond(200);

            j.scope.save();
            j.$httpBackend.flush();

            expect(j.scope.updateIntercomAvailability).toHaveBeenCalled();
            expect(j.scope.configuration).toEqual('newConfig');
            expect(j.scope.originalConfiguration).toEqual('newConfig');
            expect(j.pidgeon.success).toHaveBeenCalled();
        });

        it('should save changes if logging is dirty', () => {
            j.scope.originalValues = {key: '1'};
            j.scope.values = {key: '2'};
            j.scope.values.protocolLoggingEnabled = false;
            magnifispec.mockDeferredPromise(true, j.configService, 'updateConfig', {debugLogLevel: 'off'});

            j.scope.save();
            magnifispec.processPromise();

            expect(j.scope.values.debugLogLevel).toEqual('off');
            expect(j.scope.originalValues.debugLogLevel).toEqual('off');
            expect(j.pidgeon.success).toHaveBeenCalled();
        });

        it('should save changes if both asup and logging is dirty', () => {
            j.scope.originalValues = {key: '1'};
            j.scope.values = {key: '2'};
            j.scope.values.protocolLoggingEnabled = false;
            j.scope.originalConfiguration = 'origValue';
            j.scope.configuration = 'newConfig';
            j.$httpBackend.expectPUT(j.nocBaseUrl + '/occm/asup', j.scope.configuration).respond(200);
            magnifispec.mockDeferredPromise(true, j.configService, 'updateConfig', {debugLogLevel: 'off'});

            j.scope.save();
            magnifispec.processPromise();
            j.$httpBackend.flush();

            expect(j.scope.updateIntercomAvailability).toHaveBeenCalled();
            expect(j.scope.configuration).toEqual('newConfig');
            expect(j.scope.originalConfiguration).toEqual('newConfig');
            expect(j.scope.values.debugLogLevel).toEqual('off');
            expect(j.scope.originalValues.debugLogLevel).toEqual('off');
            expect(j.pidgeon.success).toHaveBeenCalled();
        });

        it('should not show success pidgeon when theres a logging error', function () {
            j.scope.originalValues = {key: '1'};
            j.scope.values = {key: '2'};
            j.scope.values.protocolLoggingEnabled = false;
            j.scope.originalConfiguration = 'origValue';
            j.scope.configuration = 'newConfig';
            j.$httpBackend.expectPUT(j.nocBaseUrl + '/occm/asup', j.scope.configuration).respond(200);
            jest.spyOn(j.configService, 'updateConfig').mockImplementation(magnifispec.mockPromise());

            j.scope.save();
            j.$httpBackend.flush();

            expect(j.scope.updateIntercomAvailability).toHaveBeenCalled();
            expect(j.scope.configuration).toEqual('newConfig');
            expect(j.scope.originalConfiguration).toEqual('newConfig');
            expect(j.pidgeon.success).not.toHaveBeenCalled();
        });

        it('should not show success pidgeon when theres an asup error', function () {
            j.scope.originalValues = {key: '1'};
            j.scope.values = {key: '2'};
            j.scope.values.protocolLoggingEnabled = false;
            j.scope.originalConfiguration = 'origValue';
            j.scope.configuration = 'newConfig';
            j.$httpBackend.expectPUT(j.nocBaseUrl + '/occm/asup', j.scope.configuration).respond(400, j.errorResponse);
            magnifispec.mockDeferredPromise(true, j.configService, 'updateConfig', {debugLogLevel: 'off'});

            j.scope.save();
            j.$httpBackend.flush();

            expect(j.scope.values.debugLogLevel).toEqual('off');
            expect(j.scope.originalValues.debugLogLevel).toEqual('off');
            expect(j.pidgeon.success).not.toHaveBeenCalled();
        });
    });
});
