describe('Working Environments Controller:', () => {
    'use strict';

    const mockedLayoutOptions = {contentUrl: '/content.html'};
    const workingEnvironment = {
        _pathPrefix: '/vsa/',
        publicId: 1,
        _workingEnvironmentTypeByLocation: 'VSA',
        _cloudProviderName: 'AWS'
    };

    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    beforeEach(inject((nocCurrentUserService) => {
        j.$rootScope.isFeatureOn = jest.fn().mockReturnValue(false);
        jest.spyOn(nocCurrentUserService, 'isAllowed').mockReturnValue(true);
    }));

    describe('gotoUpgradeWorkingEnvironment', () => {
        const relationship = {source: {workingEnvironmentStatus: 'off'}, destination: {workingEnvironmentStatus: 'on'}};
        beforeEach(() => {
            magnifispec.mockTabsController('WorkingEnvironmentsController', {layoutOptions: mockedLayoutOptions}, {checkAndSetState: jest.fn()});
        });

        it('should go to cloud-ontap-upgrade wizard in case eligibility is valid, and no replications transferring', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1/update-eligibility').respond({eligibilityResponse: 'VALID'});
            j.$httpBackend.expectGET(j.nocBaseUrl + '/replication/status/1').respond([]);
            j.scope.gotoUpgradeWorkingEnvironment(workingEnvironment);
            j.$httpBackend.flush();
            expect(j.scope.checkAndSetState).toHaveBeenCalledWith('cloud-ontap-upgrade', {
                from: '',
                workingEnvironmentId: 1,
                type: 'VSA',
                provider: 'AWS',
                first: 'version'
            });
        });

        it('should show dialog in case eligibility is valid, yet replications are transferring', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1/update-eligibility').respond({eligibilityResponse: 'VALID'});
            j.$httpBackend.expectGET(j.nocBaseUrl + '/replication/status/1').respond([relationship]);
            magnifispec.mockConfirm(true);
            j.scope.gotoUpgradeWorkingEnvironment(workingEnvironment);
            j.$httpBackend.flush();
            expect(j.scope.checkAndSetState).toHaveBeenCalledWith('cloud-ontap-upgrade', {
                from: '',
                workingEnvironmentId: 1,
                type: 'VSA',
                provider: 'AWS',
                first: 'version'
            });
        });

        it('should show dialog in case eligibility is LICENSE_EXPIRED', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1/update-eligibility').respond({eligibilityResponse: 'LICENSE_EXPIRED'});
            j.$httpBackend.expectGET(j.nocBaseUrl + '/replication/status/1').respond([]);
            jest.spyOn(j.nocDialog, 'alert').mockImplementation(() => {
            });
            j.scope.gotoUpgradeWorkingEnvironment(workingEnvironment);
            j.$httpBackend.flush();
            expect(j.nocDialog.alert).toHaveBeenCalled();
        });

        it('should show dialog in case eligibility is NOT_REGISTERED', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1/update-eligibility').respond({eligibilityResponse: 'NOT_REGISTERED'});
            j.$httpBackend.expectGET(j.nocBaseUrl + '/replication/status/1').respond([]);
            jest.spyOn(j.nocDialog, 'dialog').mockImplementation(() => {
            });
            j.scope.gotoUpgradeWorkingEnvironment(workingEnvironment);
            j.$httpBackend.flush();
            expect(j.nocDialog.dialog).toHaveBeenCalled();
            expect(j.nocDialog.dialog.mock.calls[0][1]).toEqual('assets/templates/dialogs/eligibility/not-registered.html');
        });

        it('should show dialog in case eligibility is NOT_REGISTERED and gotoSupportRegistrationFunction supplied', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1/update-eligibility').respond({eligibilityResponse: 'NOT_REGISTERED'});
            j.$httpBackend.expectGET(j.nocBaseUrl + '/replication/status/1').respond([]);
            jest.spyOn(j.nocDialog, 'dialog').mockImplementation(() => {
            });
            const gotoSupportRegistrationFunctionSpy = jest.fn();
            j.scope.gotoUpgradeWorkingEnvironment(workingEnvironment, gotoSupportRegistrationFunctionSpy);
            j.$httpBackend.flush();
            expect(j.nocDialog.dialog).toHaveBeenCalledWith('Cloud Volumes ONTAP not registered', 'assets/templates/dialogs/eligibility/not-registered.html', {
                gotoSupportRegistrationFunction: gotoSupportRegistrationFunctionSpy
            });
        });

        it('should go to nss step in cloud-ontap-upgrade wizard in case eligibility is NSS_NOT_EXISTS', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1/update-eligibility').respond({eligibilityResponse: 'NSS_NOT_EXISTS'});
            j.$httpBackend.expectGET(j.nocBaseUrl + '/replication/status/1').respond([]);
            j.scope.gotoUpgradeWorkingEnvironment(workingEnvironment);
            j.$httpBackend.flush();
            expect(j.scope.checkAndSetState).toHaveBeenCalledWith('cloud-ontap-upgrade', {
                from: '',
                workingEnvironmentId: 1,
                type: 'VSA',
                provider: 'AWS',
                first: 'nss'
            });
        });

        it('should go to nss step in cloud-ontap-upgrade wizard in case eligibility is NSS_NOT_VALID', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1/update-eligibility').respond({eligibilityResponse: 'NSS_NOT_VALID'});
            j.$httpBackend.expectGET(j.nocBaseUrl + '/replication/status/1').respond([]);
            j.scope.gotoUpgradeWorkingEnvironment(workingEnvironment);
            j.$httpBackend.flush();
            expect(j.scope.checkAndSetState).toHaveBeenCalledWith('cloud-ontap-upgrade', {
                from: '',
                workingEnvironmentId: 1,
                type: 'VSA',
                provider: 'AWS',
                first: 'nss'
            });
        });

        it('should show dialog in case eligibility is IPA_PROBLEM', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1/update-eligibility').respond({eligibilityResponse: 'IPA_PROBLEM'});
            j.$httpBackend.expectGET(j.nocBaseUrl + '/replication/status/1').respond([]);
            jest.spyOn(j.nocDialog, 'alert').mockImplementation(() => {
            });
            j.scope.gotoUpgradeWorkingEnvironment(workingEnvironment);
            j.$httpBackend.flush();
            expect(j.nocDialog.alert).toHaveBeenCalled();
        });

        it('should go to cifs step in cloud-ontap-upgrade wizard in case there is a lock', () => {
            workingEnvironment.isHA = true;
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1/update-eligibility').respond({eligibilityResponse: 'VALID'});
            j.$httpBackend.expectGET(j.nocBaseUrl + '/replication/status/1').respond([]);
            j.$httpBackend.expectGET(j.nocBaseUrl + '/vsa/working-environments/1/cifs-locks-exist').respond(true);
            j.scope.gotoUpgradeWorkingEnvironment(workingEnvironment);
            j.$httpBackend.flush();
            expect(j.scope.checkAndSetState).toHaveBeenCalledWith('cloud-ontap-upgrade', {
                from: '',
                workingEnvironmentId: 1,
                type: 'VSA',
                provider: 'AWS',
                first: 'cifs'
            });
        });
    });

    describe('clear actions required', () => {
        beforeEach(() => {
            j.$rootScope.currentUser = {roleId: 'Role-2'};
            magnifispec.mockTabsController('WorkingEnvironmentsController', {layoutOptions: mockedLayoutOptions});
        });

        it('should clear it', () => {
            j.scope.actionsRequired = 'action1';
            expect(j.scope.actionsRequired).toEqual('action1');
            j.scope.clearActionsRequired();
            expect(j.scope.actionsRequired).toEqual({});
        });
    });

    describe('updateActionsLevel', () => {
        beforeEach(() => {
            j.scope.actionsRequiredHighestLevel['a'] = 'warning';
        });

        it('should update action level when calling with existing id', () => {
            j.scope.updateActionsLevel('a', {
                'a': [
                    {'severity': 'warning'},
                    {'severity': 'error'}
                ]
            });
            expect(j.scope.actionsRequiredHighestLevel['a']).toEqual('error');
        });

        it('should not add to the hash map when calling with non existing id', () => {
            j.scope.setActionRequiredFor('b', false, {
                'a': [
                    {'severity': 'error'}
                ]
            });
            expect(j.scope.actionsRequiredHighestLevel['a']).toEqual('warning');
            expect(j.scope.actionsRequiredHighestLevel['b']).toEqual(undefined);
        });

        it('should add to the hash map when calling with non existing id', () => {
            j.scope.setActionRequiredFor('b', false, {
                'b': [
                    {'severity': 'error'}
                ]
            });
            expect(j.scope.actionsRequiredHighestLevel['a']).toEqual('warning');
            expect(j.scope.actionsRequiredHighestLevel['b']).toEqual('error');
        });
    });

    describe('getActionsRequiredHighestLevel', () => {
        beforeEach(() => {
            j.scope.actionsRequiredHighestLevel['a'] = 'warning';
            j.scope.actionsRequiredHighestLevel['b'] = 'warning';
        });

        it('should return "warning" if it is the highest level', () => {
            expect(j.scope.getActionsRequiredHighestLevel()).toEqual('warning');
        });

        it('should return "error" if it is the highest level', () => {
            j.scope.actionsRequiredHighestLevel['a'] = 'error';
            expect(j.scope.getActionsRequiredHighestLevel()).toEqual('error');
        });

        it('should return "undefined" no actions are required', () => {
            j.scope.actionsRequiredHighestLevel = {};
            expect(j.scope.getActionsRequiredHighestLevel()).toEqual(undefined);

            j.scope.actionsRequiredHighestLevel = {'a': undefined, 'b': undefined};
            expect(j.scope.getActionsRequiredHighestLevel()).toEqual(undefined);
        });
    });

    describe('setNotificationsOnVSA', () => {
        let workingEnvironment;
        beforeEach(() => {
            workingEnvironment = {_pathPrefix: '/vsa/', publicId: 1, ontapClusterProperties: {upgradeVersions: []}};
            delete j.scope.actionsRequiredHighestLevel[1];
        });
        it('should set _notification to 3 if actionHighestLevel is error', () => {
            j.scope.actionsRequiredHighestLevel[1] = 'error';
            j.scope.setNotificationsOnVSA(workingEnvironment);
            expect(workingEnvironment._notifications).toEqual(3);
        });

        it('should set _notification to 2 if actionHighestLevel is warning', () => {
            j.scope.actionsRequiredHighestLevel[1] = 'warning';
            j.scope.setNotificationsOnVSA(workingEnvironment);
            expect(workingEnvironment._notifications).toEqual(2);
        });

        it('should set _notification to 1 if upgrade is available (and no actions required)', () => {
            workingEnvironment.ontapClusterProperties.upgradeVersions.push({});
            j.scope.setNotificationsOnVSA(workingEnvironment);
            expect(workingEnvironment._notifications).toEqual(1);
        });

        it('should set _notification to 3 if no actions required and no upgrade available', () => {
            j.scope.setNotificationsOnVSA(workingEnvironment);
            expect(workingEnvironment._notifications).toEqual(0);
        });
    });

    describe('setActionRequiredFor', () => {
        beforeEach(() => {
            j.scope.actionsRequired = {
                '1': [{
                    'something': 'something'
                }]
            };
        });

        it('should call setNotificationsOnVSA', () => {
            jest.spyOn(j.scope, 'setNotificationsOnVSA').mockImplementation(() => {
            });
            j.scope.workingEnvironments = [{_pathPrefix: '/vsa/', publicId: '2'}];
            j.scope.setActionRequiredFor('2', false, {'2': [{'something': 'else', severity: 'error'}]});
            expect(j.scope.setNotificationsOnVSA).toHaveBeenCalledWith(j.scope.workingEnvironments[0]);
        });

        it('should add to the hash map when calling with error', () => {
            jest.spyOn(j.scope, 'updateActionsLevel');
            j.scope.setActionRequiredFor('2', false, {'2': [{'something': 'else', severity: 'error'}]});
            expect(j.scope.actionsRequired['2']).toEqual([{'something': 'else', severity: 'error'}]);
            expect(j.scope.updateActionsLevel).toHaveBeenCalled();
        });

        it('should add to the hash map when calling with error, even when active', () => {
            jest.spyOn(j.scope, 'updateActionsLevel');
            j.scope.setActionRequiredFor('2', true, {'2': [{'something': 'else', severity: 'error'}]});
            expect(j.scope.actionsRequired['2']).toEqual([{'something': 'else', severity: 'error'}]);
            expect(j.scope.updateActionsLevel).toHaveBeenCalled();
        });

        it('should update the hash map when calling with an existing id', () => {
            j.scope.setActionRequiredFor('1', false, {'1': [{'something': 'else'}]});
            expect(j.scope.actionsRequired['1']).toEqual([{'something': 'else'}]);
        });

        it('should add to the hash map when calling with non existing id', () => {
            j.scope.setActionRequiredFor('2', false, {'1': [{'something': 'else'}]});
            expect(j.scope.actionsRequired['1']).toEqual([{'something': 'something'}]);
            expect(j.scope.actionsRequired['2']).toEqual(undefined);
        });

        it('should erase actions if we is active', () => {
            j.scope.setActionRequiredFor('1', true, {'1': [{'something': 'else'}]});
            expect(j.scope.actionsRequired['1']).toEqual(undefined);
        });
    });

    describe('setComputingCapacity', () => {
        it('should just set computing capacity flag', () => {
            j.scope.setComputingCapacity(true);
            expect(j.scope.computingCapacity).toEqual(true);
            j.scope.setComputingCapacity(false);
            expect(j.scope.computingCapacity).toEqual(false);
        });
    });

    describe('For currentUser is role-1', () => {
        beforeEach(() => {
            j.$rootScope.currentUser = {roleId: 'Role-1'};

            magnifispec.mockTabsController('WorkingEnvironmentsController', {layoutOptions: mockedLayoutOptions});
            j.$rootScope.checkAndSetState = jest.fn();
        });

        describe('Back to gallery', () => {
            it('should clear alerts, change mode to gallery, delete the requests', () => {
                j.scope.backToGallery();

                expect(j.scope.checkAndSetState).toHaveBeenCalledWith('main.working-environments.index');
            });
        });

        describe('deleteWorkingEnvironment', () => {
            let deletedWorkingEnvironment;

            beforeEach(() => {
                deletedWorkingEnvironment = {
                    name: 'we1',
                    publicId: 'id1',
                    workingEnvironmentType: 'ON_PREM',
                    _allowedOperations: {}
                };
                j.scope.workingEnvironments = [
                    {name: 'we2', publicId: 'id2'},
                    deletedWorkingEnvironment
                ];
                jest.spyOn(j.scope, 'showMonitoring').mockImplementation(() => {
                });
                jest.spyOn(j.replicationResource, 'getReplicationStatus').mockReturnValue(j.$q.when([]));
            });

            it('should call deletion API', () => {
                jest.spyOn(j.workingEnvironmentResource, 'deleteWorkingEnvironment').mockReturnValue(j.$q.when());
                magnifispec.mockConfirm(true);

                j.scope.deleteWorkingEnvironment(deletedWorkingEnvironment);
                magnifispec.processPromise();

                expect(j.workingEnvironmentResource.deleteWorkingEnvironment)
                    .toHaveBeenCalledWith(jasmine.any(Object), deletedWorkingEnvironment, undefined);
            });

            it('should do nothing if the user does not confirm and show the right dialog', () => {
                magnifispec.mockConfirm(false);

                j.scope.deleteWorkingEnvironment(deletedWorkingEnvironment);
                magnifispec.processPromise();

                expect(j.nocDialog.confirm).toHaveBeenCalledWith('',
                    'assets/templates/tabs/working-environments/delete-working-environment-dialog-content.html', {
                        workingEnvironment: deletedWorkingEnvironment,
                        hasReplications: 0,
                        width: 650
                    });
                expect(j.scope.showMonitoring).not.toHaveBeenCalled();
            });
        });

        describe('removeWorkingEnvironment', () => {
            let removeWorkingEnvironment;

            beforeEach(() => {
                removeWorkingEnvironment = {
                    name: 'we1',
                    publicId: 'id1',
                    workingEnvironmentType: 'ON_PREM',
                    _allowedOperations: {}
                };
                j.scope.workingEnvironments = [
                    {name: 'we2', publicId: 'id2'},
                    removeWorkingEnvironment
                ];
                jest.spyOn(j.scope, 'showMonitoring').mockImplementation(() => {
                });
            });

            it('call deletion API', () => {
                jest.spyOn(j.workingEnvironmentResource, 'deleteWorkingEnvironment', undefined).mockReturnValue(j.$q.when());
                magnifispec.mockConfirm(true);

                j.scope.removeWorkingEnvironment(removeWorkingEnvironment);
                magnifispec.processPromise();

                expect(j.workingEnvironmentResource.deleteWorkingEnvironment)
                    .toHaveBeenCalledWith(jasmine.any(Object), removeWorkingEnvironment, undefined);
            });

            it('should do nothing if the user does not confirm and show the right message', () => {
                magnifispec.mockConfirm(false);

                j.scope.removeWorkingEnvironment(removeWorkingEnvironment);
                magnifispec.processPromise();

                expect(j.nocDialog.confirm).toHaveBeenCalledWith('Are you sure you want to remove working environment we1?',
                    'assets/templates/dialogs/remove-confirm-content.html', {workingEnvironmentType: 'ON_PREM'});
                expect(j.scope.showMonitoring).not.toHaveBeenCalled();
            });
        });

        describe('showMonitoring', () => {
            it('should route to monitoring', () => {
                jest.spyOn(j.scope, 'goToGadget').mockImplementation(() => {
                });
                workingEnvironment._allowedOperations = {resources: true};

                j.scope.showMonitoring(workingEnvironment);

                expect(j.scope.goToGadget).toHaveBeenCalledWith('', '', workingEnvironment);
            });
        });

        describe('startCreateNewVolumeOnAggregateWizard', () => {
            it('should call checkAndSetState with right parameters for gp2', () => {
                workingEnvironment._allowedOperations = {'aggregate-add-capacity': true};
                const aggr = {
                    name: 'aggr1',
                    _tier: 'gp2',
                    _providerVolumeType: 'gp2',
                    capacityTier: 'S3',
                    providerVolumes: [{iops: undefined}]
                };

                j.scope.startCreateNewVolumeOnAggregateWizard(workingEnvironment, aggr);

                expect(j.scope.checkAndSetState).toHaveBeenCalledWith('create-new-volume-wizard', {
                    workingEnvironmentId: 1,
                    from: 'main.working-environments.monitoring.index',
                    gadget: 'advancedAllocation',
                    type: 'VSA',
                    provider: 'AWS',
                    aggregateName: 'aggr1',
                    providerVolumeType: 'gp2',
                    capacityTier: 'S3',
                    iops: undefined
                });
            });

            it('should call checkAndSetState with right parameters for io1', () => {
                workingEnvironment._allowedOperations = {'aggregate-add-capacity': true};
                const aggr = {
                    name: 'aggr1',
                    _tier: 'io1',
                    _providerVolumeType: 'io1',
                    capacityTier: undefined,
                    providerVolumes: [{iops: 999}]
                };

                j.scope.startCreateNewVolumeOnAggregateWizard(workingEnvironment, aggr);

                expect(j.scope.checkAndSetState).toHaveBeenCalledWith('create-new-volume-wizard', {
                    workingEnvironmentId: 1,
                    from: 'main.working-environments.monitoring.index',
                    gadget: 'advancedAllocation',
                    type: 'VSA',
                    provider: 'AWS',
                    aggregateName: 'aggr1',
                    providerVolumeType: 'io1',
                    capacityTier: undefined,
                    iops: 999
                });
            });
        });

        describe('startCreateNewVolumeWizard', () => {
            it('should call checkAndSetState with right parameters', () => {
                workingEnvironment._allowedOperations = {'add-capacity': true};

                j.scope.startCreateNewVolumeWizard(workingEnvironment);

                expect(j.scope.checkAndSetState).toHaveBeenCalledWith('create-new-volume-wizard', {
                    workingEnvironmentId: 1,
                    from: undefined,
                    type: 'VSA',
                    provider: 'AWS'
                });
            });
        });

        describe('gotoCreateWorkingEnvironment', () => {
            it('should checkAndSetState', () => {
                j.scope.gotoCreateWorkingEnvironment();
                expect(j.$rootScope.checkAndSetState).toHaveBeenCalledWith('wizard', {from: 'main.working-environments.index'});
            });
        });
    });

    describe('handle drag and drop Drop', () => {
        let ui, scopeFunc, scope;
        let _nocSelectedWorkspaceOrTenantService;

        beforeEach(inject((nocSelectedWorkspaceOrTenantService) => {
            j.$rootScope.currentUser = {roleId: 'Role-1'};
            magnifispec.mockTabsController('WorkingEnvironmentsController', {layoutOptions: mockedLayoutOptions});
            j.$rootScope.checkAndSetState = jest.fn();

            scope = {workingEnvironment: {_pathPrefix: '/vsa/', publicId: 1}};
            scopeFunc = jest.fn().mockReturnValue(scope);
            ui = {draggable: {scope: scopeFunc}};

            j.scope.selectedTenant = {publicId: 1};

            _nocSelectedWorkspaceOrTenantService = nocSelectedWorkspaceOrTenantService;
            jest.spyOn(_nocSelectedWorkspaceOrTenantService, 'getPublicId').mockReturnValue(1);
        }));

        it('should do nothing if source and target are the same on drop', () => {
            j.scope.handleWorkingEnvironmentDrop({}, ui, {_pathPrefix: '/vsa/', publicId: 1});
            expect(j.$rootScope.checkAndSetState).not.toHaveBeenCalled();
        });

        it('should navigate to the snap mirror wizard if the source and target are different, on drop', () => {
            j.scope.handleWorkingEnvironmentDrop({}, ui, {_pathPrefix: '/vsa/', publicId: 2});
            expect(j.$rootScope.checkAndSetState).toHaveBeenCalledWith('snap-mirror-wizard', {
                from: 'main.working-environments.index',
                slTenantId: 1,
                sourceId: 1,
                targetId: 2
            });
        });

        it('should stop propagation on drag start', () => {
            const stopImmediatePropagationFunc = jest.fn();
            const event = {stopImmediatePropagation: stopImmediatePropagationFunc};
            j.scope.handleWorkingEnvironmentDragStart(event);
            expect(event.stopImmediatePropagation).toHaveBeenCalled();
        });
    });

    describe('startVsa', () => {
        let workingEnvironment;

        beforeEach(() => {
            magnifispec.mockTabsController('WorkingEnvironmentsController', {layoutOptions: mockedLayoutOptions}, {checkAndSetState: jest.fn()});
            workingEnvironment = {publicId: 'lili', _pathPrefix: 'vsa/'};
        });

        it('should work upon success', () => {
            j.$httpBackend.expectPOST('/occm/api/vsa/working-environments/lili/start').respond(200, true);
            jest.spyOn(j.workingEnvironmentResource, 'updateWorkingEnvironmentStatus').mockImplementation(() => {
            });
            j.scope.startVsa(workingEnvironment);
            j.$httpBackend.flush();

            expect(j.workingEnvironmentResource.updateWorkingEnvironmentStatus).toHaveBeenCalledWith(workingEnvironment, 'starting');
        });

        it('should work upon error', () => {
            j.$httpBackend.expectPOST('/occm/api/vsa/working-environments/lili/start').respond(400, j.errorResponse);
            j.scope.startVsa(workingEnvironment);
            j.$httpBackend.flush();

            expect(j.pidgeon.error).toHaveBeenCalled();
        });
    });

    describe('createReplicationMapByType', () => {
        it('should create the correct map', () => {
            let workingEnvironments;
            workingEnvironments = [{workingEnvironmentType: 'VSA', _allowedOperations: {replicate: true}}];
            j.scope.createReplicationMapByType(workingEnvironments);
            expect(j.scope.replicationMapByType).toEqual({VSA: 1});

            workingEnvironments = [{workingEnvironmentType: 'VSA', _allowedOperations: {replicate: true}},
                {workingEnvironmentType: 'ON_PREM', _allowedOperations: {replicate: true}}];
            j.scope.createReplicationMapByType(workingEnvironments);
            expect(j.scope.replicationMapByType).toEqual({VSA: 1, ON_PREM: 1});

            workingEnvironments = [{workingEnvironmentType: 'ON_PREM', _allowedOperations: {replicate: true}},
                {workingEnvironmentType: 'ON_PREM', _allowedOperations: {replicate: false}}];
            j.scope.createReplicationMapByType(workingEnvironments);
            expect(j.scope.replicationMapByType).toEqual({ON_PREM: 1});

            workingEnvironments = [{workingEnvironmentType: 'ON_PREM', _allowedOperations: {replicate: false}}];
            j.scope.createReplicationMapByType(workingEnvironments);
            expect(j.scope.replicationMapByType).toEqual({});
        });
    });
});
