describe('AddDisksAggregate Controller', () => {
    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    const mockedResponses = magnifispec.getMockedResponses();
    const _providerVolumeTypes = {
        gp2: {
            name: 'gp2',
            supportedSizes: [
                {size: 100, unit: 'GB', _identifier: '100 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                {size: 500, unit: 'GB', _identifier: '500 GB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                {size: 1, unit: 'TB', _identifier: '1 TB', maxDisksAllow: {numOfDisks: 4, reason: null}}
            ]
        },
        st1: {
            name: 'st1',
            supportedSizes: [
                {size: 1, unit: 'TB', _identifier: '1 TB', maxDisksAllow: {numOfDisks: 6, reason: null}},
                {size: 2, unit: 'TB', _identifier: '2 TB', maxDisksAllow: {numOfDisks: 4, reason: null}}
            ]
        }
    };

    beforeEach(inject((_aggregatesResource_) => {
        jest.spyOn(_aggregatesResource_, 'getAggregates').mockReturnValue(j.$q.when(mockedResponses.aggregatesResponse));
        const workingEnvironment = {
            _pathPrefix: '/vsa/',
            _cloudProviderName: 'AWS',
            publicId: 1,
            instanceId: 'id',
            providerVolumeType: 'gp2',
            name: 'we1',
            capacityFeatures: {maxDisksPerAggregate: 6},
            _providerVolumeTypes
        };
        magnifispec.mockTabsController('AddDisksAggregateController', {}, {
            updateSelectedWorkingEnvironment: jest.fn().mockReturnValue(j.$q.when()),
            updateWorkingEnvironmentFields: jest.fn().mockReturnValue(j.$q.when()),
            selectedWorkingEnvironment: workingEnvironment,
            goToGadget: jest.fn()
        });
        magnifispec.processPromise();
    }));

    describe('initializeController', () => {
        describe('should initialize correctly when aggregate exists', () => {
            beforeEach(() => {
                j.pidgeon.error.mockClear();
            });

            it('should initialize correctly when aggregate exists with possibility to add disks', () => {
                j.$state.params = {aggregateName: 'aggr1'};
                j.scope.initializeController();
                magnifispec.processPromise();

                expect(j.scope.selectedAggregate.name).toEqual('aggr1');
                expect(j.scope.addDisksRequest).toEqual({name: 'aggr1', numberOfDisks: 1});
                expect(j.scope.numberOfDisksOptions).toEqual([1, 2, 3, 4]);
                expect(j.pidgeon.error).not.toHaveBeenCalled();
            });

            it('should initialize correctly when it\'s full', () => {
                j.$state.params = {aggregateName: 'aggr2'};
                j.scope.initializeController();
                magnifispec.processPromise();

                expect(j.scope.selectedAggregate.name).toEqual('aggr2');
                expect(j.scope.numberOfDisksOptions).toEqual([]);

                expect(j.pidgeon.error).toHaveBeenCalledWith('Cannot add any disks to aggregate, maximum capacity of 6 disks per aggregate reached', '', 'initialize');
            });

            it('should initialize correctly when there is server limit on adding more disks', () => {
                j.$state.params = {aggregateName: 'aggr1'};
                j.scope.selectedWorkingEnvironment._providerVolumeTypes.gp2.supportedSizes[2].maxDisksAllow = {
                    numOfDisks: 0,
                    reason: 'server limit'
                };
                j.scope.initializeController();
                magnifispec.processPromise();

                expect(j.pidgeon.error).toHaveBeenCalledWith('Cannot add any disks to aggregate: server limit', '', 'initialize');
            });
        });

        it('should fail gracefully when aggregate doesn\'t exist', () => {
            j.$state.params = {aggregateName: 'aggr3'};

            j.scope.initializeController();
            magnifispec.processPromise();

            expect(j.scope.selectedAggregate).toBeUndefined();
            expect(j.pidgeon.error).toHaveBeenCalledWith('aggr3 does not exist', '', 'initialize');
        });
    });

    describe('addDisksAggregate', () => {
        beforeEach(() => {
            j.scope.selectedAggregate = {
                name: 'aggr2',
                _providerVolumeType: 'st1',
                providerVolumes: [{
                    size: {
                        size: 1099511627776,
                        unit: 'Byte'
                    }
                }]
            };
        });

        it('should validate the form', () => {
            jest.spyOn(j.nocDialog, 'confirm').mockImplementation(() => {
            });
            j.scope.addDisksAggregateForm = {$valid: false};

            j.scope.addDisksAggregate();

            expect(j.nocUI.handleFormValidity).toHaveBeenCalled();
            expect(j.nocDialog.confirm).not.toHaveBeenCalled();
        });

        it('should be able to send add disks request', inject((nocApprovalMessageBuilder, aggregatesResource) => {
            magnifispec.mockConfirm(true);
            j.scope.addDisksAggregateForm = {$valid: true};
            j.scope.addDisksRequest.numberOfDisks = 1;
            jest.spyOn(aggregatesResource, 'addDisksToAggregate').mockReturnValue(j.$q.when({}));
            jest.spyOn(nocApprovalMessageBuilder, 'buildTitle').mockReturnValue('HTML code');

            j.scope.addDisksAggregate();
            magnifispec.processPromise();

            expect(nocApprovalMessageBuilder.buildTitle).toHaveBeenCalledWith({
                provider: 'AWS',
                numOfDisks: 1,
                diskSize: {size: 1, unit: 'TB'},
                diskType: 'Throughput Optimized HDD (st1)',
                isHA: undefined
            });
            expect(j.nocDialog.confirm).toHaveBeenCalledWith('', 'assets/templates/dialogs/purchase-confirm-content.html', {html: 'HTML code'});
        }));

        it('should not send post request if confirm was false', () => {
            magnifispec.mockConfirm(false);
            j.scope.addDisksAggregateForm = {$valid: true};
            j.scope.addDisksRequest.numberOfDisks = 1;

            j.scope.addDisksAggregate();

            expect(j.$httpBackend.flush).toThrowError('No pending request to flush !');
            expect(j.nocDialog.confirm).toHaveBeenCalled();
        });

        it('should fail gracefully when request denied', () => {
            const addDiskRequest = {
                numberOfDisks: 1,
                name: 'aggr2'
            };
            magnifispec.mockConfirm(true);
            j.scope.addDisksAggregateForm = {$valid: true};
            j.scope.addDisksRequest = addDiskRequest;
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/vsa/aggregates/1/aggr2/disks', addDiskRequest)
                .respond(400, j.errorResponse);

            j.scope.addDisksAggregate();
            j.$httpBackend.flush();

            expect(j.pidgeon.error).toHaveBeenCalled();
        });
    });
});
