describe('addKeyManagerController', () => {
    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    const vsaResponse = {
        vsaEncryptionResponse: {
            keyManagers: [
                {address: '1.1.1.1', name: 'key_manager_A'},
                {address: '2.2.2.2', name: 'key_manager_B'}
            ]
        },
        id: 'encryptedVsaId1'
    };

    beforeEach(() => {
        magnifispec.mockTabsController('addKeyManagerController', {}, {
            selectedWorkingEnvironment: {_pathPrefix: '/vsa/', publicId: '1', workingEnvironmentType: 'VSA'},
            vsaResponse
        });
        j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/encryption/key-managers').respond([{uninitializedValue: 'uninitializedValue'}]);
    });

    describe('getKeyManagers', () => {
        it('should work upon success and filter out the key managers already in use', () => {
            const keyManagers = [
                {name: 'b', address: '2.2.2.2'},
                {name: 'c', address: '3.3.3.3'}
            ];
            j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/encryption/key-managers').respond(200, keyManagers);

            j.scope.getKeyManagers();
            j.$httpBackend.flush();

            expect(j.scope.occmKeyManagers).toEqual(keyManagers);
            expect(j.scope.keyManagersToDisplay).toEqual([{name: 'c', address: '3.3.3.3'}]);
        });

        it('should work upon error', () => {
            j.$httpBackend.expectGET(j.nocBaseUrl + '/occm/encryption/key-managers').respond(400, j.errorResponse);

            j.scope.getKeyManagers();
            j.$httpBackend.flush();

            expect(j.scope.occmKeyManagers).toEqual([{uninitializedValue: 'uninitializedValue'}]);
            expect(j.pidgeon.error).toHaveBeenCalled();
        });

        afterEach(() => {
            expect(j.pidgeon.clear).toHaveBeenCalled();
            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });
    });

    describe('formatSelect2', () => {
        it('should return the correct format', () => {
            expect(j.scope.formatSelect2('123.123.123.123', 'one-two-three-key-manager'))
                .toEqual('<div class="select2-format-options clearfix trimmed-text" title="123.123.123.123 (one-two-three-key-manager)" style="width: 321px;"><span>123.123.123.123</span><span>one-two-three-key-manager</span></div>');
        });
    });

    describe('formatKeyManager', () => {
        beforeEach(() => {
            j.scope.keyManagersToDisplay = [
                {name: 'a', address: '1.1.1.1'},
                {name: 'b', address: '2.2.2.2'}
            ];
            jest.spyOn(j.scope, 'formatSelect2').mockImplementation(() => {
            });
        });

        it('should return nothing for nothing', () => {
            expect(j.scope.formatKeyManager({})).toEqual('');
        });

        it('should return the right format for key manager', () => {
            j.scope.formatKeyManager({text: '2.2.2.2'});
            expect(j.scope.formatSelect2).toHaveBeenCalledWith('2.2.2.2', 'b');
        });
    });

    describe('keyManagerMatcher', () => {
        it('should find the right key manager and call genericItemFilter', () => {
            jest.spyOn(j.nocUtil, 'genericItemFilter').mockImplementation(() => {
            });
            j.scope.keyManagersToDisplay = [
                {name: 'Kipi_ben_kipod', address: '12.34.56.78'},
                {name: 'Moishe_oofnik', address: '3.3.3.3'}
            ];
            j.scope.keyManagerMatcher('kipod', '12.34.56.78');
            expect(j.nocUtil.genericItemFilter).toHaveBeenCalledWith('kipod', {
                name: 'Kipi_ben_kipod',
                address: '12.34.56.78'
            }, ['name', 'address']);
        });
    });

    describe('okClicked', () => {
        it('should validate the form and call addKeyManager', () => {
            jest.spyOn(j.scope, 'addKeyManager').mockImplementation(() => {
            });
            j.scope.addKeyManagerForm = {$valid: true};
            j.scope.okClicked();
            expect(j.scope.addKeyManager).toHaveBeenCalled();
        });

        it('should handle the form validity if invalid', () => {
            j.scope.addKeyManagerForm = {$valid: false};
            j.scope.okClicked();
            expect(j.nocUI.handleFormValidity).toHaveBeenCalled();
        });
    });

    describe('addKeyManager', () => {
        it('should work upon success and close the dialog', () => {
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/vsa/encryption/encryptedVsaId1/key-managers/77.77.77.77').respond(200);
            j.scope.closeDialog = jest.fn();

            j.scope.selectedKeyManager = {address: '77.77.77.77'};

            j.scope.addKeyManager();
            j.$httpBackend.flush();

            expect(j.scope.closeDialog).toHaveBeenCalled();
        });

        it('should work upon error', () => {
            j.$httpBackend.expectPOST(j.nocBaseUrl + '/vsa/encryption/encryptedVsaId1/key-managers/77.77.77.77').respond(400, j.errorResponse);
            j.scope.selectedKeyManager = {address: '77.77.77.77'};

            j.scope.addKeyManager();
            j.$httpBackend.flush();

            expect(j.pidgeon.error).toHaveBeenCalled();
        });

        afterEach(() => {
            expect(j.pidgeon.clear).toHaveBeenCalled();
            expect(j.nocUI.block).toHaveBeenCalled();
            expect(j.nocUI.unblock).toHaveBeenCalled();
        });
    });

    describe('initialize', () => {
        it('should do its thing', () => {
            jest.spyOn(j.scope, 'getKeyManagers').mockImplementation(() => {
            });

            j.scope.initialize();

            expect(j.scope.workingEnvironmentKeyManagers).toEqual([
                {address: '1.1.1.1', name: 'key_manager_A'},
                {address: '2.2.2.2', name: 'key_manager_B'}
            ]);
            expect(j.scope.getKeyManagers).toHaveBeenCalled();
            expect(j.scope.selectedKeyManager).toEqual({});
        });
    });
});
