describe('Aggregate Controller', function () {
    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    const mockedAggregatesResponses = magnifispec.getMockedResponses().aggregatesResponse;
    let stopAggregatePolling, stopCapacityFeaturesPolling;
    let aggregatesResource;

    beforeEach(inject((_aggregatesResource_) => {
        aggregatesResource = _aggregatesResource_;
        stopAggregatePolling = jest.fn();
        jest.spyOn(aggregatesResource, 'startAggregatesPolling').mockReturnValue(stopAggregatePolling);
        stopCapacityFeaturesPolling = jest.fn();
        jest.spyOn(j.workingEnvironmentResource, 'startCapacityFeaturesPolling').mockReturnValue(stopCapacityFeaturesPolling);
        magnifispec.mockTabsController('AggregateController', {}, {
            selectedWorkingEnvironment: {
                _pathPrefix: '/vsa/',
                _cloudProviderName: 'AWS',
                publicId: 1,
                instanceId: 'id',
                _isAWS: true
            },
            updateSelectedWorkingEnvironment: j.$q.when
        });
        magnifispec.processPromise();
    }));

    describe('buildCloudProviderCapacityData', function () {
        it('should return an correct object for the relevant provider', function () {
            j.scope.selectedWorkingEnvironment._cloudProviderName = 'AWS';
            jest.spyOn(j.scope, 'calculateRawCapacity').mockReturnValue({size: 1.4645453, unit: 'TB'});
            expect(j.scope.buildCloudProviderCapacityData('')).toEqual({
                key: 'Underlying AWS Capacity:',
                text: '1.46 TB'
            });

        });
    });

    describe('deleteAggregate ', function () {
        beforeEach(function () {
            j.scope.selectedWorkingEnvironment._allowedOperations = {'aggregate-delete': true};
        });
        it('should prompt to the uer to confirm deletion', function () {
            magnifispec.fakeConfirmNew(true);
            j.scope.deleteAggregate({name: 'aggr2', volumes: []});
            expect(j.nocDialog.confirm).toHaveBeenCalledWith(j.Messages.deleteAggregateConfirm({name: 'aggr2'}), 'assets/templates/tabs/working-environments/delete-aggregate-dialog-content.html', {ontapEncrypted: false});
        });

        it('should prompt to the uer to confirm deletion when aggregate is encrypted', function () {
            magnifispec.fakeConfirmNew(true);
            j.scope.deleteAggregate({name: 'aggr2', volumes: [], encryptionType: 'ontapEncrypted'});
            expect(j.nocDialog.confirm).toHaveBeenCalledWith(j.Messages.deleteAggregateConfirm({name: 'aggr2'}), 'assets/templates/tabs/working-environments/delete-aggregate-dialog-content.html', {ontapEncrypted: true});
        });

        it('should call the right resource to delete', () => {
            j.scope.actionsRequired = {};
            j.scope.actionsRequired[j.scope.selectedWorkingEnvironment.publicId] = {};
            magnifispec.fakeConfirmNew(true);
            jest.spyOn(aggregatesResource, 'deleteAggregate').mockReturnValue(j.$q.when());
            var aggrToDelete = {name: 'aggr2', volumes: []};
            j.scope.deleteAggregate(aggrToDelete);
            magnifispec.processPromise();
            expect(j.scope.actionsRequired[j.scope.selectedWorkingEnvironment.publicId]).toBeUndefined();
            expect(aggrToDelete._deleting).toEqual(true);
        });

        it('should show correct pidgeon', () => {
            magnifispec.fakeConfirmNew(true);
            jest.spyOn(aggregatesResource, 'deleteAggregate').mockReturnValue(j.$q.when());
            const aggrToDelete = {name: 'aggr2', volumes: []};

            j.scope.deleteAggregate(aggrToDelete);
            magnifispec.processPromise();

            expect(j.pidgeon.inProgress).toHaveBeenCalledWith('Aggregate delete in progress.', true);
        });


        it('should do nothing upon canceling a delete', function () {
            magnifispec.fakeConfirmNew(false);
            j.scope.deleteAggregate({name: 'aggr2', volumes: []});
            expect(j.nocUI.block.mock.calls.length).toEqual(0);
            expect(j.nocUI.unblock.mock.calls.length).toEqual(0);
        });


        it('should not try to delete an aggregate with volumes', function () {
            j.scope.deleteAggregate({name: 'aggr2', volumes: [{name: 'vol1'}]});
            expect(j.nocUI.block.mock.calls.length).toEqual(0);
            expect(j.nocUI.unblock.mock.calls.length).toEqual(0);
        });
    });

    describe('calculate raw capacity', function () {
        it('should be able to calculate the max size', function () {
            var aggregate = {
                'providerVolumes': [
                    {size: {size: 1, unit: 'TB'}},
                    {size: {size: 1, unit: 'TB'}},
                    {size: {size: 1, unit: 'TB'}},
                    {size: {size: 1, unit: 'TB'}},
                    {size: {size: 1, unit: 'TB'}},
                    {size: {size: 1, unit: 'TB'}}
                ]
            };
            expect(j.scope.calculateRawCapacity(aggregate)).toEqual({size: 6, unit: 'TB'});
        });


        it('and even more', function () {
            var aggregate = {
                'providerVolumes': [
                    {size: {size: 100, unit: 'GB'}},
                    {size: {size: 100, unit: 'GB'}},
                    {size: {size: 100, unit: 'GB'}},
                    {size: {size: 100, unit: 'GB'}},
                    {size: {size: 100, unit: 'GB'}}
                ]
            };
            expect(j.scope.calculateRawCapacity(aggregate)).toEqual({size: 500, unit: 'GB'});
        });

        it('and even less', function () {
            var aggregate = {
                'providerVolumes': [
                    {size: {size: 0, unit: 'TB'}}
                ]
            };
            expect(j.scope.calculateRawCapacity(aggregate)).toEqual({size: 0, unit: 'Byte'});
        });

        it('should calculate 0 when there are no disks', function () {
            var aggregate = {
                'providerVolumes': []
            };
            expect(j.scope.calculateRawCapacity(aggregate)).toEqual({size: 0, unit: 'Byte'});
        });
    });

    describe('initializeController', function () {
        beforeEach(function () {
            magnifispec.mockTabsController('AggregateController', {}, {
                selectedWorkingEnvironment: {
                    _pathPrefix: '/vsa/',
                    publicId: '1',
                    instanceId: '1',
                    workingEnvironmentType: 'VSA',
                    _cloudProviderName: 'AWS'
                },
                updateSelectedWorkingEnvironment: j.$q.when,
                diskTypes: {gp2: {display: 'General Purpose SSD (gp2)', shortName: 'gp2'}}
            });
            jest.spyOn(j.nocUtil, 'mergeExistingObject').mockImplementation(() => {
            });
            magnifispec.processPromise();
        });

        describe('aggregatesPolling', () => {
            it('should get the aggregates', function () {
                expect(aggregatesResource.startAggregatesPolling).toHaveBeenCalled();
                j.scope.pollingSuccessCallback(mockedAggregatesResponses);
                expect(j.pidgeon.clear).toHaveBeenCalledWith('aggregates-poll');
                expect(j.nocUtil.mergeExistingObject).toHaveBeenCalled();
                expect(j.scope.aggregates).toEqual(mockedAggregatesResponses);
            });

            it('should fail gracefully', function () {
                j.scope.pollingErrorCallback(j.errorResponse);
                expect(j.pidgeon.error).toHaveBeenCalledWith('An error occurred while retrieving aggregates from server', j.errorResponse, 'aggregates-poll');
            });
        });

        describe('capacityFeaturesPolling', () => {
            it('should update selectedWorkingEnvironment upon success', () => {
                expect(j.workingEnvironmentResource.startCapacityFeaturesPolling).toHaveBeenCalledWith(j.scope.selectedWorkingEnvironment,
                    j.scope.capacityFeaturesPollingSuccessCallback, j.scope.capacityFeaturesPollingErrorCallback);

                j.scope.capacityFeaturesPollingSuccessCallback({
                    capacityFeatures: 'obj1',
                    _providerVolumeTypes: 'obj2'
                });
                expect(j.pidgeon.clear).toHaveBeenCalledWith('capacityFeatures-poll');
                expect(j.scope.selectedWorkingEnvironment.capacityFeatures).toEqual('obj1');
                expect(j.scope.selectedWorkingEnvironment._providerVolumeTypes).toEqual('obj2');
            });

            it('should fail gracefully', () => {
                j.scope.capacityFeaturesPollingErrorCallback(j.errorResponse);
                expect(j.pidgeon.error).toHaveBeenCalledWith('An error occurred while retrieving capacityFeatures from server', j.errorResponse, 'capacityFeatures-poll');
            });
        });
    });

    describe('disableAddAggregateTootip', () => {
        beforeEach(() => {
            j.scope.selectedWorkingEnvironment._awsEncryptionKey = null;
        });

        it('should return the correct message for disableAddingDisks', () => {
            j.scope.selectedWorkingEnvironment._awsEncryptionKey = {status: 'failed'};

            expect(j.scope.disableAddAggregateTootip()).toEqual('Cannot allocate new AWS disks for this working environment since its AWS encryption master key is not enabled.');
        });

        it('should return the correct message for reaching capacity limit', () => {
            j.scope.selectedWorkingEnvironment._providerVolumeTypes = {
                gp2: {
                    supportedSizes: [
                        {
                            size: {size: 1, unit: 'TB', _identifier: '1 TB'},
                            maxDisksAllow: {numOfDisks: 0, reason: 'reason'}
                        },
                        {
                            size: {size: 2, unit: 'TB', _identifier: '2 TB'},
                            maxDisksAllow: {numOfDisks: 0, reason: 'reason'}
                        }
                    ]
                },
                st1: {
                    supportedSizes: [
                        {
                            size: {size: 2, unit: 'TB', _identifier: '2 TB'},
                            maxDisksAllow: {numOfDisks: 0, reason: 'reason'}
                        }
                    ]
                }
            };

            expect(j.scope.disableAddAggregateTootip()).toEqual('Cannot create new aggregate, capacity limit reached');
        });

        it('should return nothing if no reason to disable', () => {
            j.scope.selectedWorkingEnvironment._providerVolumeTypes = {
                gp2: {
                    supportedSizes: [
                        {
                            size: {size: 1, unit: 'TB', _identifier: '1 TB'},
                            maxDisksAllow: {numOfDisks: 2, reason: null}
                        },
                        {
                            size: {size: 2, unit: 'TB', _identifier: '2 TB'},
                            maxDisksAllow: {numOfDisks: 0, reason: 'reason'}
                        }
                    ]
                }
            };

            expect(j.scope.disableAddAggregateTootip()).not.toBeDefined();
        });
    });

    describe('addDisksToAggregateDisableTooltip', () => {
        beforeEach(() => {
            j.scope.selectedWorkingEnvironment._awsEncryptionKey = null;
        });

        it('should return the correct message for disableAddingDisks', () => {
            j.scope.selectedWorkingEnvironment._awsEncryptionKey = {status: 'failed'};

            expect(j.scope.addDisksToAggregateDisableTooltip(j.scope.selectedWorkingEnvironment, {}))
                .toEqual('Cannot allocate new AWS disks for this working environment since its AWS encryption master key is not enabled.');
        });

        it('should return the correct message for maxDisksPerAggrLimit for single node', () => {
            const workingEnvironment2 = {
                isHA: false,
                _providerVolumeTypes: {
                    gp2: {
                        name: 'gp2',
                        supportedSizes: [{
                            size: 1, unit: 'TB', _identifier: '1 TB',
                            maxDisksAllow: {numOfDisks: 4, reason: null}
                        }]
                    }
                },
                capacityFeatures: {maxDisksPerAggregate: 6}
            };

            const aggr1 = {
                disks: ['1', '1', '1', '1', '1', '2'],
                providerVolumes: [{size: {size: 1, unit: 'TB'}, diskType: 'gp2'}]
            };
            expect(j.scope.addDisksToAggregateDisableTooltip(workingEnvironment2, aggr1))
                .toEqual('Cannot add any disks to aggregate, maximum capacity of 6 disks per aggregate reached');

            const aggr2 = {disks: ['1', '2'], providerVolumes: [{size: {size: 1, unit: 'TB'}, diskType: 'gp2'}]};
            expect(j.scope.addDisksToAggregateDisableTooltip(workingEnvironment2, aggr2)).not.toBeDefined();
        });

        it('should return the correct message for maxDisksPerAggrLimit for HA', () => {
            const workingEnvironment2 = {
                isHA: true,
                _providerVolumeTypes: {
                    gp2: {
                        name: 'gp2',
                        supportedSizes: [{
                            size: 1, unit: 'TB', _identifier: '1 TB',
                            maxDisksAllow: {numOfDisks: 4, reason: null}
                        }]
                    }
                },
                capacityFeatures: {maxDisksPerAggregate: 6}
            };

            const aggr1 = {
                disks: ['1', '1', '1', '1', '1', '2', '1', '1', '1', '1', '1', '2'],
                providerVolumes: [{size: {size: 1, unit: 'TB'}, diskType: 'gp2'}]
            };
            expect(j.scope.addDisksToAggregateDisableTooltip(workingEnvironment2, aggr1))
                .toEqual('Cannot add any disks to aggregate, maximum capacity of 6 disks per aggregate per node reached');

            const aggr2 = {
                disks: ['1', '1', '1', '1', '1', '2'], providerVolumes: [{size: {size: 1, unit: 'TB'}, diskType: 'gp2'}]
            };
            expect(j.scope.addDisksToAggregateDisableTooltip(workingEnvironment2, aggr2)).not.toBeDefined();
        });

        it('should return the correct message for server limit', () => {
            let workingEnvironment2 = {
                isHA: false,
                _providerVolumeTypes: {
                    gp2: {
                        name: 'gp2',
                        supportedSizes: [{
                            size: 1, unit: 'TB', _identifier: '1 TB',
                            maxDisksAllow: {numOfDisks: 0, reason: 'Server limit msg'}
                        }]
                    }
                },
                capacityFeatures: {maxDisksPerAggregate: 6}
            };
            const aggr = {disks: ['1', '1', '1'], providerVolumes: [{size: {size: 1, unit: 'TB'}, diskType: 'gp2'}]};

            expect(j.scope.addDisksToAggregateDisableTooltip(workingEnvironment2, aggr))
                .toEqual('Cannot add any disks to aggregate: Server limit msg');

            workingEnvironment2._providerVolumeTypes['gp2'].supportedSizes[0].maxDisksAllow = {
                numOfDisks: 3,
                reason: null
            };
            expect(j.scope.addDisksToAggregateDisableTooltip(workingEnvironment2, aggr)).not.toBeDefined();
        });
    });

    describe('showAddDisksAggregateForm', () => {
        it('should route correctly', () => {
            j.scope.checkAndSetState = jest.fn();

            j.scope.showAddDisksAggregateForm({publicId: 'id1'}, {name: 'aggr1'});

            expect(j.scope.checkAndSetState).toHaveBeenCalledWith('^.add-disks-aggregate', {
                workingEnvironmentId: 'id1',
                aggregateName: 'aggr1'
            });
        });
    });

    describe('on destroy', function () {
        it('should clean up nicely', function () {
            j.$rootScope.$broadcast('$destroy');
            expect(stopAggregatePolling).toHaveBeenCalled();
            expect(stopCapacityFeaturesPolling).toHaveBeenCalled();
        });
    });

    describe('showAggregateInfoDialog', function () {
        const aggregate = {name: 'aggr1'};
        let event = {};

        beforeEach(function () {
            j.scope.selectedWorkingEnvironment._allowedOperations = {'aggregate-info': true};
        });

        it('should do nothing if this operation is not allowed', () => {
            jest.spyOn(j.nocDialog, 'dialog').mockImplementation(() => {
            });
            j.scope.selectedWorkingEnvironment._allowedOperations['aggregate-info'] = false;
            j.scope.showAggregateInfoBox(event, aggregate);
            expect(j.nocDialog.dialog).not.toHaveBeenCalled();
        });

        it('showAggregateInfoBox should open a dialog with the right parameters', () => {
            jest.spyOn(j.scope, 'buildDataForInformationDialog').mockReturnValue(['0', '1', '2', '3', '4', '5', '6']);
            var finish = magnifispec.fakeDialog();
            event.stopPropagation = jest.fn();
            j.scope.showAggregateInfoBox(event, aggregate);
            finish();
            expect(j.nocDialog.dialog).toHaveBeenCalledWith('aggr1', 'assets/templates/dialogs/general-info-dialog.html', {
                width: 866,
                fromTop: '15%',
                lists: {leftList: ['0', '1', '2', '3'], rightList: ['4', '5', '6']}
            });
            expect(event.stopPropagation).toHaveBeenCalled();
        });
    });

    describe('buildDataForInformationDialog', () => {
        const aggr = {
            totalCapacity: {size: 987654321, unit: 'Byte'},
            usedCapacity: {size: 123456789, unit: 'Byte'},
            volumes: [
                {name: 'vol1', totalSize: {size: 5463738, unit: 'Byte'}},
                {name: 'vol2', totalSize: {size: 9384756, unit: 'Byte'}}
            ],
            providerVolumes: [
                {name: 'name1', id: 'id1', diskType: 'gp2', size: {size: 888888, unit: 'Byte'}},
                {name: 'name2', id: 'id2', diskType: 'gp2', size: {size: 777777, unit: 'Byte'}}
            ],
            disks: [
                {name: 'name1'},
                {name: 'name2'}
            ],
            state: 'myState',
            encryptionType: 'notEncrypted',
            _displayCapacity: '941.90 MB',
            _providerVolumeType: 'gp2',
            capacityTier: 'S3',
            _tier: 'gp2+S3'
        };

        it('should create the right data for AWS', () => {
            jest.spyOn(j.scope, 'buildCloudProviderCapacityData').mockReturnValue({
                key: 'Underlying AWS Capacity:',
                text: 'someData'
            });

            expect(j.scope.buildDataForInformationDialog(aggr)).toEqual([
                {
                    key: 'EBS Allocated Capacity:',
                    text: '941.90 MB'
                }, {
                    key: 'EBS Used Capacity:',
                    text: '117.74 MB'
                }, {
                    key: 'S3 Used Capacity:',
                    text: '0 KB'
                },
                {
                    key: 'Volumes:',
                    text: 2,
                    linkDataList: [{value: 'vol1 (5.21 MB)'}, {value: 'vol2 (8.95 MB)'}],
                    listShow: false,
                    trimmedText: true
                }, {
                    key: 'AWS Disks:',
                    text: 2,
                    linkDataList: [{value: 'name1', title: 'id1'}, {value: 'name2', title: 'id2'}],
                    listShow: false,
                    trimmedText: false
                }, {
                    key: 'State:',
                    text: 'myState'
                }, {
                    key: 'Underlying AWS Tier:',
                    text: 'General Purpose SSD (gp2) and S3',
                    trimmedText: true,
                    title: 'General Purpose SSD (gp2) and S3'
                }, {
                    key: 'AWS Disk Size:',
                    text: '868 KB'
                }, {
                    key: 'Underlying AWS Capacity:',
                    text: 'someData'
                }, {
                    key: 'Encryption Type:',
                    text: 'Not Encrypted'
                }
            ]);
        });

        it('should create the right data for Azure', () => {
            j.scope.selectedWorkingEnvironment._isAWS = false;
            j.scope.selectedWorkingEnvironment._cloudProviderName = 'Azure';
            jest.spyOn(j.scope, 'buildCloudProviderCapacityData').mockReturnValue({
                key: 'Underlying Azure Capacity:',
                text: 'someData'
            });

            expect(j.scope.buildDataForInformationDialog(aggr)).toEqual([
                {
                    key: 'Disk Allocated Capacity:',
                    text: '941.90 MB'
                }, {
                    key: 'Disk Used Capacity:',
                    text: '117.74 MB'
                },
                {
                    key: 'S3 Used Capacity:',
                    text: '0 KB'
                }, {
                    key: 'Volumes:',
                    text: 2,
                    linkDataList: [{value: 'vol1 (5.21 MB)'}, {value: 'vol2 (8.95 MB)'}],
                    listShow: false,
                    trimmedText: true
                }, {
                    key: 'Azure Disks:',
                    text: 2,
                    linkDataList: [{value: 'name1', title: 'id1'}, {value: 'name2', title: 'id2'}],
                    listShow: false,
                    trimmedText: true
                }, {
                    key: 'State:',
                    text: 'myState'
                }, {
                    key: 'Underlying Azure Tier:',
                    text: 'General Purpose SSD (gp2) and S3',
                    trimmedText: true,
                    title: 'General Purpose SSD (gp2) and S3'
                }, {
                    key: 'Azure Disk Size:',
                    text: '868 KB'
                }, {
                    key: 'Underlying Azure Capacity:',
                    text: 'someData'
                }, {
                    key: 'Encryption Type:',
                    text: 'Not Encrypted'
                }
            ]);
        });

        it('should display used tier capacity for tiered aggregates', () => {
            let aggr1 = _.cloneDeep(aggr);
            aggr1.capacityTier = 'S3';
            aggr1.capacityTierUsed = {size: 123, unit: 'GB'};
            expect(j.scope.buildDataForInformationDialog(aggr1)).toContainEqual({
                key: 'S3 Used Capacity:',
                text: '123 GB'
            });
        });

        it('should display "None" for no volumes', () => {
            let aggr1 = _.cloneDeep(aggr);
            aggr1.volumes = [];
            expect(j.scope.buildDataForInformationDialog(aggr1))
                .toContainEqual(jasmine.objectContaining({
                    key: 'Volumes:',
                    text: 'None',
                    linkDataList: [],
                    listShow: false,
                    trimmedText: true
                }));
        });

        it('should display the right data key for ontap encrypted', () => {
            const data = j.scope.buildDataForInformationDialog(_.assignIn(aggr, {
                encryptionType: 'ontapEncrypted',
                encryptionKeyId: 'lala'
            }));

            expect(data).toContainEqual({
                key: 'Key ID:',
                text: 'lala',
                trimmedText: true
            });
            expect(data).toContainEqual({
                key: 'Encryption Type:',
                text: 'ONTAP Managed Encryption'
            });
        });

        it('should display the right data for n/a encryption', () => {
            const data = j.scope.buildDataForInformationDialog(_.assignIn(aggr, {encryptionType: 'na'}));
            expect(data).toContainEqual({
                key: 'Key ID:',
                text: 'N/A'
            });
            expect(data).toContainEqual({
                key: 'Encryption Type:',
                text: 'N/A'
            });
        });

        it('should display the right data for AWS encryption', () => {
            const data = j.scope.buildDataForInformationDialog(_.assignIn(aggr, {encryptionType: 'awsEncrypted'}));
            expect(data).toContainEqual({
                key: 'Encryption Type:',
                text: 'AWS Managed Encryption'
            });
        });

        it('should display home node if available', () => {
            expect(j.scope.buildDataForInformationDialog(_.assignIn(aggr, {homeNode: 'ha1-01'})))
                .toContainEqual(jasmine.objectContaining({
                    key: 'Home Node:',
                    text: 'ha1-01',
                    trimmedText: true
                }));
        });

        it('should display provisioned IOPS if available', () => {
            var volumesArr = [{iops: '150'}];
            expect(j.scope.buildDataForInformationDialog(_.assignIn(aggr, {providerVolumes: volumesArr})))
                .toContainEqual(jasmine.objectContaining({
                    key: 'Provisioned IOPS:',
                    text: '150'
                }));
        });
    });
});
