describe('CloudSyncActivationS3BucketStepController', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let nocCloudSyncService;
    let getBucketsSpy;

    beforeEach(inject((_nocCloudSyncService_) => {
        nocCloudSyncService = _nocCloudSyncService_;
        getBucketsSpy = jest.spyOn(nocCloudSyncService, 'getBuckets').mockReturnValue(j.$q.when());
        magnifispec.mockWizardStepController('CloudSyncActivationS3BucketStepController', {}, {
            userId: 'user1',
            next: jest.fn(),
            type: 'VSA'
        });
    }));

    describe('initialize', () => {
        it('should get the buckets', () => {
            const bucketsResponse = ['bucket1', 'bucket2'];

            getBucketsSpy.mockReturnValue(j.$q.when(bucketsResponse));

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.buckets).toEqual(bucketsResponse);
            expect(nocCloudSyncService.getBuckets).toHaveBeenCalledWith(jasmine.any(Object), undefined);
        });
    });

    describe('locationClicked', () => {
        it('should select the location if not selected before', () => {
            j.scope.wizardController.s3Location = 'bucket1/folder1';

            j.scope.locationClicked('bucket2');

            expect(j.scope.wizardController.s3Location).toEqual('bucket2');
        });

        it('should un-select the location if selected before and current path is list of all buckets', () => {
            j.scope.currentPath = [];
            j.scope.wizardController.s3Location = 'bucket1';

            j.scope.locationClicked('bucket1');

            expect(j.scope.wizardController.s3Location).toEqual('');
        });

        it('should select the parent location when un-selecting location', () => {
            j.scope.currentPath = ['bucket1', 'folder1'];
            j.scope.wizardController.s3Location = 'bucket1/folder1/folder2';

            j.scope.locationClicked('bucket1/folder1/folder2');

            expect(j.scope.wizardController.s3Location).toEqual('bucket1/folder1');
        });
    });

    describe('getLocationSubdirectories', () => {
        beforeEach(() => {
            j.scope.wizardController.type = 'VSA';
        });
        it('should call API correctly if needed', () => {
            jest.spyOn(nocCloudSyncService, 'getBucketSubdirectories').mockReturnValue(j.$q.when());
            const dir = {dirName: 'directory1', bucketName: 'bucket1', fullPath: 'directory1'};

            j.scope.getLocationSubdirectories(dir);

            expect(nocCloudSyncService.getBucketSubdirectories).toHaveBeenCalled();
        });

        it('should not call API for location it had already called', () => {
            jest.spyOn(nocCloudSyncService, 'getBucketSubdirectories').mockReturnValue(j.$q.when(['some', 'dirs']));
            const dir = {dirName: 'directory1', bucketName: 'bucket1', fullPath: 'directory1'};

            j.scope.getLocationSubdirectories(dir);

            expect(nocCloudSyncService.getBucketSubdirectories.mock.calls.length).toEqual(1);
        });

        describe('case parent is bucket', () => {
            const bucket = {bucketName: 'bucket1'};
            beforeEach(() => {
                j.scope.wizardController.type = 'VSA';
                jest.spyOn(nocCloudSyncService, 'getBucketSubdirectories').mockReturnValue(j.$q.when([{name: 'dir1', fullPath: 'dir1'}]));
                j.scope.type = 'VSA';
                j.scope.getLocationSubdirectories(bucket);
                magnifispec.processPromise();
                j.$timeout.flush();
            });
            it('should memoize the response', () => {
                expect(bucket.subdirectories).toEqual([{name: 'dir1', fullPath: 'dir1', dirName: 'dir1'}]);
            });

            it('should select the parent bucket as S3 target location', () => {
                expect(j.scope.wizardController.s3Location).toEqual('bucket1');
            });
            it('should update the path', () => {
                expect(j.scope.currentPath).toEqual(['bucket1']);
            });
            it('should calculate dirName push the subdirectories correctly to stack', () => {
                expect(j.scope.subdirectoriesStack).toEqual([
                    [{name: 'dir1', fullPath: 'dir1', dirName: 'dir1'}],
                    [{dirName: 'dummy sub dir'}]
                ]);
            });
            it('should mark the buckets as up the stream', () => {
                expect(j.scope.bucketsUpTheStream).toBeTruthy();
            });
        });

        describe('case parent is subdirectory', () => {
            const dir = {bucketName: 'bucket1', name: 'dir1', fullPath: 'dir1', dirName: 'dir1'};
            beforeEach(() => {
                j.scope.wizardController.type = 'VSA';
                jest.spyOn(nocCloudSyncService, 'getBucketSubdirectories').mockReturnValue(j.$q.when([
                    {name: 'dir11', fullPath: 'dir1/dir11'},
                    {name: 'dir12', fullPath: 'dir1/dir12'}
                ]));
                j.scope.currentPath = ['bucket1'];
                j.scope.subdirectoriesStack = [
                    [dir], [{dirName: 'dummy sub dir'}]
                ];

                j.scope.getLocationSubdirectories(dir);
                magnifispec.processPromise();
                j.$timeout.flush();
            });
            it('should memoize the response', () => {
                expect(dir.subdirectories).toEqual([
                    {name: 'dir11', fullPath: 'dir1/dir11', dirName: 'dir11'},
                    {name: 'dir12', fullPath: 'dir1/dir12', dirName: 'dir12'}
                ]);
            });
            it('should select the parent dir as S3 target location', () => {
                expect(j.scope.wizardController.s3Location).toEqual('bucket1/dir1');
            });
            it('should update the path', () => {
                expect(j.scope.currentPath).toEqual(['bucket1', 'dir1']);
            });
            it('should calculate dirName push the subdirectories correctly to stack', () => {
                expect(j.scope.subdirectoriesStack).toEqual([
                    [
                        dir
                    ], [
                        {name: 'dir11', fullPath: 'dir1/dir11', dirName: 'dir11'},
                        {name: 'dir12', fullPath: 'dir1/dir12', dirName: 'dir12'}
                    ], [
                        {dirName: 'dummy sub dir'}
                    ]
                ]);
            });
        });
    });

    describe('goBackOneLevel', () => {
        it('should remove one level from stack (but keep the dummy)', () => {
            j.scope.subdirectoriesStack = [
                [{dirName: 'dir1'}],
                [{dirName: 'dir1-a'}, {dirName: 'dir1-b'}],
                [{dirName: 'dummy sub dir'}]
            ];

            j.scope.goBackOneLevel();
            j.$timeout.flush();

            expect(j.scope.subdirectoriesStack).toEqual([
                [{dirName: 'dir1'}],
                [{dirName: 'dummy sub dir'}]
            ]);
        });

        it('should update the path', () => {
            j.scope.currentPath = ['bucket1', 'dir1', 'dir1-a'];

            j.scope.goBackOneLevel();

            expect(j.scope.currentPath).toEqual(['bucket1', 'dir1']);
        });

        it('should mark the bucket as current location (not up the stream) after popping the last sub directories', () => {
            j.scope.bucketsUpTheStream = true;
            j.scope.subdirectoriesStack = [
                [{dirName: 'dir1'}],
                [{dirName: 'dummy sub dir'}]
            ];

            j.scope.goBackOneLevel();
            j.$timeout.flush();

            expect(j.scope.bucketsUpTheStream).toBeFalsy();
        });
    });

    describe('goBackToDir', () => {
        it('should bring back to correct dir, and update the selected location to the parent of that dir', () => {
            j.scope.currentPath = ['bucket1', 'dir1', 'dir1-a', 'dir1-aa'];
            j.scope.subdirectoriesStack = [
                [{dirName: 'dir1'}],
                [{dirName: 'dir1-a'}, {dirName: 'dir1-b'}],
                [{dirName: 'dir1-aa'}, {dirName: 'dir1-ab'}],
                [{dirName: 'dummy sub dir'}]
            ];

            j.scope.goBackToDir('dir1');
            j.$timeout.flush();

            expect(j.scope.currentPath).toEqual(['bucket1', 'dir1']);
            expect(j.scope.subdirectoriesStack).toEqual([
                [{dirName: 'dir1'}],
                [{dirName: 'dummy sub dir'}]
            ]);
            expect(j.scope.s3Location).toEqual('bucket1/dir1');
        });
    });

    describe('next', () => {
        it('should warn the user if no bucket selected', () => {
            j.scope.wizardController.selectedBucket = null;

            j.scope.next();

            expect(j.scope.userClickedNext).toBeTruthy();
        });

        it('should continue to next step if bucket selected', () => {
            j.scope.wizardController.s3Location = 'bucket1';

            j.scope.next();

            expect(j.scope.wizardController.next).toHaveBeenCalledWith('.dataBroker');
        });
    });

    describe('backing up data', () => {
        it('should backup data and clear fields upon destroy', () => {
            j.scope.currentPath = ['bucket1', 'dir1'];
            j.scope.subdirectoriesStack = [
                [{name: 'dir1', fullPath: 'dir1', dirName: 'dir1'}],
                [{dirName: 'dummy sub dir'}]
            ];
            j.scope.bucketsUpTheStream = true;

            j.scope.$broadcast('$destroy');
            expect(j.scope.wizardController.backupStepData).toHaveBeenCalledWith('s3Bucket', {
                currentPath: ['bucket1', 'dir1'],
                subdirectoriesStack: [
                    [{name: 'dir1', fullPath: 'dir1', dirName: 'dir1'}],
                    [{dirName: 'dummy sub dir'}]
                ],
                bucketsUpTheStream: true
            });

            expect(j.scope.wizardController.selectedVPC).not.toBeDefined();
            expect(j.scope.wizardController.selectedVpcId).not.toBeDefined();
            expect(j.scope.wizardController.selectedRegion).not.toBeDefined();
        });

        it('should restore backed up data upon creation ', () => {
            expect(j.scope.wizardController.getStepData).toHaveBeenCalledWith('s3Bucket');
        });
    });
});
