describe('CloudSyncActivationVolumesStepController', () => {
    'use strict';

    let j;    beforeEach(() => {        j = magnifispec.setup();    });
    let getVolumesSpy;

    beforeEach(inject((volumesResource) => {
        getVolumesSpy = jest.spyOn(volumesResource, 'getVolumes').mockReturnValue(j.$q.when([]));

        magnifispec.mockWizardStepController('CloudSyncActivationVolumesStepController', {}, {
            workingEnvironmentId: 'id1',
            type: 'VSA'
        });
        magnifispec.processPromise();
    }));

    describe('initialize', () => {
        beforeEach(inject(() => {
            jest.spyOn(j.scope, 'onSelectAll').mockImplementation(() => {});
        }));

        it('should filter out volumes which are: dp, cifs, offline, moving or has no junction path', () => {
            getVolumesSpy.mockReturnValue(j.$q.when([
                {name: 'vol1', securityStyle: 'unix', junctionPath: '/vol1', mountPoint: 'mmm'},
                {name: 'vol2', volumeType: 'dp', junctionPath: '/vol3', mountPoint: 'mmm'},
                {name: 'vol3', securityStyle: 'ntfs', junctionPath: '/vol4', mountPoint: 'mmm'},
                {name: 'vol4', junctionPath: '/vol5', mountPoint: 'mmm', state: 'offline'},
                {name: 'vol5', junctionPath: '/vol6', mountPoint: 'mmm', moving: true},
                {name: 'vol6', mountPoint: 'mmm'},
                {name: 'vol7', junctionPath: '/vol8'}
            ]));

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.volumes.length).toEqual(2);
            expect(j.scope.volumes).toEqual([
                jasmine.objectContaining({name: 'vol1', securityStyle: 'unix'}),
                jasmine.objectContaining({name: 'vol7', junctionPath: '/vol8'})
            ]);
            expect(j.scope.selectableVolumes.length).toEqual(1);
            expect(j.scope.selectableVolumes).toEqual(
                [jasmine.objectContaining({name: 'vol1', securityStyle: 'unix'})]
            );
        });

        it('should mark all volumes defaultly for VSA', () => {
            j.scope.wizardController.type = 'VSA';

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.controllerData.selectAll).toEqual(true);
        });

        it('should mark all volumes defaultly for HA', () => {
            j.scope.wizardController.type = 'HA';

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.controllerData.selectAll).toEqual(true);
        });

        it('should not mark all volumes defaultly for ON_PREM', () => {
            j.scope.wizardController.type = 'ON_PREM';

            j.scope.initialize();
            magnifispec.processPromise();

            expect(j.scope.controllerData.selectAll).toEqual(false);
        });

        afterEach(() => {
            expect(getVolumesSpy).toHaveBeenCalledWith(jasmine.any(Object), 'id1', '/vsa/');
            expect(j.scope.onSelectAll).toHaveBeenCalled();
        });
    });

    describe('onSelectAll', () => {
        beforeEach(() => {
            j.scope.selectableVolumes = [
                {name: 'vol1', selected: true},
                {name: 'vol2', selected: false},
                {name: 'vol3', selected: true}
            ];
            jest.spyOn(j.scope, 'onSelectionChange').mockImplementation(() => {});
        });

        it('should select all selectable volumes when checked', () => {
            j.scope.controllerData.selectAll = true;

            j.scope.onSelectAll();

            expect(_.every(j.scope.selectableVolumes, {selected: true})).toBeTruthy();
        });

        it('should de-select all selectable volumes when unchecked', () => {
            j.scope.controllerData.selectAll = false;

            j.scope.onSelectAll();

            expect(_.every(j.scope.selectableVolumes, {selected: false})).toBeTruthy();
        });

        afterEach(() => {
            expect(j.scope.onSelectionChange).toHaveBeenCalled();
        });
    });

    describe('onSelectionChange', () => {
        it('should update selectAll to true when all selectable volumes are selected', () => {
            j.scope.selectableVolumes = [
                {name: 'vol1', selected: true},
                {name: 'vol2', selected: false},
                {name: 'vol3', selected: true}
            ];
            j.scope.controllerData.selectAll = false;

            j.scope.selectableVolumes[1].selected = true;
            j.scope.onSelectionChange();

            expect(j.scope.controllerData.selectAll).toEqual(true);
        });

        it('should update selectAll to false when no all selectable volumes are selected', () => {
            j.scope.selectableVolumes = [
                {name: 'vol1', selected: true},
                {name: 'vol2', selected: true},
                {name: 'vol3', selected: true}
            ];
            j.scope.controllerData.selectAll = true;

            j.scope.selectableVolumes[2].selected = false;
            j.scope.onSelectionChange();

            expect(j.scope.controllerData.selectAll).toEqual(false);
        });

        it('should build list of selected volumes names', () => {
            j.scope.volumes = [
                {name: 'vol1', selected: true},
                {name: 'vol2', selected: false},
                {name: 'vol3', selected: true}
            ];

            j.scope.onSelectionChange();

            expect(j.scope.wizardController.selectedVolumes).toEqual(['vol1', 'vol3']);
        });
        it('should calculate the total used size for the selected volumes', () => {
            j.scope.volumes = [
                {name: 'vol1', selected: true, usedSize: {size: 512, unit: 'MB'}},
                {name: 'vol2', selected: false, usedSize: {size: 512, unit: 'MB'}},
                {name: 'vol3', selected: true, usedSize: {size: 1, unit: 'GB'}}
            ];

            j.scope.onSelectionChange();

            expect(j.scope.controllerData.selectedVolumesTotalUsedSize).toEqual({size: 1610612736, unit: 'Byte'}); //1.5 GB
        });
    });
});
