describe('CreateWorkingEnvironmentByolStepController', () => {
    'use strict';

    let j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    let getAvailableLicensesSpy;
    let CloudProviderAccountResource, CloudProviderAccountResourceSpy;

    beforeEach(inject((ManifestMapper, _CloudProviderAccountResource_) => {
        getAvailableLicensesSpy = jest.spyOn(ManifestMapper, 'getAvailableLicenses').mockReturnValue([{type: 'license0-paygo'}, {type: 'license1-byol'}]);
        jest.spyOn(ManifestMapper, 'getAvailableVersions').mockReturnValue(['version0', 'version1']);

        CloudProviderAccountResource = _CloudProviderAccountResource_;
        CloudProviderAccountResourceSpy = jest.spyOn(CloudProviderAccountResource, 'getAccounts').mockReturnValue(j.$q.when());
        CloudProviderAccountResourceSpy.mockReturnValue(j.$q.when());

        magnifispec.mockWizardStepController('CreateWorkingEnvironmentByolStepController', {}, {
            workingEnvironmentTypeAndProvider: {type: 'VSA', providerName: 'AWS'},
            selectedRegion: 'ab-west-1',
            next: jest.fn(),
            path: {byol: 'path.to.byol.step'},
            setWizardStep: jest.fn()
        });
    }));

    describe('initialize', () => {
        it('should get the licenses', () => {
            j.scope.initialize();

            expect(j.scope.licenses).toEqual([{type: 'license0-paygo'}, {type: 'license1-byol'}]);
            expect(getAvailableLicensesSpy).toHaveBeenCalledWith('ab-west-1', 'version0');
        });

        it('should prepare stuff for byol serial number component', () => {
            j.scope.initialize();

            expect(j.scope.uploadProblem).toEqual({});
            expect(j.scope.licenseFileName).toEqual({});
            expect(j.scope.expirationDate).toEqual({});
        });

        it('should disable payGo and select BYOL automatically if no payGo license exists', () => {
            jest.spyOn(j.scope, 'hasByolLicense').mockImplementation(() => {
            });
            getAvailableLicensesSpy.mockReturnValue([{type: 'license1-byol'}]);

            j.scope.initialize();

            expect(j.scope.paygoAvailable).toBeFalsy();
            expect(j.scope.hasByolLicense).toHaveBeenCalled();
        });

        describe('handling failed request', () => {
            it('single node case - BYOL', () => {
                jest.spyOn(j.scope, 'hasByolLicense').mockImplementation(() => {
                });

                j.scope.wizardController.failedRequestType = 'VSA';
                j.scope.wizardController.failedRequest = {vsaMetadata: {platformSerialNumber: '123123123'}};

                j.scope.initialize();

                expect(j.scope.wizardController.platformSerialNumber).toEqual('123123123');
                expect(j.scope.hasByolLicense).toHaveBeenCalled();
            });

            it('HA case - BYOL', () => {
                jest.spyOn(j.scope, 'hasByolLicense').mockImplementation(() => {
                });

                j.scope.wizardController.failedRequestType = 'HA';
                j.scope.wizardController.failedRequest = {
                    vsaMetadata: {},
                    haParams: {
                        platformSerialNumberNode1: 'platform1',
                        platformSerialNumberNode2: 'platform2'
                    }
                };

                j.scope.initialize();

                expect(j.scope.wizardController.platformSerialNumberNode1).toEqual('platform1');
                expect(j.scope.wizardController.platformSerialNumberNode2).toEqual('platform2');
                expect(j.scope.hasByolLicense).toHaveBeenCalled();
            });

            it('paygo recreate', () => {
                jest.spyOn(j.scope, 'goingPayGo').mockImplementation(() => {
                });

                j.scope.wizardController.failedRequestType = 'VSA';
                j.scope.wizardController.failedRequest = {
                    vsaMetadata: {},
                    haParams: null
                };

                j.scope.initialize();

                expect(j.scope.wizardController.platformSerialNumber).not.toBeDefined();
                expect(j.scope.wizardController.platformSerialNumberNode1).not.toBeDefined();
                expect(j.scope.wizardController.platformSerialNumberNode2).not.toBeDefined();
                expect(j.scope.goingPayGo).toHaveBeenCalled();
            });
        });
    });

    describe('getLatestOntapVersions', () => {
        it('should return the first version for this region', () => {
            expect(j.scope.getLatestOntapVersions()).toEqual('version0');
        });
    });

    describe('clearFutureStepsData', () => {
        it('should bring back to current step if scrolled up from future step', () => {
            j.scope.wizardController.currentStateName = 'future-step';

            j.scope.clearFutureStepsData();

            expect(j.scope.wizardController.setWizardStep).toHaveBeenCalledWith('path.to.byol.step');
        });

        it('should not bring back to current step if activated in current step', () => {
            j.scope.wizardController.currentStateName = 'byol';

            j.scope.clearFutureStepsData();

            expect(j.scope.wizardController.setWizardStep).not.toHaveBeenCalled();
        });

        it('should clear packages and license steps data', () => {
            j.scope.wizardController.clearStepData = jest.fn();

            j.scope.clearFutureStepsData();

            expect(j.scope.wizardController.clearStepData).toHaveBeenCalledWith('license');
            expect(j.scope.wizardController.clearStepData).toHaveBeenCalledWith('packages');
        });
    });

    describe('hasByolLicense', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'clearFutureStepsData').mockImplementation(() => {
            });
        });

        it('should clear future step data when selecting byol over no license', () => {
            j.scope.wizardController.selectedLicense = null;

            j.scope.hasByolLicense();

            expect(j.scope.clearFutureStepsData).toHaveBeenCalled();
        });

        it('should clear future step data when selecting byol over paygo license', () => {
            j.scope.wizardController.selectedLicense = {type: 'paygo-license'};

            j.scope.hasByolLicense();

            expect(j.scope.clearFutureStepsData).toHaveBeenCalled();
        });

        it('should not clear future step data when selecting byol again', () => {
            j.scope.wizardController.selectedLicense = {type: 'byol'};

            j.scope.hasByolLicense();

            expect(j.scope.clearFutureStepsData).not.toHaveBeenCalled();
        });

        it('should select byol license', () => {
            j.scope.licenses = [{type: 'paygo1'}, {type: 'byol-1'}];

            j.scope.hasByolLicense();

            expect(j.scope.selectedLicense).not.toBeDefined();
            j.$timeout.flush();

            expect(j.scope.selectedLicense).toEqual({type: 'byol-1'});
        });
    });

    describe('goingPayGo', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'clearFutureStepsData').mockImplementation(() => {
            });
        });

        it('should clear future step data when selecting byol over byol license', () => {
            j.scope.wizardController.selectedLicense = {type: 'byol-license'};

            j.scope.goingPayGo();

            expect(j.scope.clearFutureStepsData).toHaveBeenCalled();
        });

        it('should not clear future step data when selecting "No" again', () => {
            j.scope.wizardController.selectedLicense = null;

            j.scope.goingPayGo();

            expect(j.scope.clearFutureStepsData).not.toHaveBeenCalled();
        });

        it('should clear byol license selection, to allow selection in future step', () => {
            j.scope.wizardController.selectedLicense = {type: 'byol-license'};

            j.scope.goingPayGo();

            expect(j.scope.wizardController.selectedLicense).toBeNull();
        });

        it('should not change paygo license selection that was selected in future step', () => {
            j.scope.wizardController.selectedLicense = {type: 'paygo-1'};

            j.scope.goingPayGo();

            expect(j.scope.wizardController.selectedLicense).toEqual({type: 'paygo-1'});
        });


        it('should not move to next step if clicked within the context of future step', () => {
            j.scope.wizardController.currentStateName = 'future-step';

            j.scope.goingPayGo();
            j.$timeout.flush();

            expect(j.scope.wizardController.next).not.toHaveBeenCalled();
        });
    });

    describe('next', () => {
        describe('in IC', () => {
            beforeEach(() => {
                j.scope.isValid = jest.fn().mockReturnValue(true);
                j.$rootScope.runningInIC = true;
                j.scope.wizardController.selectedLicense = {};
            });

            it('should proceed to next step if BYOL license was uploaded for a single node', () => {
                jest.spyOn(JSON, 'parse').mockReturnValue({statusResp: {serialNumber: 123}});
                j.scope.wizardController.selectedLicense.name = 'Cloud Volumes ONTAP BYOL';
                j.scope.wizardController.workingEnvironmentType = 'VSA';
                j.scope.wizardController.providedLicense = 'ABC';

                j.scope.next();
                expect(j.scope.wizardController.next).toHaveBeenCalledWith('.packages');
            });

            it('should mark license field in red if no license was uploaded for a single node', () => {
                j.scope.wizardController.selectedLicense.name = 'Cloud Volumes ONTAP BYOL';
                j.scope.wizardController.workingEnvironmentType = 'VSA';
                j.scope.providedLicense = false;

                j.scope.next();

                expect(j.scope.wizardController.next).not.toHaveBeenCalled();
                expect(j.scope.uploadProblem[1]).toEqual(true);
            });

            it('should mark the missing license field in red if only one license was uploaded for HA', () => {
                j.scope.wizardController.selectedLicense.name = 'Cloud Volumes ONTAP BYOL';
                j.scope.wizardController.workingEnvironmentType = 'HA';
                j.scope.wizardController.providedLicenseNode1 = 'ABC';
                j.scope.wizardController.providedLicenseNode1 = null;

                j.scope.next();

                expect(j.scope.wizardController.next).not.toHaveBeenCalled();
                expect(j.scope.uploadProblem[2]).toEqual(true);
            });
        });

        describe('not in IC', () => {
            it('should move to next step', () => {
                j.scope.isValid = jest.fn().mockReturnValue(true);
                j.scope.next();

                expect(j.scope.wizardController.next).toHaveBeenCalledWith('.packages');
            });
        });
    });

    describe('backing up data', () => {
        it('should clear fields upon destroy', () => {
            j.scope.wizardController.selectedLicense = 'value1';
            j.scope.wizardController.platformSerialNumberNode1 = 'value2';
            j.scope.wizardController.platformSerialNumberNode2 = 'value3';
            j.scope.wizardController.providedLicense = 'value4';
            j.scope.wizardController.providedLicenseNode1 = 'value5';
            j.scope.wizardController.providedLicenseNode2 = 'value6';
            j.scope.wizardController.platformSerialNumber = 'value7';

            j.$rootScope.$broadcast('$destroy');
            expect(j.parentScope.backupStepData).toHaveBeenCalledWith('byol', {
                selectedLicense: 'value1',
                platformSerialNumberNode1: 'value2',
                platformSerialNumberNode2: 'value3',
                providedLicense: 'value4',
                providedLicenseNode1: 'value5',
                providedLicenseNode2: 'value6',
                platformSerialNumber: 'value7'
            });
            expect(j.scope.wizardController.selectedLicense).toBeNull();
            expect(j.scope.wizardController.platformSerialNumberNode1).toBeNull();
            expect(j.scope.wizardController.platformSerialNumberNode2).toBeNull();
            expect(j.scope.wizardController.providedLicense).toBeNull();
            expect(j.scope.wizardController.providedLicenseNode1).toBeNull();
            expect(j.scope.wizardController.providedLicenseNode2).toBeNull();
            expect(j.scope.wizardController.platformSerialNumber).toBeNull();
        });

        it('should restore backed up data upon creation ', () => {
            expect(j.parentScope.getStepData).toHaveBeenCalledWith('byol');
        });
    });
});
