describe('VirginCreateWorkingEnvironmentEncryptionStepController', function () {
    'use strict';

    var j;
    beforeEach(() => {
        j = magnifispec.setup();
    });
    let encryptionService;

    beforeEach(inject((nocEncryptionResourcesService) => {
        encryptionService = nocEncryptionResourcesService;
        jest.spyOn(encryptionService, 'getOccmCertificate').mockReturnValue(j.$q.when({
            publicId: 'initialized_value1',
            _expirationState: 'mocked_state'
        }));
        jest.spyOn(encryptionService, 'getKeyManagers').mockReturnValue(j.$q.when('value2'));
        jest.spyOn(encryptionService, 'getKeyManagersCaCertificatesWithOptions').mockReturnValue(j.$q.when({
            data: [{
                publicId: 'initialized_value3',
                endDate: 'someDay',
                _expirationState: 'mocked_state'
            }],
            select2options: 'keyManagersCaCertificateOptions'
        }));
        jest.spyOn(encryptionService, 'getAwsEncryptionKeys').mockReturnValue(j.$q.when(magnifispec.awsEncryptionKeysObject));
        magnifispec.mockWizardStepController('VirginCreateWorkingEnvironmentEncryptionStepController', {}, {
            createVpcSelectObjects: {region: {code: 'r1'}},
            path: {encryption: 'path.to.this.step'},
            setWizardStep: jest.fn(),
            workingEnvironmentTypeAndProvider: {type: 'VSA', providerName: 'AWS', combined: 'VSA_AWS'},
            next: jest.fn()
        });
    }));

    describe('initialize', function () {
        it('Should initialize step correctly when everything is OK', function () {
            j.scope.wizardController.workingEnvironmentTypeAndProvider = {
                type: 'VSA',
                providerName: 'AWS',
                combined: 'VSA_AWS'
            };
            magnifispec.processPromise();

            expect(j.scope.certificate).toEqual({publicId: 'initialized_value1', _expirationState: 'mocked_state'});
            expect(j.scope.wizardController.keyManagers).toEqual('value2');
            expect(j.scope.wizardController.keyManagersCaCertificates).toEqual([{
                publicId: 'initialized_value3',
                endDate: 'someDay',
                _expirationState: 'mocked_state'
            }]);
            expect(j.scope.wizardController.keyManagersCaCertificateOptions).toEqual('keyManagersCaCertificateOptions');
            expect(j.scope.wizardController.awsEncryptionKeys).toEqual(magnifispec.awsEncryptionKeysObject.awsEncryptionKeys);
            expect(j.scope.wizardController.awsEncryptionKey).toEqual(magnifispec.awsEncryptionKeysObject.awsEncryptionKey);
            expect(j.scope.wizardController.encryptionType).toEqual('AWS');
            expect(j.scope.disableEditawsEncryptionKeys).toEqual(false);
            expect(j.scope.disableEditawsEncryptionKeysTooltip).toEqual('');
            expect(j.scope.cotEncryptionDisabled).toEqual(false);
        });
    });

    describe('shouldCotEncryptionBeDisabled', function () {
        beforeEach(() => {
            j.scope.wizardController.certificate = {_expirationState: {state: 'certificate-valid-long'}};
            j.scope.wizardController.keyManagers = ['km1', 'km2'];
            j.scope.wizardController.keyManagersCaCertificates = [
                {id: 'kmCaCertificate1', _expirationState: {state: 'certificate-valid-long'}},
                {id: 'kmCaCertificate2', _expirationState: {state: 'certificate-valid-long'}}
            ];
            j.scope.wizardController.workingEnvironmentTypeAndProvider = {
                type: 'VSA',
                providerName: 'AWS',
                combined: 'VSA_AWS'
            };
        });

        it('should return false if everything OK', function () {
            expect(j.scope.shouldCotEncryptionBeDisabled()).toEqual(false);
        });

        it('should return true when no certificate exist', function () {
            j.scope.wizardController.certificate = null;
            expect(j.scope.shouldCotEncryptionBeDisabled()).toEqual(true);
        });

        it('should return true when certificate expired', function () {
            j.scope.wizardController.certificate._expirationState.state = 'certificate-expired';
            expect(j.scope.shouldCotEncryptionBeDisabled()).toEqual(true);
        });

        it('should return true when no key managers exist', function () {
            j.scope.wizardController.keyManagers = null;
            expect(j.scope.shouldCotEncryptionBeDisabled()).toEqual(true);

            j.scope.wizardController.keyManagers = [];
            expect(j.scope.shouldCotEncryptionBeDisabled()).toEqual(true);
        });

        it('should return true when no or all expired key managers\' CA certificates', function () {
            j.scope.wizardController.keyManagersCaCertificates = null;
            expect(j.scope.shouldCotEncryptionBeDisabled()).toEqual(true);

            j.scope.wizardController.keyManagersCaCertificates = [];
            expect(j.scope.shouldCotEncryptionBeDisabled()).toEqual(true);

            j.scope.wizardController.keyManagersCaCertificates = [{
                id: 'kmCaCertificate1',
                _expirationState: {state: 'certificate-expired'}
            }];
            expect(j.scope.shouldCotEncryptionBeDisabled()).toEqual(true);

            j.scope.wizardController.keyManagersCaCertificates = [
                {id: 'kmCaCertificate1', _expirationState: {state: 'certificate-expired'}},
                {id: 'kmCaCertificate2', _expirationState: {state: 'certificate-valid-long'}}
            ]; // Only one expired
            expect(j.scope.shouldCotEncryptionBeDisabled()).toEqual(false);
        });
    });

    describe('editAwsEncryptionKey', function () {
        it('Edit - should call dialog with correct locals and refresh the list upon ok', function () {
            j.scope.wizardController = {
                awsEncryptionKeys: magnifispec.awsEncryptionKeysObject.awsEncryptionKeys,
                awsEncryptionKey: magnifispec.awsEncryptionKeysObject.awsEncryptionKeys,
                disableEditawsEncryptionKeys: false,
                backupStepData: _.noop
            };
            j.scope.keyWithStatusNotEnabled = 'true';
            j.scope.disableEditawsEncryptionKeys = false;
            const finish = magnifispec.fakeDialog();
            j.scope.editAwsEncryptionKey();
            finish();

            expect(j.nocDialog.dialog).toHaveBeenCalledWith('', 'assets/templates/wizards/virgin-create-working-environment/edit-aws-encryption-keys-dialog.html', {
                wizardController: j.scope.wizardController,
                keyWithStatusNotEnabled: j.scope.keyWithStatusNotEnabled,
                width: 890,
                fromTop: '12%'
            });
        });
    });

    describe('selectEncryptionType', function () {
        it('should mark appropriate selection as selected', function () {
            j.scope.selectEncryptionType('value4');

            expect(j.scope.encryptionType).toEqual('value4');
        });

        it('should clear future steps data in case selecting ontapEncryption after selecting something else', () => {
            j.scope.wizardController.encryptionType = 'NONE';

            j.scope.selectEncryptionType('ONTAP');

            expect(j.scope.wizardController.setWizardStep).toHaveBeenCalledWith('path.to.this.step');
            expect(j.parentScope.clearStepData).toHaveBeenCalledWith('packages');
            expect(j.parentScope.clearStepData).toHaveBeenCalledWith('license');
        });

        it('should clear future steps data in case selecting something else after selecting ontapEncryption', () => {
            j.scope.wizardController.encryptionType = 'ONTAP';

            j.scope.selectEncryptionType('NONE');

            expect(j.scope.wizardController.setWizardStep).toHaveBeenCalledWith('path.to.this.step');
            expect(j.parentScope.clearStepData).toHaveBeenCalledWith('packages');
            expect(j.parentScope.clearStepData).toHaveBeenCalledWith('license');
        });

        it('should not collapse wizard if switching from non-ontap to non-ontap', () => {
            j.scope.wizardController.encryptionType = 'AWS';

            j.scope.selectEncryptionType('NONE');

            expect(j.scope.wizardController.setWizardStep).not.toHaveBeenCalled();
            expect(j.parentScope.clearStepData).not.toHaveBeenCalled();
        });

        it('should not collapse wizard if switching from ontap to ontap', () => {
            j.scope.wizardController.encryptionType = 'ONTAP';

            j.scope.selectEncryptionType('ONTAP');

            expect(j.scope.wizardController.setWizardStep).not.toHaveBeenCalled();
            expect(j.parentScope.clearStepData).not.toHaveBeenCalled();
        });
    });

    describe('backing up data', function () {
        it('should backup data and clear fields upon destroy', function () {
            j.scope.wizardController.encryptionType = 'bestEncryptionEver';
            j.$rootScope.$broadcast('$destroy');
            expect(j.parentScope.backupStepData).toHaveBeenCalledWith('create-encryption', {
                encryptionType: 'bestEncryptionEver',
                awsEncryptionKey: undefined,
                useKeysFromAnotherAccount: undefined
            });
            expect(j.scope.wizardController.encryptionType).toBeUndefined();
        });

        it('should restore backed up data upon creation ', function () {
            expect(j.parentScope.getStepData).toHaveBeenCalledWith('create-encryption');
        });

    });
});
