describe('VirginCreateWorkingEnvironmentTieringStepController', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let TieringMetadataResource, TieringMetadataResourceSpy;
    let MetadataResource, MetadataResourceSpy;

    beforeEach(inject((_TieringMetadataResource_, _MetadataResource_) => {
        jest.spyOn(j.configService, 'getConfig').mockImplementation(magnifispec.mockPromise({ebsSizeToPiopsRatio: 456}));

        TieringMetadataResource = _TieringMetadataResource_;
        TieringMetadataResourceSpy = jest.spyOn(TieringMetadataResource, 'getPossibleTiers').mockReturnValue(j.$q.when());

        MetadataResource = _MetadataResource_;
        MetadataResourceSpy = jest.spyOn(MetadataResource, 'getSupportedFeatures').mockReturnValue(j.$q.when());

        magnifispec.mockWizardStepController('VirginCreateWorkingEnvironmentTieringStepController', {}, {
            setWizardStep: jest.fn(),
            clearStepData: jest.fn(),
            selectedRegion: 'us-east-1',
            workingEnvironmentTypeAndProvider: {type: 'VSA', providerName: 'AWS', combined: 'VSA_AWS'},
            selectedLicense: {type: 'cot-standard-paygo', name: 'Cloud Volumes ONTAP Standard'}
        });
    }));

    describe('getPossibleTiers', () => {
        beforeEach(() => {
            jest.spyOn(j.scope, 'tierSelected').mockImplementation(() => {});
            jest.spyOn(TieringMetadataResource, 'getSupportedCapacityTiers').mockImplementation(() => {});
            j.scope.wizardController.selectedOntapVersion = 'ontap-version-1';
            j.scope.wizardController.encryptionType = 'ENCRYPTION';
            j.scope.wizardController.selectedInstanceType = 'x4.large';
            j.scope.wizardController.cloudProviderAccountId = 'cloudProviderAccountId';
            MetadataResourceSpy.mockReturnValue(j.$q.when());
        });

        it('should get the right tiers for AWS and remove deprecated disk types', () => {
            TieringMetadataResourceSpy.mockReturnValue(j.$q.when({gp2: 1, standard: 2}));

            j.scope.getPossibleTiers();
            magnifispec.processPromise();

            expect(TieringMetadataResource.getPossibleTiers).toHaveBeenCalledWith(jasmine.any(Object), 'VSA', 'AWS', {type: 'cot-standard-paygo', name: 'Cloud Volumes ONTAP Standard'}, 'us-east-1', 'ontap-version-1', 'ENCRYPTION', 'x4.large', 'cloudProviderAccountId');
            expect(j.scope.possibleTiers).toEqual({gp2: 1});
            expect(j.scope.tierSelected).toHaveBeenCalled();
        });

        it('should get the right tiers for Azure', () => {
            const azureResponse = {
                'standard_lrs': {name: 'standard_lrs'},
                'premium_lrs': {name: 'premium_lrs', default: true}
            };
            TieringMetadataResourceSpy.mockReturnValue(j.$q.when(azureResponse));
            j.scope.wizardController.workingEnvironmentTypeAndProvider = {
                type: 'VSA',
                providerName: 'Azure',
                combined: 'VSA_Azure'
            };
            j.scope.getPossibleTiers();
            magnifispec.processPromise();

            expect(TieringMetadataResource.getPossibleTiers).toHaveBeenCalledWith(jasmine.any(Object), 'VSA', 'Azure', {type: 'cot-standard-paygo', name: 'Cloud Volumes ONTAP Standard'}, 'us-east-1', 'ontap-version-1', 'ENCRYPTION', 'x4.large', 'cloudProviderAccountId');
            expect(j.scope.possibleTiers).toEqual(azureResponse);
            expect(j.scope.tierSelected).toHaveBeenCalledWith({name: 'premium_lrs', default: true});
        });

        describe('should restore data from failedToCreateResponse', () => {
            it('case AWS', () => {
                j.scope.wizardController.failedRequestType = true;
                j.scope.wizardController.failedRequest = {
                    ebsVolumeType: 'gp2',
                    ebsVolumeSize: {size: 2, unit: 'TB'}
                };

                TieringMetadataResourceSpy.mockReturnValue(j.$q.when({
                    gp2: {name: 'gp2', description: 'I am gp2'},
                    standard: {name: 'standard', description: 'I am standard'}
                }));

                j.scope.getPossibleTiers();
                magnifispec.processPromise();

                expect(j.scope.tierSelected).not.toHaveBeenCalled();
                expect(j.scope.wizardController.selectedTier).toEqual({name: 'gp2', description: 'I am gp2'});
                expect(j.scope.wizardController.providerVolumeSize).toEqual({size: 2, unit: 'TB', _identifier: '2 TB'});
            });

            it('case AWS with S3', () => {
                j.scope.wizardController.failedRequestType = true;
                j.scope.wizardController.failedRequest = {
                    ebsVolumeType: 'gp2',
                    ebsVolumeSize: {size: 500, unit: 'GB'},
                    capacityTier: 'S3'
                };

                TieringMetadataResourceSpy.mockReturnValue(j.$q.when({
                    'gp2': {name: 'gp2', description: 'I am gp2'},
                    'st1': {name: 'st1', description: 'I am st1'},
                    'standard': {name: 'standard', description: 'I am standard'}
                }));

                j.scope.getPossibleTiers();
                magnifispec.processPromise();

                expect(j.scope.tierSelected).not.toHaveBeenCalled();
                expect(j.scope.wizardController.selectedTier).toEqual({name: 'gp2', description: 'I am gp2'});
                expect(j.scope.wizardController.tieringEnabled).toEqual(true);
                expect(j.scope.wizardController.providerVolumeSize).toEqual({size: 500, unit: 'GB', _identifier: '500 GB'});
            });

            it('case Azure', () => {
                j.scope.wizardController.workingEnvironmentTypeAndProvider = {
                    type: 'VSA',
                    providerName: 'Azure',
                    combined: 'VSA_Azure'
                };
                j.scope.wizardController.failedRequestType = true;
                j.scope.wizardController.failedRequest = {
                    storageType: 'standard_lrs',
                    supportedSizes: {size: 2, unit: 'TB'}
                };

                TieringMetadataResourceSpy.mockReturnValue(j.$q.when({
                    'standard_lrs': {name: 'standard_lrs'},
                    'premium_lrs': {name: 'premium_lrs', default: true}
                }));

                j.scope.getPossibleTiers();
                magnifispec.processPromise();

                expect(j.scope.tierSelected).not.toHaveBeenCalled();
                expect(j.scope.wizardController.selectedTier).toEqual({name: 'standard_lrs'});
            });
        });

        describe('should initialize isAllocatingS3Bucket correctly', () => {
            it('case composite tiers are supported', () => {
                TieringMetadataResourceSpy.mockReturnValue(j.$q.when({
                    gp2: {supportedCapacityTiers: ['S3']},
                    st1: {supportedCapacityTiers: ['S3']}
                }));

                j.scope.getPossibleTiers();
                magnifispec.processPromise();

                expect(j.scope.wizardController.isAllocatingS3Bucket).toBeTruthy();
            });

            it('case composite tiers are not supported', () => {
                TieringMetadataResourceSpy.mockReturnValue(j.$q.when({
                    gp2: {supportedCapacityTiers: ['S3'], disabledMessage: 'This version does not support tiering'},
                    st1: {supportedCapacityTiers: ['S3'], disabledMessage: 'This version does not support tiering'}
                }));

                j.scope.getPossibleTiers();
                magnifispec.processPromise();

                expect(j.scope.wizardController.isAllocatingS3Bucket).toBeFalsy();
            });
        });

        it('should not override existing selectedTier', () => {
            j.scope.wizardController.selectedTier = 'already_selected';
            TieringMetadataResourceSpy.mockReturnValue(j.$q.when({gp2: 1, standard: 2}));

            j.scope.getPossibleTiers();
            magnifispec.processPromise();

            expect(j.scope.tierSelected).not.toHaveBeenCalled();
        });

        it('should pidgeon for unknown provider', () => {
            TieringMetadataResourceSpy.mockReturnValue(j.$q.reject('unknown provider'));

            j.scope.getPossibleTiers();
            magnifispec.processPromise();

            expect(j.pidgeon.error).toHaveBeenCalledWith('unknown provider', null, 'getPossibleTiers');
        });
    });

    describe('disabledTooltipForTier', () => {
        it('should return tier disabledMessage if exists', () => {
            expect(j.scope.disabledTooltipForTier({name: 'gp2', disabledMessage: 'reason1'})).toEqual('reason1');
            expect(j.scope.disabledTooltipForTier({name: 'gp2', supportedCapacityTiers: ['S3']})).not.toBeDefined();
            expect(j.scope.disabledTooltipForTier({name: 'io1'})).toEqual('S3 tiering is not available for this disk type.');
        });
    });

    describe('tierSelected', () => {
        it('should set selectedTier and update providerVolumeSize', () => {
            const tier = {
                name: 'gp2',
                description: 'I am gp2',
                supportedSizes: [{_identifier: '500 GB'}, {_identifier: '1 TB'}, {_identifier: '2 TB'}]
            };

            j.scope.tierSelected(tier);

            expect(j.scope.wizardController.selectedTier).toEqual(tier);
            expect(j.scope.wizardController.providerVolumeSize).toEqual({_identifier: '1 TB'});
            expect(j.scope.wizardController.setWizardStep).not.toHaveBeenCalled();
        });

        it('should pick 2 TB disk for sc1', () => {
            const tier = {
                name: 'sc1',
                supportedSizes: [{_identifier: '2 TB'}, {_identifier: '4 TB'}, {_identifier: '8 TB'}]
            };
            j.scope.tierSelected(tier);

            expect(j.scope.wizardController.selectedTier).toEqual(tier);
            expect(j.scope.wizardController.providerVolumeSize).toEqual({_identifier: '2 TB'});
        });
    });

    describe('tiering checkbox', () => {
        it('should clear volume step data and set step back to tiering to prevent the user from creating "No Storage Efficiency" volume on a disk type with S3', () => {
            j.scope.wizardController.path = {tiering: 'path.to.tiering.step'};
            j.scope.wizardController.tieringEnabled = true;

            j.scope.tieringCheckboxSelected();

            expect(j.scope.wizardController.setWizardStep).toHaveBeenCalledWith('path.to.tiering.step');
            expect(j.scope.wizardController.clearStepData).toHaveBeenCalledWith('volume');
            expect(j.scope.wizardController.tieringEnabled).toEqual = false;
        });

        it('should clear volume step data and set step back to tiering to allow the user to create "No Storage Efficiency" volume on tier without +S3', () => {
            j.scope.wizardController.path = {tiering: 'path.to.tiering.step'};
            j.scope.wizardController.tieringEnabled = false;

            j.scope.tieringCheckboxSelected();

            expect(j.scope.wizardController.setWizardStep).toHaveBeenCalledWith('path.to.tiering.step');
            expect(j.scope.wizardController.clearStepData).toHaveBeenCalledWith('volume');
            expect(j.scope.wizardController.tieringEnabled).toEqual = true;
        });
    });

    describe('backing up data', () => {
        it('should backup data and clear fields upon destroy', () => {
            j.scope.wizardController.selectedTier = 'value1';
            j.scope.wizardController.providerVolumeSize = 'value3';

            j.$rootScope.$broadcast('$destroy');
            expect(j.parentScope.backupStepData).toHaveBeenCalledWith('tiering', {
                selectedTier: 'value1',
                providerVolumeSize: 'value3'
            });
            expect(j.scope.wizardController.selectedTier).not.toBeDefined();
            expect(j.scope.wizardController.providerVolumeSize).not.toBeDefined();
        });

        it('should restore backed up data upon creation ', () => {
            expect(j.parentScope.getStepData).toHaveBeenCalledWith('tiering');
        });
    });
});
