describe('VirginCloudOntapRegionStepController', () => {
    'use strict';

    var j;    beforeEach(() => {        j = magnifispec.setup();    });
    let getRegionsSpy, discoveryUrl, HADiscoveryUrl;

    beforeEach(inject((MetadataResource) => {
        discoveryUrl = j.nocBaseUrl + '/vsa/discover';
        HADiscoveryUrl = j.nocBaseUrl + '/aws/ha/discovery/discover';
        getRegionsSpy = jest.spyOn(MetadataResource, 'getRegions').mockReturnValue(j.$q.when([]));
        j.$rootScope.instancePlacement = 'placement1';

        magnifispec.mockWizardStepController('VirginCloudOntapRegionStepController', {}, {
            workingEnvironmentTypeAndProvider: {type: 'VSA', providerName: 'AWS'},
            providerAccounts: {awsAccounts: [{publicId: '1'}], azureAccounts: [{publicId: '2'}]}
        });

        j.$state.go = jest.fn();
        magnifispec.processPromise();
    }));

    it('should initialize properly', () => {
        expect(j.scope.wizardController.title).toEqual('Discover an Existing Cloud Volumes ONTAP');
        expect(j.scope.instancePlacement).toEqual('placement1');
    });

    it('should call initialize again if the choice has changed and the cloudProvider has changed', () => {
        jest.spyOn(j.scope, 'initialize').mockImplementation(() => {});
        j.scope.wizardController.choice = 'RecoverAzure';
        j.scope.wizardController.workingEnvironmentTypeAndProvider.providerName = 'Azure';

        j.scope.$digest(); //Triggers scope.$watch

        expect(j.scope.initialize).toHaveBeenCalled();
    });

    describe('getRegions', () => {
        beforeEach(() => {
            j.scope.wizardController.selectedRegion = undefined;
            jest.spyOn(j.scope, 'handleRegionChange').mockImplementation(() => {});
        });

        it('should get the regions and update the scope, and call handle region change', () => {
            getRegionsSpy.mockReturnValue(j.$q.when([
                {code: 'region1', location: 'aza', name: 'gaza'},
                {code: 'region2', location: 'gazaza', name: 'gziza'}
            ]));

            j.scope.getRegions();
            magnifispec.processPromise();

            expect(j.scope.regions).toEqual([
                {code: 'region1', location: 'aza', name: 'gaza'},
                {code: 'region2', location: 'gazaza', name: 'gziza'}
            ]);
            expect(j.scope.handleRegionChange).toHaveBeenCalledWith({code: 'region1', location: 'aza', name: 'gaza'}, undefined);
        });

        it('should choose the region of the instance placement if exists', () => {
            j.$rootScope.instancePlacement = {region: 'winterfell'};
            getRegionsSpy.mockReturnValue(j.$q.when([
                {code: 'region1', location: 'aza', name: 'gaza'},
                {code: 'winterfell', location: 'gazaza', name: 'gziza'}
            ]));

            j.scope.getRegions();
            magnifispec.processPromise();

            expect(j.scope.handleRegionChange)
                .toHaveBeenCalledWith({code: 'winterfell', location: 'gazaza', name: 'gziza'}, undefined);
        });
    });

    describe('handle region change', () => {
        beforeEach(() => {
            j.scope.regions = ['region1', 'region2'];
            j.scope.wizardController.selectedRegion = 'region2';
            j.scope.discoveredWorkingEnvironments = 'something';
            j.scope.lastSelectedRegion = 'something';
            j.scope.wizardController.selectedRegion = 'something else';
            j.scope.wizardController.selectedWorkingEnvironment = 'also something';
        });

        it('should do nothing if the last region is the current selected region', () => {
            j.scope.wizardController.selectedRegion = 'west-1';
            j.scope.lastSelectedRegion = 'west-1';

            j.scope.handleRegionChange({code: 'west-1'});

            expect(j.scope.discoveredWorkingEnvironments).toEqual('something');
            expect(j.scope.wizardController.selectedWorkingEnvironment).toEqual('also something');
        });

        it('should do nothing if there are no regions yet', () => {
            j.scope.regions = undefined;

            j.scope.handleRegionChange({code: 'west-1'});

            expect(j.scope.lastSelectedRegion).toEqual('something');
        });

        it('should do nothing if there is no selected region', () => {
            j.scope.regions = ['west-1', 'east-1'];

            j.scope.handleRegionChange();

            expect(j.scope.lastSelectedRegion).toEqual('something');
        });
    });

    describe('handle region change actual flow', () => {
        beforeEach(() => {
            j.scope.lastSelectedRegion = 'differentRegion';
            j.scope.wizardController.selectedAzureSubscription = {name: 'lili'};
        });

        it('should update last selected region, and clear working environments information', () => {
            j.scope.regions = ['west-1', 'east-1'];
            j.scope.wizardController.selectedRegion = 'west-1';
            j.scope.lastSelectedRegion = 'east-1';
            j.scope.discoveredWorkingEnvironments = ['1', '2'];
            j.scope.wizardController.selectedWorkingEnvironment = '1';

            j.scope.handleRegionChange({code: 'west-1'}, true);

            expect(j.scope.discoveredWorkingEnvironments).toEqual([]);
            expect(j.scope.wizardController.selectedWorkingEnvironment).toBeNull();
        });

        it('should call get discovery and update the discoveredWorkingEnvironments list and the fetched flag', () => {
            j.$httpBackend.expectGET(discoveryUrl + '?cloudProviderAccountId=1&region=region1').respond(200, [{name: 'we1'}, {name: 'we2'}]);
            j.$httpBackend.expectGET(HADiscoveryUrl + '?cloudProviderAccountId=1&region=region1').respond(200, [{name: 'lilalo2'}]);
            j.scope.regions = ['west-1', 'east-1'];

            j.scope.handleRegionChange({code: 'region1'}, true);
            j.$httpBackend.flush();

            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.scope.discoveredWorkingEnvironments).toEqual([{
                name: 'we1',
                type: 'Cloud Volumes ONTAP',
                _availabilityZones: [],
                _subnetIds: []
            }, {name: 'we2', type: 'Cloud Volumes ONTAP', _availabilityZones: [], _subnetIds: []}, {
                name: 'lilalo2',
                type: 'HA Cloud Volumes ONTAP',
                _availabilityZones: [],
                _subnetIds: []
            }]);
            expect(j.scope.fetched).toEqual(true);
            expect(j.nocUI.unloading).toHaveBeenCalled();
        });

        it('should call get call correct for Azure', () => {
            j.scope.wizardController.workingEnvironmentTypeAndProvider.providerName = 'Azure';
            j.scope.wizardController.selectedAzureSubscription.subscriptionId = '1234';

            j.$httpBackend.expectGET(discoveryUrl + '?region=region1&providerAccount=1234').respond(200, [{name: 'we1'}, {name: 'we2'}]);
        });

        it('should update the error upon error response', () => {
            j.$httpBackend.expectGET(discoveryUrl + '?cloudProviderAccountId=1&region=region1').respond(400, j.errorResponse);
            j.$httpBackend.expectGET(HADiscoveryUrl + '?cloudProviderAccountId=1&region=region1').respond(200, [{name: 'lilalo2'}]);
            j.scope.regions = ['west-1', 'east-1'];

            j.scope.handleRegionChange({code: 'region1'});
            j.$httpBackend.flush();

            expect(j.nocUI.loading).toHaveBeenCalled();
            expect(j.pidgeon.error).toHaveBeenCalled();
            expect(j.nocUI.unloading.mock.calls.length).toEqual(2);
        });
    });

    describe('handleSelectWorkingEnvironment', () => {
        let newWorkingEnvironment;
        beforeEach(() => {
            j.scope.wizardController.selectedRegion = 'west-1';
            j.scope.wizardController.selectedWorkingEnvironment = {name: 'we1'};
            j.scope.fetched = true;
            j.scope.wizardController.svmPassword = 'pass';
            newWorkingEnvironment = {name: 'we2', recoverable: true};
            jest.spyOn(j.nocDialog, 'alert').mockImplementation(() => {});
        });

        it('should do nothing if the working environment is not recoverable', () => {
            j.scope.handleSelectWorkingEnvironment({recoverable: false});
            //Expect nothing to be called
            expect(j.nocUI.handleFormValidity).not.toHaveBeenCalled();
            expect(j.nocDialog.alert).not.toHaveBeenCalled();
            expect(j.$state.go).not.toHaveBeenCalled();
        });

        it('should update the selected working environment and update the newly selected working environment', () => {
            j.scope.isValid = jest.fn().mockReturnValue(true);
            j.scope.handleSelectWorkingEnvironment(newWorkingEnvironment);
            expect(j.scope.wizardController.selectedWorkingEnvironment).toEqual(newWorkingEnvironment);
            expect(j.scope.wizardController.svmPassword).toEqual('');
            expect(j.scope.clearAllStepData).toHaveBeenCalled();
        });

        it('should handle form validity if form is not valid', () => {
            j.scope.isValid = jest.fn().mockReturnValue(false);
            j.scope.handleSelectWorkingEnvironment(newWorkingEnvironment);
            expect(j.nocUI.handleFormValidity).toHaveBeenCalled();
            expect(j.nocDialog.alert).not.toHaveBeenCalled();
            expect(j.$state.go).not.toHaveBeenCalled();
        });

        it('should give an alert if a working environment is not selected', () => {
            j.scope.isValid = jest.fn().mockReturnValue(true);
            j.scope.handleSelectWorkingEnvironment(null);
            expect(j.nocUI.handleFormValidity).not.toHaveBeenCalled();
            expect(j.nocDialog.alert).toHaveBeenCalledWith('Cloud Volumes ONTAP must be selected in order to proceed');
            expect(j.$state.go).not.toHaveBeenCalled();
        });
    });

    describe('backing up data', () => {
        it('should backup data and clear fields upon destroy', () => {
            j.scope.wizardController.selectedWorkingEnvironment = {name: 'workingEnvName'};
            j.scope.wizardController.selectedRegion = 'region1';
            j.$rootScope.$broadcast('$destroy');
            expect(j.parentScope.backupStepData).toHaveBeenCalledWith('region', {
                selectedWorkingEnvironment: {name: 'workingEnvName'},
                selectedRegion: 'region1',
                selectedAzureSubscription: undefined,
                cloudProviderAccount: { publicId: '1' }
            });
            expect(j.scope.selectedWorkingEnvironment).toBeNull();
            expect(j.scope.selectedRegion).toEqual('');
        });

        it('should restore backed up data upon creation ', () => {
            expect(j.parentScope.getStepData).toHaveBeenCalledWith('region');
        });
    });
});
